{
    $Id: compiler.pas,v 1.1.1.1 1998/03/25 11:18:17 root Exp $
  this unit calls the compiler and restores the heap state etc.
    Copyright (c) 1996,97 by Florian Klaempfl


    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


 ****************************************************************************}
unit compiler;

  interface

    uses
       dos,
{$ifdef i386}
       sysutils,
{$endif i386}
       errors,globals,verbose,options,
       systems,tree,symtable,parser;

    { this must be called to execute the compiler
      what's to do before call:
      1) nothing (removed)

      2) install your own error handlers (if needed)
         (assign to errors.do_* your own procedures
          where * means warning, error, fatalerror and
          internalerror)

      3) install a handler which sets the switches to values given by the
         IDE

        Notes: - Verbose and very verbose work only
                 for a command line compiler


      4) setup exepath to the compiler executable

        Notes: - Don't allocate memory while in the error handler (or in
                 the status proc), because the old state of the heap
                 heap is restored after compile
               - To stop compiler execution return from an error handler or the
                 status handler with true (else the result must be false !!!!!)
               - To get the error message text for an error/warning
                 look at errors.geterrormsg

      5) call compiler.compile

        Notes: - the file filename must exist

      General notes:
             - This procedure works only for a real mode version
             - The types tcompilestatus and tcompilestatusproc are
               defined in the unit GLOBALS

    }
    procedure compile(const filename : string;_statusproc : tcompilestatusproc);

    var
       setcompilerswitches : procedure;

  implementation

{$ifdef TP}
{$ifdef DPMI}
function MemFreeBlock(Selector: Word): Integer; far;
external 'RTM' index $0006;
type  pword = ^word;
{$endif DPMI}
{$endif TP}
    var
       recoverpos : tjmprec;

    procedure default_stop;far;

      begin
         if errortext then
           close(errorfile);
         errortext:=false;
         longjmp(recoverpos,1);
      end;


    procedure compile(const filename : string;_statusproc : tcompilestatusproc);

      var
         oldfreelist,oldheapptr,oldheaporg,oldheapend : pointer;
         oldheaplist,heap,heap2 :word;
         storeheapblock,storeheaplimit : word;
         mem : longint;
         fdir : dirstr;
         fname : namestr;
         fext : extstr;

      begin
         compilestatusproc:=_statusproc;
         { install another handler for stops }
{$ifdef tp}
         do_stop:=default_stop;
{$else}
         do_stop:=@default_stop;
{$endif}
         { save heap state: }
{$ifndef DPMI}
         oldfreelist:=freelist;
         oldheapptr:=heapptr;
         oldheaporg:=heaporg;

{$ifndef PMD}
         { make a new heap }
         heaporg:=oldheapptr;
         storeheaplimit:=heaplimit;
         storeheapblock:=heapblock;
         { try to avoid global segment allocation
         that cannot be freed }
         heapblock:=$F000;
         heaplimt:=$F000;
         heapptr:=heaporg;
         freelist:=heaporg;
{$endif PMD}
{$else DPMI}
         oldheaplist:=heaplist;
         heaplist:=0;
{$endif DPMI}
         mem:=memavail;

         { init some units }

         { inits only node management }
         init_tree;

         globalsinit;
         init_symtable;

         { search config file: }
         { first current path ... }
         { this can be changed by the args }
         { def_symbol(target_info.short_name); }
         def_symbol('FPK');
         def_symbol('VER'+version_nr);
         def_symbol('VER'+version_nr+'_'+release_nr);
         def_symbol('VER'+version_nr+'_'+release_nr+'_'+patch_nr);

         { call  *** after ***  init_symtable because macro }
         { symtable must be inited...                       }
         setcompilerswitches;

         { do some definitions which are done only once }
         initparser;
         fexpand(filename);
         fsplit(filename,fdir,fname,fext);
         { we should add this directory to the list !! }
         { as last or as first ? }
         AddPathToList(unitsearchpath,fdir,true);

         { now we're ready to start the compiler }

         { mark the current position }
         if setjmp(recoverpos)=0 then
           begin
              { init names for debugger }
              inputdir:=fdir;
              inputfile:=fname;
              inputextension:=fext;
              { call the compiler }
              parser.compile(fdir,fname,fext,false);
              if errortext then
                close(errorfile);
              errortext:=false;
           end
         else
           begin
           Writeln('An error occured in FPK');
           end;

         mem:=mem-memavail;
         Writeln('Used memory : ',mem);
         Writeln('Type return to go on');
         Readln;
{$ifndef DPMI}
         { restore the heap }
         freelist:=oldfreelist;
         heapptr:=oldheapptr;
         heaporg:=oldheaporg;
{$else DPMI}
         { this only removes the heap blocks allocated for small sizes}
         { big block or seg alloc are not freed !!! }
{$ifndef PMD}
         heap:=heaplist;
         heap2:=0;
         if heap<>0 then
           while heap2<>heaplist do
           begin
              heap2:=heap;
              {next heap is at offset 10 in the segment }
              heap:=pword(ptr(heap2,10))^;
              MemFreeBlock(heap2);
              heap2:=heap;
           end;
         heaplimit:=storeheaplimit;
         heapblock:=storeheapblock;
         heaplist:=oldheaplist;
{$endif PMD}
{$endif DPMI}
      end;

  end.
{
  $Log: compiler.pas,v $
  Revision 1.1.1.1  1998/03/25 11:18:17  root
  * Restored version

  Revision 1.1  1998/03/10 02:06:05  peter
    * moved to unused/

  Revision 1.6  1998/03/02 01:48:32  peter
    * renamed target_DOS to target_GO32V1
    + new verbose system, merged old errors and verbose units into one new
      verbose.pas, so errors.pas is obsolete

  Revision 1.5  1998/02/13 10:34:56  daniel
  * Made Motorola version compilable.
  * Fixed optimizer

  Revision 1.4  1998/01/16 12:52:10  michael
  + Path treatment and file searching should now be more or less in their
    definite form:
    - Using now modified AddPathToList everywhere.
    - File Searching mechanism is uniform for all files.
    - Include path is working now !!
    All fixes by Peter Vreman. Tested with remake3 target.

  Revision 1.3  1997/12/09 13:36:53  carl
  - sysutils now specific to i386

  Revision 1.2  1997/11/28 18:14:29  pierre
   working version with several bug fixes

  Revision 1.1.1.1  1997/11/27 08:32:56  michael
  FPC Compiler CVS start


  Pre-CVS log:

  History:
       5th november 1996:
         + unit started

       2th december 1996:
         + the tcompilestatus call is implemented in the scanner unit
         + implement the init and done code for the compiler call

  What's to do:
    o store and restore heap to disk
    o verbose and very verbose
    o pm (DPMI) support
    o test with FPKPascal
}
