{
    $Id: sysutils.pas,v 1.1.1.1 1998/03/25 11:18:17 root Exp $
    Copyright (c) 1996-98 by Florian Klaempfl

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 ***************************************************************************}

{$ifndef FPC}
{$define TP}
{$endif FPC}
unit sysutils;

  interface

{$ifdef TP}
{$G+}
  uses objects,cobjects,errors;

  type
       exception = object(TObject)
{$else}
  uses
     go32, objects,cobjects,errors;

  type
       { exception = class  always derived from tobject }
       { yes... but this code fails miserably with a GPF !! (PM) }
       exception = object(TObject)
{$endif TP}
          _msg : pstring;
          public
             constructor create(const msg : string);
             destructor destroy;virtual; { must be overriden }
             function message : string;
       end;

       tjmprec = record
{$ifdef TP}
          _ax,_bx,_cx,_dx,_si,_di,_bp,_sp,_ip,flags : word;
          _cs,_ds,_es,_ss : word;
{$else}
          eax,ebx,ecx,edx,esi,edi,ebp,esp,eip,flags : longint;
          cs,ds,es,fs,gs,ss : word;
{$endif TP}
          { we should also save the FPU state, if we use this for excpections }
          { and the compiler supports FPU register variables }
       end;

       pexception_handler = ^texception_handler;

       texception_handler = record
          e : exception;
          addr : longint;
          next : pexception_handler;
          handler : tjmprec;
       end;

{$ifdef TP}
    { used by raise and try }
    function setjmp(var rec : tjmprec) : integer;far;
    procedure longjmp(const rec : tjmprec;return_value : integer);far;
{$else}
    { used by raise and try }
    function setjmp(var rec : tjmprec) : longint;
    procedure longjmp(const rec : tjmprec;return_value : longint);
{$endif TP}

    var
       exception_handler : pexception_handler;

  implementation

    var
       last_handler : texception_handler;

    function exception.message : string;

      begin
         if _msg<>nil then
           message:=_msg^
         else
           message:='';
      end;

    constructor exception.create(const msg : string);

      begin

{$ifdef TP}
        { FPKPascal has some problems with FAIL }
        if not inherited Init then
          Fail;
{$endif TP}
        _msg:=stringdup(msg);
      end;

    destructor exception.destroy;

      begin
{$ifdef TP}
        { FPKPascal has some problems with TObject isn't defined }
        inherited Done;
{$endif TP}
      end;

{$ifdef TP}
{$S-}
    function setjmp(var rec : tjmprec) : integer;

      begin
         asm
            push di
            push es
            les di,rec
            mov es:[di].tjmprec._ax,ax
            mov es:[di].tjmprec._bx,bx
            mov es:[di].tjmprec._cx,cx
            mov es:[di].tjmprec._dx,dx
            mov es:[di].tjmprec._si,si

            { load di }
            mov ax,[bp-4]

            { ... and store it }
            mov es:[di].tjmprec._di,ax

            { load es }
            mov ax,[bp-6]

            { ... and store it }
            mov es:[di].tjmprec._es,ax

            { bp ... }
            mov ax,[bp]
            mov es:[di].tjmprec._bp,ax

            { sp ... }
            mov ax,bp
            add ax,10
            mov es:[di].tjmprec._sp,ax

            { the return address }
            mov ax,[bp+2]
            mov es:[di].tjmprec._ip,ax
            mov ax,[bp+4]
            mov es:[di].tjmprec._cs,ax

            { flags ... }
            pushf
            pop word ptr es:[di].tjmprec.flags

            mov es:[di].tjmprec._ds,ds
            mov es:[di].tjmprec._ss,ss

            { restore es:di }
            pop es
            pop di

            { we come from the initial call }
            xor ax,ax
            leave
            retf 4
         end;
      end;

{$else}

    function setjmp(var rec : tjmprec) : longint;

      begin
         asm
            pushl %edi
            movl rec,%edi
            movl %eax,(%edi)
            movl %ebx,4(%edi)
            movl %ecx,8(%edi)
            movl %edx,12(%edi)
            movl %esi,16(%edi)

            { load edi }
            movl -4(%ebp),%eax

            { ... and store it }
            movl %eax,20(%edi)

            { ebp ... }
            movl (%ebp),%eax
            movl %eax,24(%edi)

            { esp ... }
            movl %esp,%eax
            addl $12,%eax
            movl %eax,28(%edi)

            { the return address }
            movl 4(%ebp),%eax
            movl %eax,32(%edi)

            { flags ... }
            pushfl
            popl 36(%edi)

            { !!!!! the segment registers, not yet needed }
            { you need them if the exception comes from
            an interrupt or a seg_move }
            movw %cs,40(%edi)
            movw %ds,42(%edi)
            movw %es,44(%edi)
            movw %fs,46(%edi)
            movw %gs,48(%edi)
            movw %ss,50(%edi)

            { restore EDI }
            pop %edi

            { we come from the initial call }
            xorl %eax,%eax

            leave
            ret $4
         end;
      end;
{$endif TP}

{$ifdef TP}
    procedure longjmp(const rec : tjmprec;return_value : integer);

      begin
         asm

            { this is the address of rec }
            lds di,rec

            { save return value }
            mov ax,return_value
            mov ds:[di].tjmprec._ax,ax

            { restore compiler shit }
            pop bp

            { restore some registers }
            mov bx,ds:[di].tjmprec._bx
            mov cx,ds:[di].tjmprec._cx
            mov dx,ds:[di].tjmprec._dx
            mov bp,ds:[di].tjmprec._bp

            { create a stack frame for the return }
            mov es,ds:[di].tjmprec._ss
            mov si,ds:[di].tjmprec._sp

            sub si,12

            { store ds }
            mov ax,ds:[di].tjmprec._ds
            mov es:[si],ax

            { store di }
            mov ax,ds:[di].tjmprec._di
            mov es:[si+2],ax

            { store si }
            mov ax,ds:[di].tjmprec._si
            mov es:[si+4],ax

            { store flags }
            mov ax,ds:[di].tjmprec.flags
            mov es:[si+6],ax

            { store ip }
            mov ax,ds:[di].tjmprec._ip
            mov es:[si+8],ax

            { store cs }
            mov ax,ds:[di].tjmprec._cs
            mov es:[si+10],ax

            { load stack }
            mov ax,es
            mov ss,ax
            mov sp,si

            { load return value }
            mov ax,ds:[di].tjmprec._ax

            { load old ES }
            mov es,ds:[di].tjmprec._es

            pop ds
            pop di
            pop si

            popf
            retf
         end;
      end;
{$else}

    procedure longjmp(const rec : tjmprec;return_value : longint);

      begin
         asm
            { restore compiler shit }
            popl %ebp
            { this is the address of rec }
            movl 4(%esp),%edi

            { save return value }
            movl 8(%esp),%eax
            movl %eax,0(%edi)

            { !!!!! load segment registers }
            movw 46(%edi),%fs
            movw 48(%edi),%gs

            { ... and some other registers }
            movl 4(%edi),%ebx
            movl 8(%edi),%ecx
            movl 12(%edi),%edx
            movl 24(%edi),%ebp

            { !!!!! movw 50(%edi),%es }
            movl 28(%edi),%esi

            { create a stack frame for the return }
            subl $16,%esi

            {
            movzwl 42(%edi),%eax
             !!!!! es
            movl %eax,(%esi)
            }

            { edi }
            movl 20(%edi),%eax
            { !!!!! es }
            movl %eax,(%esi)

            { esi }
            movl 16(%edi),%eax
            { !!!!! es }
            movl %eax,4(%esi)

            { eip }
            movl 32(%edi),%eax
            { !!!!! es }
            movl %eax,12(%esi)

            { !!!!! cs
            movl 40(%edi),%eax
            es
            movl %eax,16(%esi)
            }

            { load and store flags }
            movl 36(%edi),%eax
            { !!!!!
            es
            }
            movl %eax,8(%esi)

            { load return value }
            movl 0(%edi),%eax

            { load old ES
            !!!!! movw 44(%edi),%es
            }

            { load stack
            !!!!! movw 50(%edi),%ss }
            movl %esi,%esp

            { !!!!
            popl %ds
            }
            popl %edi
            popl %esi

            popfl
            ret
         end;
      end;

      procedure pm_int5;

        begin
           fatalerror(bound_check_error);
        end;
{$endif TP}

{$ifndef TP}
    function test_type(obj,cl : pointer) : boolean;[public,alias: 'DO_IS'];

      var
         vmt : pointer;

      begin
         if obj=nil then
           begin
              test_type:=false;
              exit;
           end;
         { get vmt of instance }
         vmt:=pointer(obj^);
         test_type:=true;
         while assigned(vmt) do
           begin
              if cl=vmt then
                exit;
              vmt:=pointer((vmt+8)^);
           end;
         test_type:=false;
      end;

    procedure test_as(obj,cl : pointer);[public,alias: 'DO_AS'];

      begin
         if obj=nil then
           exit;
         if not(test_type(obj,cl)) then
{           raise tinvalidcast.create('');}
            runerror(210);
      end;

    procedure do_raise(e : exception;addr : longint);[public,alias: 'DO_RAISE'];

      begin
         exception_handler^.e:=e;
         exception_handler^.addr:=addr;
         longjmp(exception_handler^.handler,1);
      end;

    procedure do_reraise;[public,alias: 'DO_RERAISE'];

      var
         old_exception_handler : pexception_handler;

      begin
         exception_handler^.next^.e:=exception_handler^.e;
         exception_handler^.next^.addr:=exception_handler^.addr;
         old_exception_handler:=exception_handler;
         exception_handler:=exception_handler^.next;
         dispose(old_exception_handler);
         longjmp(exception_handler^.handler,1);
      end;

    procedure default_exception_handler;

      begin
         if setjmp(last_handler.handler)<>0 then
           { this was an exception }
           begin
              writeln('FPK unhandled exception at ',last_handler.addr,': ',
                last_handler.e.message);
           {   last_handler.e.free;}
              halt(1);
           end
      end;

var storeexit : pointer;
    old_int5,new_int5 : tseginfo;

procedure sysutils_exit;

  begin
     exitproc:=storeexit;
     set_pm_interrupt(5,old_int5);
  end;
begin
   { install default handler }
   last_handler.next:=nil;
   default_exception_handler;
   new_int5.segment:=get_cs;
   new_int5.offset:=@pm_int5;
   storeexit:=exitproc;
   exitproc:=@sysutils_exit;
   get_pm_interrupt(5,old_int5);
   set_pm_interrupt(5,new_int5);
{$else not TP}
begin
{$endif not TP}
end.
{
  $Log: sysutils.pas,v $
  Revision 1.1.1.1  1998/03/25 11:18:17  root
  * Restored version

  Revision 1.1  1998/03/10 02:06:05  peter
    * moved to unused/

  Revision 1.6  1998/03/10 01:17:29  peter
    * all files have the same header
    * messages are fully implemented, EXTDEBUG uses Comment()
    + AG... files for the Assembler generation

  Revision 1.5  1998/03/04 17:34:13  michael
  + Changed ifdef FPK to ifdef FPC

  Revision 1.4  1998/03/02 01:49:35  peter
    * renamed target_DOS to target_GO32V1
    + new verbose system, merged old errors and verbose units into one new
      verbose.pas, so errors.pas is obsolete

  Revision 1.3  1998/02/13 10:35:51  daniel
  * Made Motorola version compilable.
  * Fixed optimizer

  Revision 1.2  1997/11/28 18:14:49  pierre
   working version with several bug fixes

  Revision 1.1.1.1  1997/11/27 08:33:03  michael
  FPC Compiler CVS start

}
