
(* Yacc parser template (TP Yacc V3.0), V1.2 6-17-91 AG *)

(* global definitions: *)

program convert;

(*
 History:
   25.9.1996:
      first version

   26.9.1996:
      - structs are supported
      - unsigned implemented
      - procedure variables implemented
      - void * becomes pointer
      - unions implemented
      - enumerations
      - number post- and prefixes
      - operatores unary-, << and >>
      - problem with the priority of [], (), and * fixed
      - procedures and functions

   28.9.1996:
      - formal paramters

   22-26.5.1997  made by Mark Malakanov     mark@av.krasnoyarsk.su
      - full ariphmetic and logic expressions in #define

      - #define with params changes to function (first param

        disappears by unknown reason!).

        Adds func body into implementation section.

      - real numbers

      - handling

       #ifdef ID  to {$ifdef ID}

       #ifundef ID  to {$ifundef ID}

       #else to {$else}
       #define ID to {$define ID}
       #endif to {$endif}

      -"extern" fully handled . Adds proc/func + 'external _ExternalLibrary;'to
        implementation section
       you must assign _ExternalLibrary later.

      -"const" skips in func/proc arguments.


      changes in convert.y and scan.l
      - "convert" renamed to "h2pas"
      - Inserted the checking "IsAssigned(Pointer)" everywhere access to pointers
       It preserv from Acces Violation Errors.
      - A little remade for TP Lex and Yacc 4.01 -

           changed function "input" to "get_char"
      -!!! because of peculiarity TPLY4.01 you must create unit CONVERU.PAS by
       your hand! Cut const definitions from CONVERT.PAS and paste into CONVERU.PAS


 What need

   * handle struct a {  }; in the right way
   * all predefined C types
   * misplaced comments
   * handle functions without result
*)

  uses
   {$IFDEF WIN32}
   SysUtils,
   {$else}
   strings,
   {$endif}
   scan,converu,lexlib,yacclib;

  type
     YYSTYPE = presobject;

     tblocktype = (bt_type,bt_const,bt_var,bt_func,bt_no);

  const
     INT_STR = 'longint';
     UINT_STR = 'cardinal';
     SHORT_STR = 'integer';
     USHORT_STR = 'word';
     CHAR_STR = 'char';
     UCHAR_STR = 'char';

  var
     debug : boolean;
     hp,ph : presobject;
     block_type : tblocktype;
     extfile: text; {file for implementation headers extern procs}
     IsExtern:boolean;

{ $ define yydebug}

    procedure write_type_specifier(p : presobject; var outfile:text);forward;
    procedure write_p_a_def(p,simple_type : presobject; var outfile:text);forward;

    procedure yymsg(const msg : string);
      begin
         writeln('line ',line_no,': ',msg);
      end;

    procedure write_expr(p : presobject; var outfile:text);
      begin
      if assigned(p) then
         case p^.typ of
            t_id : write(outfile,p^.p);
            t_preop : begin
                         write(outfile,p^.p,'');
                         write_expr(p^.p1,outfile);
                         write(outfile,'');
                      flush(outfile);
                      end;
            t_bop : begin
                       write(outfile,'(');
                       write_expr(p^.p1,outfile);
                       write(outfile,')',p^.p,'(');
                       write_expr(p^.p2,outfile);
                       write(outfile,')');
                    flush(outfile);
                    end;
            else internalerror(2);
         end;
      end;

    (* writes an argument list, where p is t_arglist *)
    procedure write_args(p : presobject;  var outfile:text);
      var   para : longint;
      begin
         para:=1;
         write(outfile,'(');
         aktspace:=aktspace+'  ';

         (* walk through all arguments *)
         while assigned(p) do
           begin

              (* write new type name *)
              if assigned(p^.p1^.p2^.p2) then
                write(outfile,p^.p1^.p2^.p2^.p)
              else
                write(outfile,'_para',para);
              write(outfile,':');
              write_p_a_def(p^.p1^.p2^.p1,p^.p1^.p1,  outfile);

              (* p2 is next, this is a little bit inconsistent *)
              p:=p^.p2;
              if assigned(p) then
                write(outfile,'; ');
              inc(para);
           end;
         write(outfile,')');
         flush(outfile);
         dec(byte(aktspace[0]),2);
      end;

    procedure write_p_a_def(p,simple_type : presobject;  var outfile:text);
      begin
         if not(assigned(p)) then
           begin
              write_type_specifier(simple_type,  outfile);
              exit;
           end;
         case p^.typ of
            t_pointerdef : begin
                              (* procedure variable ? *)
                              if assigned(p^.p1) and (p^.p1^.typ=t_procdef) then
                                begin
                                   (* distinguish between procedure and function *)
                                   if (simple_type^.typ=t_void) and (p^.p1^.p1=nil) then
                                     begin
                                        write(outfile,'procedure');

                                        (* write arguments *)
                                        if assigned(p^.p1^.p2) then
                                          write_args(p^.p1^.p2,  outfile);
                                     flush(outfile);
                                     end
                                   else
                                     begin
                                        write(outfile,'function');
                                        (* write arguments *)
                                        if assigned(p^.p1^.p2) then
                                          write_args(p^.p1^.p2,  outfile);
                                        write(outfile,':');
                                        flush(outfile);
                                        write_p_a_def(p^.p1^.p1,simple_type,  outfile);
                                     end
                                end
                              else
                                begin
                                   (* generate "pointer" ? *)
                                   if (simple_type^.typ=t_void) and (p^.p1=nil) then
                                      begin
                                       write(outfile,'pointer');
                                       flush(outfile);
                                      end
                                   else
                                     begin
                                        write(outfile,'^');
                                        flush(outfile);
                                        write_p_a_def(p^.p1,simple_type,  outfile);
                                     end;
                                end;
                           end;
            t_arraydef : begin
                             write(outfile,'array[0..(');
                             write_expr(p^.p2,  outfile);
                             write(outfile,')-1] of ');
                             flush(outfile);
                             write_p_a_def(p^.p1,simple_type,  outfile);
                          end;
            else internalerror(1);
         end;
      end;

    procedure write_type_specifier(p : presobject;  var outfile:text);
      var
         hp1,hp2,hp3 : presobject;
         l,w : longint;

      begin
         case p^.typ of
            t_id : write(outfile,p^.p);
            t_enumdef : begin
                           write(outfile,'(');
                           hp1:=p^.p1;   w:=length(aktspace);
                           while assigned(hp1) do
                             begin
                                write(outfile,hp1^.p1^.p);
                                w:=w+strlen(hp1^.p1^.p);
                                hp1:=hp1^.p2;
                                if assigned(hp1) then
                                  write(outfile,',');
                                if w>40 then begin

                                  writeln(outfile);
                                  write(outfile,aktspace);
                                  end;
                                flush(outfile);
                             end;
                           write(outfile,')');
                           flush(outfile);
                        end;
            t_structdef : begin
                          writeln(outfile,'record');
                          aktspace:=aktspace+'   ';
                          hp1:=p^.p1;

                          { walk through all members }
                          while assigned(hp1) do
                            begin
                               { hp2 is t_memberdec }
                               hp2:=hp1^.p1;
                               { hp3 is t_declist }
                               hp3:=hp2^.p2;
                               while assigned(hp3) do
                                 begin
                                    write(outfile,aktspace,hp3^.p1^.p2^.p);
                                    write(outfile,' : ');
                                    aktspace:=aktspace+'  ';
                                    write_p_a_def(hp3^.p1^.p1,hp2^.p1, outfile);
                                    dec(byte(aktspace[0]),2);
                                    writeln(outfile,';');
                                    hp3:=hp3^.p2;
                                 end;
                               hp1:=hp1^.p2;
                            end;
                          dec(byte(aktspace[0]),3);
                          write(outfile,aktspace,'end');
                       flush(outfile);
                       end;
            t_uniondef : begin
                            writeln(outfile,'record');
                            writeln(outfile,aktspace,'  case longint of');
                            aktspace:=aktspace+'     ';
                            l:=0;
                            hp1:=p^.p1;

                            { walk through all members }
                            while assigned(hp1) do
                              begin
                                 { hp2 is t_memberdec }
                                 hp2:=hp1^.p1;
                                 { hp3 is t_declist }
                                 hp3:=hp2^.p2;
                                 while assigned(hp3) do
                                   begin
                                      write(outfile,aktspace,l,' : ( ');
                                      write(outfile,hp3^.p1^.p2^.p);
                                      write(outfile,':');
                                      aktspace:=aktspace+'  ';
                                      write_p_a_def(hp3^.p1^.p1,hp2^.p1, outfile);
                                      dec(byte(aktspace[0]),2);
                                      writeln(outfile,' );');
                                      hp3:=hp3^.p2;
                                      inc(l);
                                   end;
                                 hp1:=hp1^.p2;
                              end;
                            dec(byte(aktspace[0]),5);
                            write(outfile,aktspace,'end');
                           flush(outfile);
                      end;
         else internalerror(3);
         end;
      end;

    procedure write_def_params(p : presobject;  var outfile:text);
      var
         hp1 : presobject;
      begin
         case p^.typ of
            t_enumdef : begin
                           hp1:=p^.p1;
                           while assigned(hp1) do
                             begin
                                write(outfile,hp1^.p1^.p);
                                hp1:=hp1^.p2;
                                if assigned(hp1) then
                                  write(outfile,',')
                                else
                                  write(outfile);
                                flush(outfile);
                             end;
                           flush(outfile);
                        end;
         else internalerror(3);
         end;
      end;



{Set up as you need!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!}
const __ExtrnlLibrary='system.dll';

const EXTERN = 257;
const ID = 258;
const TYPEDEF = 259;
const INT = 260;
const SEMICOLON = 261;
const STRUCT = 262;
const LGKLAMMER = 263;
const RGKLAMMER = 264;
const LECKKLAMMER = 265;
const RECKKLAMMER = 266;
const LKLAMMER = 267;
const RKLAMMER = 268;
const _CHAR = 269;
const DEFINE = 270;
const NUMBER = 271;
const SHORT = 272;
const UNSIGNED = 273;
const LONG = 274;
const VOID = 275;
const UNION = 276;
const CSTRING = 277;
const ENUM = 278;
const _CONST = 279;
const _SHR = 280;
const _SHL = 281;
const _OR = 282;
const _PLUS = 283;
const MINUS = 284;
const COMMA = 285;
const STAR = 286;
const _SLASH = 287;
const _AND = 288;
const _NOT = 289;

var yylval : YYSType;

function yylex : Integer; forward;

function yyparse : Integer;

var yystate, yysp, yyn : Integer;
    yys : array [1..yymaxdepth] of Integer;
    yyv : array [1..yymaxdepth] of YYSType;
    yyval : YYSType;

procedure yyaction ( yyruleno : Integer );
  (* local definitions: *)
begin
  (* actions: *)
  case yyruleno of
   1 : begin
         yyval := yyv[yysp-0];
       end;
   2 : begin
         yyval := yyv[yysp-1];
       end;
   3 : begin
         yyval := yyv[yysp-0];
       end;
   4 : begin
         yyval:=new(presobject,init_id('extern'));

       end;
   5 : begin
         yyval:=new(presobject,init_id('intern'));

       end;
   6 : begin
         IsExtern:=false;


         (*  writeln(outfile,'{*dec_specifier type_specifier declarator_list SEMICOLON}');


         if assigned(yyv[yysp-1]) then writeln(outfile,'{*$3}');
         if assigned(yyv[yysp-1])and assigned(yyv[yysp-1].p1)
         then writeln(outfile,'{*$3.p1}');
         if assigned(yyv[yysp-1])and assigned(yyv[yysp-1].p1)and assigned(yyv[yysp-1].p1^.p1)
         then writeln(outfile,'{*$3^.p1^.p1}');
         *)


         if ( assigned(yyv[yysp-1]) and

              assigned(yyv[yysp-1]^.p1) and

              assigned(yyv[yysp-1]^.p1^.p1) ) and

              (yyv[yysp-1]^.p1^.p1^.typ=t_procdef) then
         begin
         IsExtern:=assigned(yyv[yysp-3]) and (strpas(yyv[yysp-3]^.p)='extern');

         if block_type<>bt_func then
         writeln(outfile);


         block_type:=bt_func;
         write(outfile,aktspace);
         aktspace:=aktspace+'  ';
         if IsExtern then
         write(extfile,aktspace);


         (* distinguish between procedure and function *)
         if assigned(yyv[yysp-2]) then
         if (yyv[yysp-2]^.typ=t_void) and (yyv[yysp-1]^.p1^.p1^.p1=nil) then
         begin
         write(outfile,'procedure ',yyv[yysp-1]^.p1^.p2^.p);
         (* write arguments *)
         if assigned(yyv[yysp-1]^.p1^.p1^.p2) then
         write_args(yyv[yysp-1]^.p1^.p1^.p2, outfile);
         if IsExtern then
         begin
         write(extfile,'procedure ',yyv[yysp-1]^.p1^.p2^.p);
         (* write arguments *)
         if assigned(yyv[yysp-1]^.p1^.p1^.p2) then
         write_args(yyv[yysp-1]^.p1^.p1^.p2,extfile);
         end
         end
         else
         begin
         write(outfile,'function ',yyv[yysp-1]^.p1^.p2^.p);
         if IsExtern then
         write(extfile,'function ',yyv[yysp-1]^.p1^.p2^.p);


         (* write arguments *)
         if assigned(yyv[yysp-1]^.p1^.p1^.p2) then
         write_args(yyv[yysp-1]^.p1^.p1^.p2,outfile);
         if IsExtern then
         if assigned(yyv[yysp-1]^.p1^.p1^.p2) then
         write_args(yyv[yysp-1]^.p1^.p1^.p2, extfile);


         write(outfile,':');
         if IsExtern then
         write(extfile,':');
         write_p_a_def(yyv[yysp-1]^.p1^.p1^.p1,yyv[yysp-2],outfile);
         if IsExtern then
         write_p_a_def(yyv[yysp-1]^.p1^.p1^.p1,yyv[yysp-2],extfile);
         end;
         if IsExtern then

         writeln(outfile,';CDECL;')
         else writeln(outfile,';');


         if IsExtern then writeln(extfile,';CDECL; external __ExtrnlLibrary;');
         dec(byte(aktspace[0]),2);
         IsExtern:=false;
         end
         else (* yyv[yysp-1]^.p1^.p1^.typ=t_procdef *)
         if assigned(yyv[yysp-1]) and assigned(yyv[yysp-1]^.p1) then
         begin
         if block_type<>bt_var then
         begin
         writeln(outfile);
         writeln(outfile,aktspace,'var');
         end;
         block_type:=bt_var;


         aktspace:=aktspace+'   ';


         (* walk through all declarations *)
         hp:=yyv[yysp-1];
         while assigned(hp) do
         begin
         (* write new type name *)
         if assigned(hp^.p1^.p2)and assigned(hp^.p1^.p2^.p)then
         write(outfile,aktspace,hp^.p1^.p2^.p);
         write(outfile,' : ');
         aktspace:=aktspace+'  ';
         write_p_a_def(hp^.p1^.p1,yyv[yysp-2],outfile);
         dec(byte(aktspace[0]),2);
         writeln(outfile,';');
         hp:=hp^.p2;
         end;
         dec(byte(aktspace[0]),3);
         end;
         if assigned(yyv[yysp-3])then  dispose(yyv[yysp-3],done);
         if assigned(yyv[yysp-2])then  dispose(yyv[yysp-2],done);
         if assigned(yyv[yysp-1])then  dispose(yyv[yysp-1],done);


       end;
   7 : begin


         if block_type<>bt_type then
         begin
         writeln(outfile);
         writeln(outfile,aktspace,'type');
         end;
         block_type:=bt_type;


         aktspace:=aktspace+'   ';


         (* walk through all declarations *)
         hp:=yyv[yysp-1];
         while assigned(hp) do
         begin
         (* write new type name *)
         write(outfile,aktspace,hp^.p1^.p2^.p);
         write(outfile,' = ');
         aktspace:=aktspace+'  ';
         write_p_a_def(hp^.p1^.p1,yyv[yysp-2],outfile);
         dec(byte(aktspace[0]),2);
         writeln(outfile,';');
         flush(outfile);
         hp:=hp^.p2;
         end;
         dec(byte(aktspace[0]),3);
         if assigned(yyv[yysp-2])then
         dispose(yyv[yysp-2],done);
         if assigned(yyv[yysp-1])then
         dispose(yyv[yysp-1],done);


       end;
   8 : begin


         writeln (outfile,'{it was #define name(param) expression}   ');
         writeln (extfile,'{it was #define name(param) expression}   ');
         aktspace:=aktspace+'   ';
         if block_type<>bt_func then
         writeln(outfile);
         block_type:=bt_func;
         write(outfile,aktspace,'function ',yyv[yysp-4]^.p,'(');
         write(extfile,aktspace,'function ',yyv[yysp-4]^.p,'(');


         ph:=new(presobject,init_one(t_enumdef,yyv[yysp-2]));

         write_def_params(ph,outfile);
         write_def_params(ph,extfile);
         if assigned(ph) then dispose(ph,Done);


         writeln(outfile,':integer):integer;');
         writeln(extfile,':integer):integer;');

         write(extfile,aktspace,' begin ',yyv[yysp-4]^.p,':=(');
         flush(outfile);
         write_expr(yyv[yysp-0],extfile);
         writeln(extfile,') end;');
         dec(byte(aktspace[0]),3);
         if assigned(yyv[yysp-4])then dispose(yyv[yysp-4],done);
         if assigned(yyv[yysp-2])then dispose(yyv[yysp-2],done);
         if assigned(yyv[yysp-0])then dispose(yyv[yysp-0],done);


       end;
   9 : begin


         writeln (outfile,'{it was #define name(param) expression}   ');
         writeln (extfile,'{it was #define name(param) expression}   ');
         aktspace:=aktspace+'   ';
         if block_type<>bt_func then
         writeln(outfile);
         block_type:=bt_func;
         write(outfile,aktspace,'function ',yyv[yysp-4]^.p,'(');
         write(extfile,aktspace,'function ',yyv[yysp-4]^.p,'(');


         writeln(outfile,yyv[yysp-2]^.p,':integer):integer;');
         writeln(extfile,yyv[yysp-2]^.p,':integer):integer;');

         write(extfile,aktspace,' begin ',yyv[yysp-4]^.p,':=(');
         flush(outfile);
         write_expr(yyv[yysp-0],extfile);
         writeln(extfile,') end;');
         dec(byte(aktspace[0]),3);
         if assigned(yyv[yysp-4])then dispose(yyv[yysp-4],done);
         if assigned(yyv[yysp-2])then dispose(yyv[yysp-2],done);
         if assigned(yyv[yysp-0])then dispose(yyv[yysp-0],done);


       end;
  10 : begin


         if block_type<>bt_const then
         begin
         writeln(outfile);
         writeln(outfile,aktspace,'const');
         end;
         block_type:=bt_const;
         aktspace:=aktspace+'   ';
         write(outfile,aktspace,yyv[yysp-3]^.p);
         write(outfile,' = ',yyv[yysp-1]^.p);
         flush(outfile);
         writeln(outfile,';');
         dec(byte(aktspace[0]),3);
         if assigned(yyv[yysp-3])then
         dispose(yyv[yysp-3],done);
         if assigned(yyv[yysp-2])then
         dispose(yyv[yysp-1],done);


       end;
  11 : begin


         if block_type<>bt_const then
         begin
         writeln(outfile);
         writeln(outfile,aktspace,'const');
         end;
         block_type:=bt_const;


         aktspace:=aktspace+'   ';
         write(outfile,aktspace,yyv[yysp-1]^.p);
         write(outfile,' = ');
         flush(outfile);
         write_expr(yyv[yysp-0],outfile);
         writeln(outfile,';');
         dec(byte(aktspace[0]),3);
         if assigned(yyv[yysp-1])then
         dispose(yyv[yysp-1],done);
         if assigned(yyv[yysp-0])then
         dispose(yyv[yysp-0],done);


       end;
  12 : begin


         writeln(outfile,'{$define ',yyv[yysp-0]^.p,'}');
         flush(outfile);
         if assigned(yyv[yysp-0])then
         dispose(yyv[yysp-0],done);


       end;
  13 : begin


         writeln(line_no,' warning: struct xxx ignored');
         if assigned(yyv[yysp-3])then dispose(yyv[yysp-3],done);
         yyval:=new(presobject,init_one(t_structdef,yyv[yysp-1]));


       end;
  14 : begin


         yyval:=new(presobject,init_one(t_structdef,yyv[yysp-1]));


       end;
  15 : begin


         yyval:=new(presobject,init_one(t_uniondef,yyv[yysp-1]));


       end;
  16 : begin
         yyval:=yyv[yysp-0];

       end;
  17 : begin


         yyval:=new(presobject,init_one(t_enumdef,yyv[yysp-1]));


       end;
  18 : begin
         yyval:=yyv[yysp-0];

       end;
  19 : begin


         yyval:=yyv[yysp-1];
         hp:=yyv[yysp-1];
         while assigned(hp^.p2) do
         hp:=hp^.p2;
         hp^.p2:=new(presobject,init_two(t_memberdeclist,yyv[yysp-0],nil));


       end;
  20 : begin


         yyval:=new(presobject,init_two(t_memberdeclist,yyv[yysp-0],nil));


       end;
  21 : begin


         yyval:=new(presobject,init_two(t_memberdec,yyv[yysp-2],yyv[yysp-1]));


       end;
  22 : begin


         yyval:=new(presobject,init_id(INT_STR));


       end;
  23 : begin


         yyval:=new(presobject,init_id(UINT_STR));


       end;
  24 : begin


         yyval:=new(presobject,init_id(INT_STR));


       end;
  25 : begin


         yyval:=new(presobject,init_id(UINT_STR));


       end;
  26 : begin


         yyval:=new(presobject,init_id(UINT_STR));


       end;
  27 : begin


         yyval:=new(presobject,init_id(USHORT_STR));


       end;
  28 : begin


         yyval:=new(presobject,init_id(UCHAR_STR));


       end;
  29 : begin


         yyval:=new(presobject,init_no(t_void));


       end;
  30 : begin


         yyval:=new(presobject,init_id(SHORT_STR));


       end;
  31 : begin


         yyval:=new(presobject,init_id(CHAR_STR));


       end;
  32 : begin


         yyval:=new(presobject,init_id(act_token));


       end;
  33 : begin


         yyval:=yyv[yysp-2];
         hp:=yyv[yysp-2];
         while assigned(hp^.p2) do
         hp:=hp^.p2;
         hp^.p2:=new(presobject,init_two(t_declist,yyv[yysp-0],nil));


       end;
  34 : begin
         yyval:=new(presobject,init_two(t_declist,yyv[yysp-0],nil));

       end;
  35 : begin
         yyval := yyv[yysp-0];
       end;
  36 : begin
       end;
  37 : begin


         yyval:=new(presobject,init_two(t_arg,yyv[yysp-1],yyv[yysp-0]));


       end;
  38 : begin


         yyval:=new(presobject,init_two(t_arg,yyv[yysp-1],yyv[yysp-0]));


       end;
  39 : begin


         yyval:=new(presobject,init_two(t_arglist,yyv[yysp-0],nil));


       end;
  40 : begin


         yyval:=yyv[yysp-2];
         hp:=yyv[yysp-2];
         while assigned(hp^.p2) do
         hp:=hp^.p2;
         hp^.p2:=new(presobject,init_two(t_arglist,yyv[yysp-0],nil));


       end;
  41 : begin


         yyval:=new(presobject,init_two(t_dec,nil,yyv[yysp-0]));


       end;
  42 : begin


         hp:=yyv[yysp-0];
         yyval:=hp;
         while assigned(hp^.p1) do
         hp:=hp^.p1;
         hp^.p1:=new(presobject,init_one(t_pointerdef,nil));


       end;
  43 : begin


         hp:=yyv[yysp-3];
         yyval:=hp;
         while assigned(hp^.p1) do
         hp:=hp^.p1;
         hp^.p1:=new(presobject,init_two(t_procdef,nil,yyv[yysp-1]));


       end;
  44 : begin


         (* the double declaration makes parsing easier *)


         hp:=yyv[yysp-2];
         yyval:=hp;
         while assigned(hp^.p1) do
         hp:=hp^.p1;
         hp^.p1:=new(presobject,init_two(t_procdef,nil,nil));


       end;
  45 : begin


         (* the double declaration makes parsing easier *)


         hp:=yyv[yysp-3];
         yyval:=hp;
         while assigned(hp^.p1) do
         hp:=hp^.p1;
         hp^.p1:=new(presobject,init_two(t_procdef,nil,nil));


       end;
  46 : begin


         hp:=yyv[yysp-3];
         yyval:=hp;
         while assigned(hp^.p1) do
         hp:=hp^.p1;
         hp^.p1:=new(presobject,init_two(t_arraydef,nil,yyv[yysp-1]));


       end;
  47 : begin
         yyval:=yyv[yysp-1];

       end;
  48 : begin


         hp:=yyv[yysp-0];
         yyval:=hp;
         while assigned(hp^.p1) do
         hp:=hp^.p1;
         hp^.p1:=new(presobject,init_one(t_pointerdef,nil));


       end;
  49 : begin


         hp:=yyv[yysp-3];
         yyval:=hp;
         while assigned(hp^.p1) do
         hp:=hp^.p1;
         hp^.p1:=new(presobject,init_two(t_procdef,nil,yyv[yysp-1]));


       end;
  50 : begin


         (* the double declaration makes parsing easier *)


         hp:=yyv[yysp-2];
         yyval:=hp;
         while assigned(hp^.p1) do
         hp:=hp^.p1;
         hp^.p1:=new(presobject,init_two(t_procdef,nil,nil));


       end;
  51 : begin


         (* the double declaration makes parsing easier *)


         hp:=yyv[yysp-3];
         yyval:=hp;
         while assigned(hp^.p1) do
         hp:=hp^.p1;
         hp^.p1:=new(presobject,init_two(t_procdef,nil,nil));


       end;
  52 : begin


         hp:=yyv[yysp-3];
         yyval:=hp;
         while assigned(hp^.p1) do
         hp:=hp^.p1;
         hp^.p1:=new(presobject,init_two(t_arraydef,nil,yyv[yysp-1]));


       end;
  53 : begin
         yyval:=yyv[yysp-1];

       end;
  54 : begin


         yyval:=new(presobject,init_two(t_dec,nil,nil));


       end;
  55 : begin
         yyval := yyv[yysp-0];
       end;
  56 : begin
         (*dname*)
         yyval:=new(presobject,init_id(act_token));


       end;
  57 : begin
         yyval:=yyv[yysp-0];
       end;
  58 : begin
         yyval:=new(presobject,init_bop(' + ',yyv[yysp-2],yyv[yysp-0]));
       end;
  59 : begin
         yyval:=new(presobject,init_bop(' - ',yyv[yysp-2],yyv[yysp-0]));
       end;
  60 : begin
         yyval:=new(presobject,init_bop(' * ',yyv[yysp-2],yyv[yysp-0]));
       end;
  61 : begin
         yyval:=new(presobject,init_bop(' / ',yyv[yysp-2],yyv[yysp-0]));
       end;
  62 : begin
         yyval:=new(presobject,init_bop(' or ',yyv[yysp-2],yyv[yysp-0]));
       end;
  63 : begin
         yyval:=new(presobject,init_bop(' and ',yyv[yysp-2],yyv[yysp-0]));
       end;
  64 : begin
         yyval:=new(presobject,init_bop(' not ',yyv[yysp-2],yyv[yysp-0]));
       end;
  65 : begin
         yyval:=new(presobject,init_bop(' shl ',yyv[yysp-2],yyv[yysp-0]));
       end;
  66 : begin
         yyval:=new(presobject,init_bop(' shr ',yyv[yysp-2],yyv[yysp-0]));
       end;
  67 : begin
         yyval := yyv[yysp-0];
       end;
  68 : begin
         yyval:=new(presobject,init_id(act_token));

       end;
  69 : begin
         yyval := yyv[yysp-0];
       end;
  70 : begin
         yyval:=yyv[yysp-1];
       end;
  71 : begin
         yyval:=new(presobject,init_id(''''+copy(act_token,2,length(act_token)-2)+''''));

       end;
  72 : begin
         yyval:=new(presobject,init_id(act_token));

       end;
  73 : begin
         yyval:=new(presobject,init_preop('-',yyv[yysp-0]));

       end;
  74 : begin
         yyval:=new(presobject,init_preop(' not ',yyv[yysp-0]));

       end;
  75 : begin
         (*enum_list COMMA dname*)
         yyval:=yyv[yysp-2];
         hp:=yyv[yysp-2];
         while assigned(hp^.p2) do
         hp:=hp^.p2;
         hp^.p2:=new(presobject,init_two(t_enumlist,yyv[yysp-0],nil));


       end;
  76 : begin
         begin (*enum_list: dname*)
         yyval:=new(presobject,init_two(t_enumlist,yyv[yysp-0],nil));

         end;
       end;
  77 : begin
         yyval:=yyv[yysp-1]
       end;
  end;
end(*yyaction*);

(* parse table: *)

type YYARec = record
                sym, act : Integer;
              end;
     YYRRec = record
                len, sym : Integer;
              end;

const

yynacts   = 442;
yyngotos  = 123;
yynstates = 134;
yynrules  = 77;

yya : array [1..yynacts] of YYARec = (
{ 0: }
  ( sym: 257; act: 5 ),
  ( sym: 259; act: 6 ),
  ( sym: 270; act: 7 ),
  ( sym: 258; act: -5 ),
  ( sym: 260; act: -5 ),
  ( sym: 262; act: -5 ),
  ( sym: 269; act: -5 ),
  ( sym: 272; act: -5 ),
  ( sym: 273; act: -5 ),
  ( sym: 274; act: -5 ),
  ( sym: 275; act: -5 ),
  ( sym: 276; act: -5 ),
  ( sym: 278; act: -5 ),
{ 1: }
  ( sym: 258; act: 10 ),
  ( sym: 260; act: 11 ),
  ( sym: 262; act: 12 ),
  ( sym: 269; act: 13 ),
  ( sym: 272; act: 14 ),
  ( sym: 273; act: 15 ),
  ( sym: 274; act: 16 ),
  ( sym: 275; act: 17 ),
  ( sym: 276; act: 18 ),
  ( sym: 278; act: 19 ),
{ 2: }
  ( sym: 257; act: 5 ),
  ( sym: 259; act: 6 ),
  ( sym: 270; act: 7 ),
  ( sym: 0; act: -3 ),
  ( sym: 258; act: -5 ),
  ( sym: 260; act: -5 ),
  ( sym: 262; act: -5 ),
  ( sym: 269; act: -5 ),
  ( sym: 272; act: -5 ),
  ( sym: 273; act: -5 ),
  ( sym: 274; act: -5 ),
  ( sym: 275; act: -5 ),
  ( sym: 276; act: -5 ),
  ( sym: 278; act: -5 ),
{ 3: }
{ 4: }
  ( sym: 0; act: 0 ),
{ 5: }
{ 6: }
  ( sym: 258; act: 10 ),
  ( sym: 260; act: 11 ),
  ( sym: 262; act: 12 ),
  ( sym: 269; act: 13 ),
  ( sym: 272; act: 14 ),
  ( sym: 273; act: 15 ),
  ( sym: 274; act: 16 ),
  ( sym: 275; act: 17 ),
  ( sym: 276; act: 18 ),
  ( sym: 278; act: 19 ),
{ 7: }
  ( sym: 258; act: 23 ),
{ 8: }
{ 9: }
  ( sym: 258; act: 23 ),
  ( sym: 267; act: 27 ),
  ( sym: 286; act: 28 ),
{ 10: }
{ 11: }
{ 12: }
  ( sym: 258; act: 31 ),
  ( sym: 260; act: 11 ),
  ( sym: 263; act: 32 ),
  ( sym: 269; act: 13 ),
  ( sym: 272; act: 14 ),
  ( sym: 273; act: 15 ),
  ( sym: 274; act: 16 ),
  ( sym: 275; act: 17 ),
{ 13: }
{ 14: }
{ 15: }
  ( sym: 260; act: 33 ),
  ( sym: 269; act: 34 ),
  ( sym: 272; act: 35 ),
  ( sym: 274; act: 36 ),
  ( sym: 258; act: -25 ),
  ( sym: 265; act: -25 ),
  ( sym: 267; act: -25 ),
  ( sym: 268; act: -25 ),
  ( sym: 285; act: -25 ),
  ( sym: 286; act: -25 ),
{ 16: }
{ 17: }
{ 18: }
  ( sym: 263; act: 37 ),
{ 19: }
  ( sym: 263; act: 38 ),
{ 20: }
{ 21: }
  ( sym: 258; act: 23 ),
  ( sym: 267; act: 27 ),
  ( sym: 286; act: 28 ),
{ 22: }
  ( sym: 258; act: 44 ),
  ( sym: 267; act: 45 ),
  ( sym: 271; act: 46 ),
  ( sym: 277; act: 47 ),
  ( sym: 284; act: 48 ),
  ( sym: 289; act: 49 ),
  ( sym: 0; act: -12 ),
  ( sym: 257; act: -12 ),
  ( sym: 259; act: -12 ),
  ( sym: 260; act: -12 ),
  ( sym: 262; act: -12 ),
  ( sym: 269; act: -12 ),
  ( sym: 270; act: -12 ),
  ( sym: 272; act: -12 ),
  ( sym: 273; act: -12 ),
  ( sym: 274; act: -12 ),
  ( sym: 275; act: -12 ),
  ( sym: 276; act: -12 ),
  ( sym: 278; act: -12 ),
{ 23: }
{ 24: }
  ( sym: 265; act: 50 ),
  ( sym: 267; act: 51 ),
  ( sym: 261; act: -34 ),
  ( sym: 285; act: -34 ),
{ 25: }
{ 26: }
  ( sym: 261; act: 52 ),
  ( sym: 285; act: 53 ),
{ 27: }
  ( sym: 258; act: 23 ),
  ( sym: 267; act: 27 ),
  ( sym: 286; act: 28 ),
{ 28: }
  ( sym: 258; act: 23 ),
  ( sym: 267; act: 27 ),
  ( sym: 286; act: 28 ),
{ 29: }
{ 30: }
  ( sym: 263; act: 56 ),
{ 31: }
  ( sym: 258; act: -32 ),
  ( sym: 265; act: -32 ),
  ( sym: 267; act: -32 ),
  ( sym: 268; act: -32 ),
  ( sym: 285; act: -32 ),
  ( sym: 286; act: -32 ),
  ( sym: 263; act: -56 ),
{ 32: }
  ( sym: 258; act: 10 ),
  ( sym: 260; act: 11 ),
  ( sym: 262; act: 12 ),
  ( sym: 269; act: 13 ),
  ( sym: 272; act: 14 ),
  ( sym: 273; act: 15 ),
  ( sym: 274; act: 16 ),
  ( sym: 275; act: 17 ),
  ( sym: 276; act: 18 ),
  ( sym: 278; act: 19 ),
{ 33: }
{ 34: }
{ 35: }
{ 36: }
{ 37: }
  ( sym: 258; act: 10 ),
  ( sym: 260; act: 11 ),
  ( sym: 262; act: 12 ),
  ( sym: 269; act: 13 ),
  ( sym: 272; act: 14 ),
  ( sym: 273; act: 15 ),
  ( sym: 274; act: 16 ),
  ( sym: 275; act: 17 ),
  ( sym: 276; act: 18 ),
  ( sym: 278; act: 19 ),
{ 38: }
  ( sym: 258; act: 23 ),
{ 39: }
  ( sym: 261; act: 63 ),
  ( sym: 285; act: 53 ),
{ 40: }
{ 41: }
  ( sym: 280; act: 64 ),
  ( sym: 281; act: 65 ),
  ( sym: 282; act: 66 ),
  ( sym: 283; act: 67 ),
  ( sym: 284; act: 68 ),
  ( sym: 286; act: 69 ),
  ( sym: 287; act: 70 ),
  ( sym: 288; act: 71 ),
  ( sym: 289; act: 72 ),
  ( sym: 0; act: -57 ),
  ( sym: 257; act: -57 ),
  ( sym: 258; act: -57 ),
  ( sym: 259; act: -57 ),
  ( sym: 260; act: -57 ),
  ( sym: 262; act: -57 ),
  ( sym: 266; act: -57 ),
  ( sym: 268; act: -57 ),
  ( sym: 269; act: -57 ),
  ( sym: 270; act: -57 ),
  ( sym: 272; act: -57 ),
  ( sym: 273; act: -57 ),
  ( sym: 274; act: -57 ),
  ( sym: 275; act: -57 ),
  ( sym: 276; act: -57 ),
  ( sym: 278; act: -57 ),
{ 42: }
{ 43: }
{ 44: }
{ 45: }
  ( sym: 258; act: 44 ),
  ( sym: 267; act: 76 ),
  ( sym: 271; act: 46 ),
  ( sym: 277; act: 47 ),
  ( sym: 284; act: 48 ),
  ( sym: 289; act: 49 ),
{ 46: }
{ 47: }
{ 48: }
  ( sym: 258; act: 44 ),
  ( sym: 267; act: 76 ),
  ( sym: 271; act: 46 ),
  ( sym: 277; act: 47 ),
  ( sym: 284; act: 48 ),
  ( sym: 289; act: 49 ),
{ 49: }
  ( sym: 258; act: 44 ),
  ( sym: 267; act: 76 ),
  ( sym: 271; act: 46 ),
  ( sym: 277; act: 47 ),
  ( sym: 284; act: 48 ),
  ( sym: 289; act: 49 ),
{ 50: }
  ( sym: 258; act: 44 ),
  ( sym: 267; act: 76 ),
  ( sym: 271; act: 46 ),
  ( sym: 277; act: 47 ),
  ( sym: 284; act: 48 ),
  ( sym: 289; act: 49 ),
{ 51: }
  ( sym: 268; act: 83 ),
  ( sym: 275; act: 84 ),
  ( sym: 279; act: 85 ),
  ( sym: 258; act: -36 ),
  ( sym: 260; act: -36 ),
  ( sym: 262; act: -36 ),
  ( sym: 269; act: -36 ),
  ( sym: 272; act: -36 ),
  ( sym: 273; act: -36 ),
  ( sym: 274; act: -36 ),
  ( sym: 276; act: -36 ),
  ( sym: 278; act: -36 ),
{ 52: }
{ 53: }
  ( sym: 258; act: 23 ),
  ( sym: 267; act: 27 ),
  ( sym: 286; act: 28 ),
{ 54: }
  ( sym: 265; act: 50 ),
  ( sym: 267; act: 51 ),
  ( sym: 268; act: 87 ),
{ 55: }
  ( sym: 265; act: 50 ),
  ( sym: 267; act: 51 ),
  ( sym: 261; act: -42 ),
  ( sym: 268; act: -42 ),
  ( sym: 285; act: -42 ),
{ 56: }
  ( sym: 258; act: 10 ),
  ( sym: 260; act: 11 ),
  ( sym: 262; act: 12 ),
  ( sym: 269; act: 13 ),
  ( sym: 272; act: 14 ),
  ( sym: 273; act: 15 ),
  ( sym: 274; act: 16 ),
  ( sym: 275; act: 17 ),
  ( sym: 276; act: 18 ),
  ( sym: 278; act: 19 ),
{ 57: }
{ 58: }
  ( sym: 258; act: 10 ),
  ( sym: 260; act: 11 ),
  ( sym: 262; act: 12 ),
  ( sym: 264; act: 90 ),
  ( sym: 269; act: 13 ),
  ( sym: 272; act: 14 ),
  ( sym: 273; act: 15 ),
  ( sym: 274; act: 16 ),
  ( sym: 275; act: 17 ),
  ( sym: 276; act: 18 ),
  ( sym: 278; act: 19 ),
{ 59: }
  ( sym: 258; act: 23 ),
  ( sym: 267; act: 27 ),
  ( sym: 286; act: 28 ),
{ 60: }
  ( sym: 258; act: 10 ),
  ( sym: 260; act: 11 ),
  ( sym: 262; act: 12 ),
  ( sym: 264; act: 92 ),
  ( sym: 269; act: 13 ),
  ( sym: 272; act: 14 ),
  ( sym: 273; act: 15 ),
  ( sym: 274; act: 16 ),
  ( sym: 275; act: 17 ),
  ( sym: 276; act: 18 ),
  ( sym: 278; act: 19 ),
{ 61: }
  ( sym: 264; act: 93 ),
  ( sym: 285; act: 94 ),
{ 62: }
{ 63: }
{ 64: }
  ( sym: 258; act: 44 ),
  ( sym: 267; act: 76 ),
  ( sym: 271; act: 46 ),
  ( sym: 277; act: 47 ),
  ( sym: 284; act: 48 ),
  ( sym: 289; act: 49 ),
{ 65: }
  ( sym: 258; act: 44 ),
  ( sym: 267; act: 76 ),
  ( sym: 271; act: 46 ),
  ( sym: 277; act: 47 ),
  ( sym: 284; act: 48 ),
  ( sym: 289; act: 49 ),
{ 66: }
  ( sym: 258; act: 44 ),
  ( sym: 267; act: 76 ),
  ( sym: 271; act: 46 ),
  ( sym: 277; act: 47 ),
  ( sym: 284; act: 48 ),
  ( sym: 289; act: 49 ),
{ 67: }
  ( sym: 258; act: 44 ),
  ( sym: 267; act: 76 ),
  ( sym: 271; act: 46 ),
  ( sym: 277; act: 47 ),
  ( sym: 284; act: 48 ),
  ( sym: 289; act: 49 ),
{ 68: }
  ( sym: 258; act: 44 ),
  ( sym: 267; act: 76 ),
  ( sym: 271; act: 46 ),
  ( sym: 277; act: 47 ),
  ( sym: 284; act: 48 ),
  ( sym: 289; act: 49 ),
{ 69: }
  ( sym: 258; act: 44 ),
  ( sym: 267; act: 76 ),
  ( sym: 271; act: 46 ),
  ( sym: 277; act: 47 ),
  ( sym: 284; act: 48 ),
  ( sym: 289; act: 49 ),
{ 70: }
  ( sym: 258; act: 44 ),
  ( sym: 267; act: 76 ),
  ( sym: 271; act: 46 ),
  ( sym: 277; act: 47 ),
  ( sym: 284; act: 48 ),
  ( sym: 289; act: 49 ),
{ 71: }
  ( sym: 258; act: 44 ),
  ( sym: 267; act: 76 ),
  ( sym: 271; act: 46 ),
  ( sym: 277; act: 47 ),
  ( sym: 284; act: 48 ),
  ( sym: 289; act: 49 ),
{ 72: }
  ( sym: 258; act: 44 ),
  ( sym: 267; act: 76 ),
  ( sym: 271; act: 46 ),
  ( sym: 277; act: 47 ),
  ( sym: 284; act: 48 ),
  ( sym: 289; act: 49 ),
{ 73: }
  ( sym: 268; act: 104 ),
{ 74: }
  ( sym: 268; act: 105 ),
  ( sym: 285; act: 94 ),
{ 75: }
  ( sym: 268; act: 106 ),
  ( sym: 280; act: -69 ),
  ( sym: 281; act: -69 ),
  ( sym: 282; act: -69 ),
  ( sym: 283; act: -69 ),
  ( sym: 284; act: -69 ),
  ( sym: 286; act: -69 ),
  ( sym: 287; act: -69 ),
  ( sym: 288; act: -69 ),
  ( sym: 289; act: -69 ),
  ( sym: 285; act: -76 ),
{ 76: }
  ( sym: 258; act: 44 ),
  ( sym: 267; act: 76 ),
  ( sym: 271; act: 46 ),
  ( sym: 277; act: 47 ),
  ( sym: 284; act: 48 ),
  ( sym: 289; act: 49 ),
{ 77: }
{ 78: }
{ 79: }
  ( sym: 266; act: 107 ),
{ 80: }
  ( sym: 268; act: 108 ),
  ( sym: 285; act: 109 ),
{ 81: }
{ 82: }
  ( sym: 258; act: 10 ),
  ( sym: 260; act: 11 ),
  ( sym: 262; act: 12 ),
  ( sym: 269; act: 13 ),
  ( sym: 272; act: 14 ),
  ( sym: 273; act: 15 ),
  ( sym: 274; act: 16 ),
  ( sym: 275; act: 17 ),
  ( sym: 276; act: 18 ),
  ( sym: 278; act: 19 ),
{ 83: }
{ 84: }
  ( sym: 268; act: 111 ),
{ 85: }
{ 86: }
  ( sym: 265; act: 50 ),
  ( sym: 267; act: 51 ),
  ( sym: 261; act: -33 ),
  ( sym: 285; act: -33 ),
{ 87: }
{ 88: }
  ( sym: 258; act: 10 ),
  ( sym: 260; act: 11 ),
  ( sym: 262; act: 12 ),
  ( sym: 264; act: 112 ),
  ( sym: 269; act: 13 ),
  ( sym: 272; act: 14 ),
  ( sym: 273; act: 15 ),
  ( sym: 274; act: 16 ),
  ( sym: 275; act: 17 ),
  ( sym: 276; act: 18 ),
  ( sym: 278; act: 19 ),
{ 89: }
{ 90: }
{ 91: }
  ( sym: 261; act: 113 ),
  ( sym: 285; act: 53 ),
{ 92: }
{ 93: }
{ 94: }
  ( sym: 258; act: 23 ),
{ 95: }
{ 96: }
{ 97: }
{ 98: }
{ 99: }
{ 100: }
{ 101: }
{ 102: }
{ 103: }
{ 104: }
{ 105: }
  ( sym: 258; act: 44 ),
  ( sym: 267; act: 76 ),
  ( sym: 271; act: 46 ),
  ( sym: 277; act: 47 ),
  ( sym: 284; act: 48 ),
  ( sym: 289; act: 49 ),
{ 106: }
  ( sym: 258; act: 44 ),
  ( sym: 267; act: 76 ),
  ( sym: 271; act: 46 ),
  ( sym: 277; act: 47 ),
  ( sym: 284; act: 48 ),
  ( sym: 289; act: 49 ),
  ( sym: 0; act: -10 ),
  ( sym: 257; act: -10 ),
  ( sym: 259; act: -10 ),
  ( sym: 260; act: -10 ),
  ( sym: 262; act: -10 ),
  ( sym: 269; act: -10 ),
  ( sym: 270; act: -10 ),
  ( sym: 272; act: -10 ),
  ( sym: 273; act: -10 ),
  ( sym: 274; act: -10 ),
  ( sym: 275; act: -10 ),
  ( sym: 276; act: -10 ),
  ( sym: 278; act: -10 ),
{ 107: }
{ 108: }
{ 109: }
  ( sym: 279; act: 85 ),
  ( sym: 258; act: -36 ),
  ( sym: 260; act: -36 ),
  ( sym: 262; act: -36 ),
  ( sym: 269; act: -36 ),
  ( sym: 272; act: -36 ),
  ( sym: 273; act: -36 ),
  ( sym: 274; act: -36 ),
  ( sym: 275; act: -36 ),
  ( sym: 276; act: -36 ),
  ( sym: 278; act: -36 ),
{ 110: }
  ( sym: 258; act: 23 ),
  ( sym: 267; act: 120 ),
  ( sym: 286; act: 121 ),
  ( sym: 265; act: -54 ),
  ( sym: 268; act: -54 ),
  ( sym: 285; act: -54 ),
{ 111: }
{ 112: }
{ 113: }
{ 114: }
{ 115: }
{ 116: }
{ 117: }
{ 118: }
  ( sym: 265; act: 122 ),
  ( sym: 267; act: 123 ),
  ( sym: 268; act: -38 ),
  ( sym: 285; act: -38 ),
{ 119: }
  ( sym: 265; act: 50 ),
  ( sym: 267; act: 51 ),
  ( sym: 268; act: -37 ),
  ( sym: 285; act: -37 ),
{ 120: }
  ( sym: 258; act: 23 ),
  ( sym: 267; act: 120 ),
  ( sym: 286; act: 121 ),
  ( sym: 265; act: -54 ),
  ( sym: 268; act: -54 ),
{ 121: }
  ( sym: 258; act: 23 ),
  ( sym: 267; act: 120 ),
  ( sym: 286; act: 121 ),
  ( sym: 265; act: -54 ),
  ( sym: 268; act: -54 ),
  ( sym: 285; act: -54 ),
{ 122: }
  ( sym: 258; act: 44 ),
  ( sym: 267; act: 76 ),
  ( sym: 271; act: 46 ),
  ( sym: 277; act: 47 ),
  ( sym: 284; act: 48 ),
  ( sym: 289; act: 49 ),
{ 123: }
  ( sym: 268; act: 128 ),
  ( sym: 275; act: 129 ),
  ( sym: 279; act: 85 ),
  ( sym: 258; act: -36 ),
  ( sym: 260; act: -36 ),
  ( sym: 262; act: -36 ),
  ( sym: 269; act: -36 ),
  ( sym: 272; act: -36 ),
  ( sym: 273; act: -36 ),
  ( sym: 274; act: -36 ),
  ( sym: 276; act: -36 ),
  ( sym: 278; act: -36 ),
{ 124: }
  ( sym: 265; act: 122 ),
  ( sym: 267; act: 123 ),
  ( sym: 268; act: 130 ),
{ 125: }
  ( sym: 265; act: 122 ),
  ( sym: 267; act: 123 ),
  ( sym: 268; act: -48 ),
  ( sym: 285; act: -48 ),
{ 126: }
  ( sym: 266; act: 131 ),
{ 127: }
  ( sym: 268; act: 132 ),
  ( sym: 285; act: 109 ),
{ 128: }
{ 129: }
  ( sym: 268; act: 133 )
{ 130: }
{ 131: }
{ 132: }
{ 133: }
);

yyg : array [1..yyngotos] of YYARec = (
{ 0: }
  ( sym: -5; act: 1 ),
  ( sym: -4; act: 2 ),
  ( sym: -3; act: 3 ),
  ( sym: -2; act: 4 ),
{ 1: }
  ( sym: -12; act: 8 ),
  ( sym: -6; act: 9 ),
{ 2: }
  ( sym: -5; act: 1 ),
  ( sym: -4; act: 2 ),
  ( sym: -3; act: 20 ),
{ 3: }
{ 4: }
{ 5: }
{ 6: }
  ( sym: -12; act: 8 ),
  ( sym: -6; act: 21 ),
{ 7: }
  ( sym: -8; act: 22 ),
{ 8: }
{ 9: }
  ( sym: -14; act: 24 ),
  ( sym: -8; act: 25 ),
  ( sym: -7; act: 26 ),
{ 10: }
{ 11: }
{ 12: }
  ( sym: -12; act: 29 ),
  ( sym: -8; act: 30 ),
{ 13: }
{ 14: }
{ 15: }
{ 16: }
{ 17: }
{ 18: }
{ 19: }
{ 20: }
{ 21: }
  ( sym: -14; act: 24 ),
  ( sym: -8; act: 25 ),
  ( sym: -7; act: 39 ),
{ 22: }
  ( sym: -21; act: 40 ),
  ( sym: -20; act: 41 ),
  ( sym: -10; act: 42 ),
  ( sym: -8; act: 43 ),
{ 23: }
{ 24: }
{ 25: }
{ 26: }
{ 27: }
  ( sym: -14; act: 54 ),
  ( sym: -8; act: 25 ),
{ 28: }
  ( sym: -14; act: 55 ),
  ( sym: -8; act: 25 ),
{ 29: }
{ 30: }
{ 31: }
{ 32: }
  ( sym: -13; act: 57 ),
  ( sym: -12; act: 8 ),
  ( sym: -11; act: 58 ),
  ( sym: -6; act: 59 ),
{ 33: }
{ 34: }
{ 35: }
{ 36: }
{ 37: }
  ( sym: -13; act: 57 ),
  ( sym: -12; act: 8 ),
  ( sym: -11; act: 60 ),
  ( sym: -6; act: 59 ),
{ 38: }
  ( sym: -9; act: 61 ),
  ( sym: -8; act: 62 ),
{ 39: }
{ 40: }
{ 41: }
{ 42: }
{ 43: }
{ 44: }
{ 45: }
  ( sym: -21; act: 40 ),
  ( sym: -20; act: 41 ),
  ( sym: -10; act: 73 ),
  ( sym: -9; act: 74 ),
  ( sym: -8; act: 75 ),
{ 46: }
{ 47: }
{ 48: }
  ( sym: -21; act: 77 ),
  ( sym: -8; act: 43 ),
{ 49: }
  ( sym: -21; act: 78 ),
  ( sym: -8; act: 43 ),
{ 50: }
  ( sym: -21; act: 40 ),
  ( sym: -20; act: 41 ),
  ( sym: -10; act: 79 ),
  ( sym: -8; act: 43 ),
{ 51: }
  ( sym: -18; act: 80 ),
  ( sym: -16; act: 81 ),
  ( sym: -15; act: 82 ),
{ 52: }
{ 53: }
  ( sym: -14; act: 86 ),
  ( sym: -8; act: 25 ),
{ 54: }
{ 55: }
{ 56: }
  ( sym: -13; act: 57 ),
  ( sym: -12; act: 8 ),
  ( sym: -11; act: 88 ),
  ( sym: -6; act: 59 ),
{ 57: }
{ 58: }
  ( sym: -13; act: 89 ),
  ( sym: -12; act: 8 ),
  ( sym: -6; act: 59 ),
{ 59: }
  ( sym: -14; act: 24 ),
  ( sym: -8; act: 25 ),
  ( sym: -7; act: 91 ),
{ 60: }
  ( sym: -13; act: 89 ),
  ( sym: -12; act: 8 ),
  ( sym: -6; act: 59 ),
{ 61: }
{ 62: }
{ 63: }
{ 64: }
  ( sym: -21; act: 95 ),
  ( sym: -8; act: 43 ),
{ 65: }
  ( sym: -21; act: 96 ),
  ( sym: -8; act: 43 ),
{ 66: }
  ( sym: -21; act: 97 ),
  ( sym: -8; act: 43 ),
{ 67: }
  ( sym: -21; act: 98 ),
  ( sym: -8; act: 43 ),
{ 68: }
  ( sym: -21; act: 99 ),
  ( sym: -8; act: 43 ),
{ 69: }
  ( sym: -21; act: 100 ),
  ( sym: -8; act: 43 ),
{ 70: }
  ( sym: -21; act: 101 ),
  ( sym: -8; act: 43 ),
{ 71: }
  ( sym: -21; act: 102 ),
  ( sym: -8; act: 43 ),
{ 72: }
  ( sym: -21; act: 103 ),
  ( sym: -8; act: 43 ),
{ 73: }
{ 74: }
{ 75: }
{ 76: }
  ( sym: -21; act: 40 ),
  ( sym: -20; act: 41 ),
  ( sym: -10; act: 73 ),
  ( sym: -8; act: 43 ),
{ 77: }
{ 78: }
{ 79: }
{ 80: }
{ 81: }
{ 82: }
  ( sym: -12; act: 8 ),
  ( sym: -6; act: 110 ),
{ 83: }
{ 84: }
{ 85: }
{ 86: }
{ 87: }
{ 88: }
  ( sym: -13; act: 89 ),
  ( sym: -12; act: 8 ),
  ( sym: -6; act: 59 ),
{ 89: }
{ 90: }
{ 91: }
{ 92: }
{ 93: }
{ 94: }
  ( sym: -8; act: 114 ),
{ 95: }
{ 96: }
{ 97: }
{ 98: }
{ 99: }
{ 100: }
{ 101: }
{ 102: }
{ 103: }
{ 104: }
{ 105: }
  ( sym: -21; act: 40 ),
  ( sym: -20; act: 41 ),
  ( sym: -10; act: 115 ),
  ( sym: -8; act: 43 ),
{ 106: }
  ( sym: -21; act: 40 ),
  ( sym: -20; act: 41 ),
  ( sym: -10; act: 116 ),
  ( sym: -8; act: 43 ),
{ 107: }
{ 108: }
{ 109: }
  ( sym: -16; act: 117 ),
  ( sym: -15; act: 82 ),
{ 110: }
  ( sym: -17; act: 118 ),
  ( sym: -14; act: 119 ),
  ( sym: -8; act: 25 ),
{ 111: }
{ 112: }
{ 113: }
{ 114: }
{ 115: }
{ 116: }
{ 117: }
{ 118: }
{ 119: }
{ 120: }
  ( sym: -17; act: 124 ),
  ( sym: -14; act: 54 ),
  ( sym: -8; act: 25 ),
{ 121: }
  ( sym: -17; act: 125 ),
  ( sym: -14; act: 55 ),
  ( sym: -8; act: 25 ),
{ 122: }
  ( sym: -21; act: 40 ),
  ( sym: -20; act: 41 ),
  ( sym: -10; act: 126 ),
  ( sym: -8; act: 43 ),
{ 123: }
  ( sym: -18; act: 127 ),
  ( sym: -16; act: 81 ),
  ( sym: -15; act: 82 )
{ 124: }
{ 125: }
{ 126: }
{ 127: }
{ 128: }
{ 129: }
{ 130: }
{ 131: }
{ 132: }
{ 133: }
);

yyd : array [0..yynstates-1] of Integer = (
{ 0: } 0,
{ 1: } 0,
{ 2: } 0,
{ 3: } -1,
{ 4: } 0,
{ 5: } -4,
{ 6: } 0,
{ 7: } 0,
{ 8: } -18,
{ 9: } 0,
{ 10: } -32,
{ 11: } -22,
{ 12: } 0,
{ 13: } -31,
{ 14: } -30,
{ 15: } 0,
{ 16: } -24,
{ 17: } -29,
{ 18: } 0,
{ 19: } 0,
{ 20: } -2,
{ 21: } 0,
{ 22: } 0,
{ 23: } -56,
{ 24: } 0,
{ 25: } -41,
{ 26: } 0,
{ 27: } 0,
{ 28: } 0,
{ 29: } -16,
{ 30: } 0,
{ 31: } 0,
{ 32: } 0,
{ 33: } -23,
{ 34: } -28,
{ 35: } -27,
{ 36: } -26,
{ 37: } 0,
{ 38: } 0,
{ 39: } 0,
{ 40: } -67,
{ 41: } 0,
{ 42: } -11,
{ 43: } -69,
{ 44: } -56,
{ 45: } 0,
{ 46: } -72,
{ 47: } -71,
{ 48: } 0,
{ 49: } 0,
{ 50: } 0,
{ 51: } 0,
{ 52: } -6,
{ 53: } 0,
{ 54: } 0,
{ 55: } 0,
{ 56: } 0,
{ 57: } -20,
{ 58: } 0,
{ 59: } 0,
{ 60: } 0,
{ 61: } 0,
{ 62: } -76,
{ 63: } -7,
{ 64: } 0,
{ 65: } 0,
{ 66: } 0,
{ 67: } 0,
{ 68: } 0,
{ 69: } 0,
{ 70: } 0,
{ 71: } 0,
{ 72: } 0,
{ 73: } 0,
{ 74: } 0,
{ 75: } 0,
{ 76: } 0,
{ 77: } -73,
{ 78: } -74,
{ 79: } 0,
{ 80: } 0,
{ 81: } -39,
{ 82: } 0,
{ 83: } -44,
{ 84: } 0,
{ 85: } -35,
{ 86: } 0,
{ 87: } -47,
{ 88: } 0,
{ 89: } -19,
{ 90: } -14,
{ 91: } 0,
{ 92: } -15,
{ 93: } -17,
{ 94: } 0,
{ 95: } -66,
{ 96: } -65,
{ 97: } -62,
{ 98: } -58,
{ 99: } -59,
{ 100: } -60,
{ 101: } -61,
{ 102: } -63,
{ 103: } -64,
{ 104: } -70,
{ 105: } 0,
{ 106: } 0,
{ 107: } -46,
{ 108: } -43,
{ 109: } 0,
{ 110: } 0,
{ 111: } -45,
{ 112: } -13,
{ 113: } -21,
{ 114: } -75,
{ 115: } -8,
{ 116: } -9,
{ 117: } -40,
{ 118: } 0,
{ 119: } 0,
{ 120: } 0,
{ 121: } 0,
{ 122: } 0,
{ 123: } 0,
{ 124: } 0,
{ 125: } 0,
{ 126: } 0,
{ 127: } 0,
{ 128: } -50,
{ 129: } 0,
{ 130: } -53,
{ 131: } -52,
{ 132: } -49,
{ 133: } -51
);

yyal : array [0..yynstates-1] of Integer = (
{ 0: } 1,
{ 1: } 14,
{ 2: } 24,
{ 3: } 38,
{ 4: } 38,
{ 5: } 39,
{ 6: } 39,
{ 7: } 49,
{ 8: } 50,
{ 9: } 50,
{ 10: } 53,
{ 11: } 53,
{ 12: } 53,
{ 13: } 61,
{ 14: } 61,
{ 15: } 61,
{ 16: } 71,
{ 17: } 71,
{ 18: } 71,
{ 19: } 72,
{ 20: } 73,
{ 21: } 73,
{ 22: } 76,
{ 23: } 95,
{ 24: } 95,
{ 25: } 99,
{ 26: } 99,
{ 27: } 101,
{ 28: } 104,
{ 29: } 107,
{ 30: } 107,
{ 31: } 108,
{ 32: } 115,
{ 33: } 125,
{ 34: } 125,
{ 35: } 125,
{ 36: } 125,
{ 37: } 125,
{ 38: } 135,
{ 39: } 136,
{ 40: } 138,
{ 41: } 138,
{ 42: } 163,
{ 43: } 163,
{ 44: } 163,
{ 45: } 163,
{ 46: } 169,
{ 47: } 169,
{ 48: } 169,
{ 49: } 175,
{ 50: } 181,
{ 51: } 187,
{ 52: } 199,
{ 53: } 199,
{ 54: } 202,
{ 55: } 205,
{ 56: } 210,
{ 57: } 220,
{ 58: } 220,
{ 59: } 231,
{ 60: } 234,
{ 61: } 245,
{ 62: } 247,
{ 63: } 247,
{ 64: } 247,
{ 65: } 253,
{ 66: } 259,
{ 67: } 265,
{ 68: } 271,
{ 69: } 277,
{ 70: } 283,
{ 71: } 289,
{ 72: } 295,
{ 73: } 301,
{ 74: } 302,
{ 75: } 304,
{ 76: } 315,
{ 77: } 321,
{ 78: } 321,
{ 79: } 321,
{ 80: } 322,
{ 81: } 324,
{ 82: } 324,
{ 83: } 334,
{ 84: } 334,
{ 85: } 335,
{ 86: } 335,
{ 87: } 339,
{ 88: } 339,
{ 89: } 350,
{ 90: } 350,
{ 91: } 350,
{ 92: } 352,
{ 93: } 352,
{ 94: } 352,
{ 95: } 353,
{ 96: } 353,
{ 97: } 353,
{ 98: } 353,
{ 99: } 353,
{ 100: } 353,
{ 101: } 353,
{ 102: } 353,
{ 103: } 353,
{ 104: } 353,
{ 105: } 353,
{ 106: } 359,
{ 107: } 378,
{ 108: } 378,
{ 109: } 378,
{ 110: } 389,
{ 111: } 395,
{ 112: } 395,
{ 113: } 395,
{ 114: } 395,
{ 115: } 395,
{ 116: } 395,
{ 117: } 395,
{ 118: } 395,
{ 119: } 399,
{ 120: } 403,
{ 121: } 408,
{ 122: } 414,
{ 123: } 420,
{ 124: } 432,
{ 125: } 435,
{ 126: } 439,
{ 127: } 440,
{ 128: } 442,
{ 129: } 442,
{ 130: } 443,
{ 131: } 443,
{ 132: } 443,
{ 133: } 443
);

yyah : array [0..yynstates-1] of Integer = (
{ 0: } 13,
{ 1: } 23,
{ 2: } 37,
{ 3: } 37,
{ 4: } 38,
{ 5: } 38,
{ 6: } 48,
{ 7: } 49,
{ 8: } 49,
{ 9: } 52,
{ 10: } 52,
{ 11: } 52,
{ 12: } 60,
{ 13: } 60,
{ 14: } 60,
{ 15: } 70,
{ 16: } 70,
{ 17: } 70,
{ 18: } 71,
{ 19: } 72,
{ 20: } 72,
{ 21: } 75,
{ 22: } 94,
{ 23: } 94,
{ 24: } 98,
{ 25: } 98,
{ 26: } 100,
{ 27: } 103,
{ 28: } 106,
{ 29: } 106,
{ 30: } 107,
{ 31: } 114,
{ 32: } 124,
{ 33: } 124,
{ 34: } 124,
{ 35: } 124,
{ 36: } 124,
{ 37: } 134,
{ 38: } 135,
{ 39: } 137,
{ 40: } 137,
{ 41: } 162,
{ 42: } 162,
{ 43: } 162,
{ 44: } 162,
{ 45: } 168,
{ 46: } 168,
{ 47: } 168,
{ 48: } 174,
{ 49: } 180,
{ 50: } 186,
{ 51: } 198,
{ 52: } 198,
{ 53: } 201,
{ 54: } 204,
{ 55: } 209,
{ 56: } 219,
{ 57: } 219,
{ 58: } 230,
{ 59: } 233,
{ 60: } 244,
{ 61: } 246,
{ 62: } 246,
{ 63: } 246,
{ 64: } 252,
{ 65: } 258,
{ 66: } 264,
{ 67: } 270,
{ 68: } 276,
{ 69: } 282,
{ 70: } 288,
{ 71: } 294,
{ 72: } 300,
{ 73: } 301,
{ 74: } 303,
{ 75: } 314,
{ 76: } 320,
{ 77: } 320,
{ 78: } 320,
{ 79: } 321,
{ 80: } 323,
{ 81: } 323,
{ 82: } 333,
{ 83: } 333,
{ 84: } 334,
{ 85: } 334,
{ 86: } 338,
{ 87: } 338,
{ 88: } 349,
{ 89: } 349,
{ 90: } 349,
{ 91: } 351,
{ 92: } 351,
{ 93: } 351,
{ 94: } 352,
{ 95: } 352,
{ 96: } 352,
{ 97: } 352,
{ 98: } 352,
{ 99: } 352,
{ 100: } 352,
{ 101: } 352,
{ 102: } 352,
{ 103: } 352,
{ 104: } 352,
{ 105: } 358,
{ 106: } 377,
{ 107: } 377,
{ 108: } 377,
{ 109: } 388,
{ 110: } 394,
{ 111: } 394,
{ 112: } 394,
{ 113: } 394,
{ 114: } 394,
{ 115: } 394,
{ 116: } 394,
{ 117: } 394,
{ 118: } 398,
{ 119: } 402,
{ 120: } 407,
{ 121: } 413,
{ 122: } 419,
{ 123: } 431,
{ 124: } 434,
{ 125: } 438,
{ 126: } 439,
{ 127: } 441,
{ 128: } 441,
{ 129: } 442,
{ 130: } 442,
{ 131: } 442,
{ 132: } 442,
{ 133: } 442
);

yygl : array [0..yynstates-1] of Integer = (
{ 0: } 1,
{ 1: } 5,
{ 2: } 7,
{ 3: } 10,
{ 4: } 10,
{ 5: } 10,
{ 6: } 10,
{ 7: } 12,
{ 8: } 13,
{ 9: } 13,
{ 10: } 16,
{ 11: } 16,
{ 12: } 16,
{ 13: } 18,
{ 14: } 18,
{ 15: } 18,
{ 16: } 18,
{ 17: } 18,
{ 18: } 18,
{ 19: } 18,
{ 20: } 18,
{ 21: } 18,
{ 22: } 21,
{ 23: } 25,
{ 24: } 25,
{ 25: } 25,
{ 26: } 25,
{ 27: } 25,
{ 28: } 27,
{ 29: } 29,
{ 30: } 29,
{ 31: } 29,
{ 32: } 29,
{ 33: } 33,
{ 34: } 33,
{ 35: } 33,
{ 36: } 33,
{ 37: } 33,
{ 38: } 37,
{ 39: } 39,
{ 40: } 39,
{ 41: } 39,
{ 42: } 39,
{ 43: } 39,
{ 44: } 39,
{ 45: } 39,
{ 46: } 44,
{ 47: } 44,
{ 48: } 44,
{ 49: } 46,
{ 50: } 48,
{ 51: } 52,
{ 52: } 55,
{ 53: } 55,
{ 54: } 57,
{ 55: } 57,
{ 56: } 57,
{ 57: } 61,
{ 58: } 61,
{ 59: } 64,
{ 60: } 67,
{ 61: } 70,
{ 62: } 70,
{ 63: } 70,
{ 64: } 70,
{ 65: } 72,
{ 66: } 74,
{ 67: } 76,
{ 68: } 78,
{ 69: } 80,
{ 70: } 82,
{ 71: } 84,
{ 72: } 86,
{ 73: } 88,
{ 74: } 88,
{ 75: } 88,
{ 76: } 88,
{ 77: } 92,
{ 78: } 92,
{ 79: } 92,
{ 80: } 92,
{ 81: } 92,
{ 82: } 92,
{ 83: } 94,
{ 84: } 94,
{ 85: } 94,
{ 86: } 94,
{ 87: } 94,
{ 88: } 94,
{ 89: } 97,
{ 90: } 97,
{ 91: } 97,
{ 92: } 97,
{ 93: } 97,
{ 94: } 97,
{ 95: } 98,
{ 96: } 98,
{ 97: } 98,
{ 98: } 98,
{ 99: } 98,
{ 100: } 98,
{ 101: } 98,
{ 102: } 98,
{ 103: } 98,
{ 104: } 98,
{ 105: } 98,
{ 106: } 102,
{ 107: } 106,
{ 108: } 106,
{ 109: } 106,
{ 110: } 108,
{ 111: } 111,
{ 112: } 111,
{ 113: } 111,
{ 114: } 111,
{ 115: } 111,
{ 116: } 111,
{ 117: } 111,
{ 118: } 111,
{ 119: } 111,
{ 120: } 111,
{ 121: } 114,
{ 122: } 117,
{ 123: } 121,
{ 124: } 124,
{ 125: } 124,
{ 126: } 124,
{ 127: } 124,
{ 128: } 124,
{ 129: } 124,
{ 130: } 124,
{ 131: } 124,
{ 132: } 124,
{ 133: } 124
);

yygh : array [0..yynstates-1] of Integer = (
{ 0: } 4,
{ 1: } 6,
{ 2: } 9,
{ 3: } 9,
{ 4: } 9,
{ 5: } 9,
{ 6: } 11,
{ 7: } 12,
{ 8: } 12,
{ 9: } 15,
{ 10: } 15,
{ 11: } 15,
{ 12: } 17,
{ 13: } 17,
{ 14: } 17,
{ 15: } 17,
{ 16: } 17,
{ 17: } 17,
{ 18: } 17,
{ 19: } 17,
{ 20: } 17,
{ 21: } 20,
{ 22: } 24,
{ 23: } 24,
{ 24: } 24,
{ 25: } 24,
{ 26: } 24,
{ 27: } 26,
{ 28: } 28,
{ 29: } 28,
{ 30: } 28,
{ 31: } 28,
{ 32: } 32,
{ 33: } 32,
{ 34: } 32,
{ 35: } 32,
{ 36: } 32,
{ 37: } 36,
{ 38: } 38,
{ 39: } 38,
{ 40: } 38,
{ 41: } 38,
{ 42: } 38,
{ 43: } 38,
{ 44: } 38,
{ 45: } 43,
{ 46: } 43,
{ 47: } 43,
{ 48: } 45,
{ 49: } 47,
{ 50: } 51,
{ 51: } 54,
{ 52: } 54,
{ 53: } 56,
{ 54: } 56,
{ 55: } 56,
{ 56: } 60,
{ 57: } 60,
{ 58: } 63,
{ 59: } 66,
{ 60: } 69,
{ 61: } 69,
{ 62: } 69,
{ 63: } 69,
{ 64: } 71,
{ 65: } 73,
{ 66: } 75,
{ 67: } 77,
{ 68: } 79,
{ 69: } 81,
{ 70: } 83,
{ 71: } 85,
{ 72: } 87,
{ 73: } 87,
{ 74: } 87,
{ 75: } 87,
{ 76: } 91,
{ 77: } 91,
{ 78: } 91,
{ 79: } 91,
{ 80: } 91,
{ 81: } 91,
{ 82: } 93,
{ 83: } 93,
{ 84: } 93,
{ 85: } 93,
{ 86: } 93,
{ 87: } 93,
{ 88: } 96,
{ 89: } 96,
{ 90: } 96,
{ 91: } 96,
{ 92: } 96,
{ 93: } 96,
{ 94: } 97,
{ 95: } 97,
{ 96: } 97,
{ 97: } 97,
{ 98: } 97,
{ 99: } 97,
{ 100: } 97,
{ 101: } 97,
{ 102: } 97,
{ 103: } 97,
{ 104: } 97,
{ 105: } 101,
{ 106: } 105,
{ 107: } 105,
{ 108: } 105,
{ 109: } 107,
{ 110: } 110,
{ 111: } 110,
{ 112: } 110,
{ 113: } 110,
{ 114: } 110,
{ 115: } 110,
{ 116: } 110,
{ 117: } 110,
{ 118: } 110,
{ 119: } 110,
{ 120: } 113,
{ 121: } 116,
{ 122: } 120,
{ 123: } 123,
{ 124: } 123,
{ 125: } 123,
{ 126: } 123,
{ 127: } 123,
{ 128: } 123,
{ 129: } 123,
{ 130: } 123,
{ 131: } 123,
{ 132: } 123,
{ 133: } 123
);

yyr : array [1..yynrules] of YYRRec = (
{ 1: } ( len: 1; sym: -2 ),
{ 2: } ( len: 2; sym: -3 ),
{ 3: } ( len: 1; sym: -3 ),
{ 4: } ( len: 1; sym: -5 ),
{ 5: } ( len: 0; sym: -5 ),
{ 6: } ( len: 4; sym: -4 ),
{ 7: } ( len: 4; sym: -4 ),
{ 8: } ( len: 6; sym: -4 ),
{ 9: } ( len: 6; sym: -4 ),
{ 10: } ( len: 5; sym: -4 ),
{ 11: } ( len: 3; sym: -4 ),
{ 12: } ( len: 2; sym: -4 ),
{ 13: } ( len: 5; sym: -6 ),
{ 14: } ( len: 4; sym: -6 ),
{ 15: } ( len: 4; sym: -6 ),
{ 16: } ( len: 2; sym: -6 ),
{ 17: } ( len: 4; sym: -6 ),
{ 18: } ( len: 1; sym: -6 ),
{ 19: } ( len: 2; sym: -11 ),
{ 20: } ( len: 1; sym: -11 ),
{ 21: } ( len: 3; sym: -13 ),
{ 22: } ( len: 1; sym: -12 ),
{ 23: } ( len: 2; sym: -12 ),
{ 24: } ( len: 1; sym: -12 ),
{ 25: } ( len: 1; sym: -12 ),
{ 26: } ( len: 2; sym: -12 ),
{ 27: } ( len: 2; sym: -12 ),
{ 28: } ( len: 2; sym: -12 ),
{ 29: } ( len: 1; sym: -12 ),
{ 30: } ( len: 1; sym: -12 ),
{ 31: } ( len: 1; sym: -12 ),
{ 32: } ( len: 1; sym: -12 ),
{ 33: } ( len: 3; sym: -7 ),
{ 34: } ( len: 1; sym: -7 ),
{ 35: } ( len: 1; sym: -15 ),
{ 36: } ( len: 0; sym: -15 ),
{ 37: } ( len: 3; sym: -16 ),
{ 38: } ( len: 3; sym: -16 ),
{ 39: } ( len: 1; sym: -18 ),
{ 40: } ( len: 3; sym: -18 ),
{ 41: } ( len: 1; sym: -14 ),
{ 42: } ( len: 2; sym: -14 ),
{ 43: } ( len: 4; sym: -14 ),
{ 44: } ( len: 3; sym: -14 ),
{ 45: } ( len: 4; sym: -14 ),
{ 46: } ( len: 4; sym: -14 ),
{ 47: } ( len: 3; sym: -14 ),
{ 48: } ( len: 2; sym: -17 ),
{ 49: } ( len: 4; sym: -17 ),
{ 50: } ( len: 3; sym: -17 ),
{ 51: } ( len: 4; sym: -17 ),
{ 52: } ( len: 4; sym: -17 ),
{ 53: } ( len: 3; sym: -17 ),
{ 54: } ( len: 0; sym: -17 ),
{ 55: } ( len: 1; sym: -19 ),
{ 56: } ( len: 1; sym: -8 ),
{ 57: } ( len: 1; sym: -10 ),
{ 58: } ( len: 3; sym: -20 ),
{ 59: } ( len: 3; sym: -20 ),
{ 60: } ( len: 3; sym: -20 ),
{ 61: } ( len: 3; sym: -20 ),
{ 62: } ( len: 3; sym: -20 ),
{ 63: } ( len: 3; sym: -20 ),
{ 64: } ( len: 3; sym: -20 ),
{ 65: } ( len: 3; sym: -20 ),
{ 66: } ( len: 3; sym: -20 ),
{ 67: } ( len: 1; sym: -20 ),
{ 68: } ( len: 1; sym: -21 ),
{ 69: } ( len: 1; sym: -21 ),
{ 70: } ( len: 3; sym: -21 ),
{ 71: } ( len: 1; sym: -21 ),
{ 72: } ( len: 1; sym: -21 ),
{ 73: } ( len: 2; sym: -21 ),
{ 74: } ( len: 2; sym: -21 ),
{ 75: } ( len: 3; sym: -9 ),
{ 76: } ( len: 1; sym: -9 ),
{ 77: } ( len: 3; sym: -22 )
);


const _error = 256; (* error token *)

function yyact(state, sym : Integer; var act : Integer) : Boolean;
  (* search action table *)
  var k : Integer;
  begin
    k := yyal[state];
    while (k<=yyah[state]) and (yya[k].sym<>sym) do inc(k);
    if k>yyah[state] then
      yyact := false
    else
      begin
        act := yya[k].act;
        yyact := true;
      end;
  end(*yyact*);

function yygoto(state, sym : Integer; var nstate : Integer) : Boolean;
  (* search goto table *)
  var k : Integer;
  begin
    k := yygl[state];
    while (k<=yygh[state]) and (yyg[k].sym<>sym) do inc(k);
    if k>yygh[state] then
      yygoto := false
    else
      begin
        nstate := yyg[k].act;
        yygoto := true;
      end;
  end(*yygoto*);

label parse, next, error, errlab, shift, reduce, accept, abort;

begin(*yyparse*)

  (* initialize: *)

  yystate := 0; yychar := -1; yynerrs := 0; yyerrflag := 0; yysp := 0;

{$ifdef yydebug}
  yydebug := true;
{$else}
  yydebug := false;
{$endif}

parse:

  (* push state and value: *)

  inc(yysp);
  if yysp>yymaxdepth then
    begin
      yyerror('yyparse stack overflow');
      goto abort;
    end;
  yys[yysp] := yystate; yyv[yysp] := yyval;

next:

  if (yyd[yystate]=0) and (yychar=-1) then
    (* get next symbol *)
    begin
      yychar := yylex; if yychar<0 then yychar := 0;
    end;

  if yydebug then writeln('state ', yystate, ', char ', yychar);

  (* determine parse action: *)

  yyn := yyd[yystate];
  if yyn<>0 then goto reduce; (* simple state *)

  (* no default action; search parse table *)

  if not yyact(yystate, yychar, yyn) then goto error
  else if yyn>0 then                      goto shift
  else if yyn<0 then                      goto reduce
  else                                    goto accept;

error:

  (* error; start error recovery: *)

  if yyerrflag=0 then yyerror('syntax error');

errlab:

  if yyerrflag=0 then inc(yynerrs);     (* new error *)

  if yyerrflag<=2 then                  (* incomplete recovery; try again *)
    begin
      yyerrflag := 3;
      (* uncover a state with shift action on error token *)
      while (yysp>0) and not ( yyact(yys[yysp], _error, yyn) and
                               (yyn>0) ) do
        begin
          if yydebug then
            if yysp>1 then
              writeln('error recovery pops state ', yys[yysp], ', uncovers ',
                      yys[yysp-1])
            else
              writeln('error recovery fails ... abort');
          dec(yysp);
        end;
      if yysp=0 then goto abort; (* parser has fallen from stack; abort *)
      yystate := yyn;            (* simulate shift on error *)
      goto parse;
    end
  else                                  (* no shift yet; discard symbol *)
    begin
      if yydebug then writeln('error recovery discards char ', yychar);
      if yychar=0 then goto abort; (* end of input; abort *)
      yychar := -1; goto next;     (* clear lookahead char and try again *)
    end;

shift:

  (* go to new state, clear lookahead character: *)

  yystate := yyn; yychar := -1; yyval := yylval;
  if yyerrflag>0 then dec(yyerrflag);

  goto parse;

reduce:

  (* execute action, pop rule from stack, and go to next state: *)

  if yydebug then writeln('reduce ', -yyn);

  yyflag := yyfnone; yyaction(-yyn);
  dec(yysp, yyr[-yyn].len);
  if yygoto(yys[yysp], yyr[-yyn].sym, yyn) then yystate := yyn;

  (* handle action calls to yyaccept, yyabort and yyerror: *)

  case yyflag of
    yyfaccept : goto accept;
    yyfabort  : goto abort;
    yyferror  : goto errlab;
  end;

  goto parse;

accept:

  yyparse := 0; exit;

abort:

  yyparse := 1; exit;

end(*yyparse*);


function yylex : Integer;

 begin
 yylex:=scan.yylex;
 end;

var r:integer; SS:string;
begin
   debug:=true;
   aktspace:='    ';
   block_type:=bt_no;
   IsExtern:=false;
   Assign(extfile,'ext.tmp'); rewrite(extfile);
   r:=yyparse;
   writeln(outfile);
   writeln(outfile,'  implementation');
   writeln(outfile);
   writeln(outfile,'const __ExtrnlLibrary=''system.dll''; {Setup as you need!}');
   writeln(outfile);
   reset(extfile);
   while not eof(extfile) do
    begin
    readln(extfile,SS);

    writeln(outfile,SS);
    end;

   writeln(outfile);
   writeln(outfile,'end.');
   close(extfile);
   close(outfile);

end.