%{
program convert;

(*
 History:
   25.9.1996:
      first version

   26.9.1996:
      - structs are supported
      - unsigned implemented
      - procedure variables implemented
      - void * becomes pointer
      - unions implemented
      - enumerations
      - number post- and prefixes
      - operatores unary-, << and >>
      - problem with the priority of [], (), and * fixed
      - procedures and functions

   28.9.1996:
      - formal paramters

   22-26.5.1997  made by Mark Malakanov     mark@av.krasnoyarsk.su
      - full ariphmetic and logic expressions in #define

      - #define with params changes to function (first param 
        disappears by unknown reason!). 
	Adds func body into implementation section.

      - real numbers

      - handling 
       #ifdef ID  to {$ifdef ID} 
       #ifundef ID  to {$ifundef ID} 
       #else to {$else}
       #define ID to {$define ID}
       #endif to {$endif}

      -"extern" fully handled . Adds proc/func + 'external _ExternalLibrary;'to
        implementation section
       you must assign _ExternalLibrary later.

      -"const" skips in func/proc arguments.
 
      changes in convert.y and scan.l
      - "convert" renamed to "h2pas"
      - Inserted the checking "IsAssigned(Pointer)" everywhere access to pointers
       It preserv from Acces Violation Errors.
      - A little remade for TP Lex and Yacc 4.01 - 
           changed function "input" to "get_char"
      -!!! because of peculiarity TPLY4.01 you must create unit CONVERU.PAS by
       your hand! Cut const definitions from CONVERT.PAS and paste into CONVERU.PAS
 
 What need 
   * handle struct a {  }; in the right way
   * all predefined C types
   * misplaced comments
   * handle functions without result
*)

  uses
   {$IFDEF WIN32}
   SysUtils,
   {$else}
   strings,
   {$endif}
   scan,converu,lexlib,yacclib;

  type
     YYSTYPE = presobject;

     tblocktype = (bt_type,bt_const,bt_var,bt_func,bt_no);

  const
     INT_STR = 'longint';
     UINT_STR = 'cardinal';
     SHORT_STR = 'integer';
     USHORT_STR = 'word';
     CHAR_STR = 'char';
     UCHAR_STR = 'char';

  var
     debug : boolean;
     hp,ph : presobject;
     block_type : tblocktype;
     extfile: text; {file for implementation headers extern procs}
     IsExtern:boolean;

{ $ define yydebug}

    procedure write_type_specifier(p : presobject; var outfile:text);forward;
    procedure write_p_a_def(p,simple_type : presobject; var outfile:text);forward;

    procedure yymsg(const msg : string);
      begin
         writeln('line ',line_no,': ',msg);
      end;

    procedure write_expr(p : presobject; var outfile:text);
      begin
      if assigned(p) then
         case p^.typ of
            t_id : write(outfile,p^.p);
            t_preop : begin
                         write(outfile,p^.p,'');
                         write_expr(p^.p1,outfile);
                         write(outfile,'');
                      flush(outfile);
                      end;
            t_bop : begin
                       write(outfile,'(');
                       write_expr(p^.p1,outfile);
                       write(outfile,')',p^.p,'(');
                       write_expr(p^.p2,outfile);
                       write(outfile,')');
                    flush(outfile);
                    end;
            else internalerror(2);
         end;
      end;

    (* writes an argument list, where p is t_arglist *)
    procedure write_args(p : presobject;  var outfile:text);
      var   para : longint;
      begin
         para:=1;
         write(outfile,'(');
         aktspace:=aktspace+'  ';

         (* walk through all arguments *)
         while assigned(p) do
           begin

              (* write new type name *)
              if assigned(p^.p1^.p2^.p2) then
                write(outfile,p^.p1^.p2^.p2^.p)
              else
                write(outfile,'_para',para);
              write(outfile,':');
              write_p_a_def(p^.p1^.p2^.p1,p^.p1^.p1,  outfile);

              (* p2 is next, this is a little bit inconsistent *)
              p:=p^.p2;
              if assigned(p) then
                write(outfile,'; ');
              inc(para);
           end;
         write(outfile,')');
         flush(outfile);
         dec(byte(aktspace[0]),2);
      end;

    procedure write_p_a_def(p,simple_type : presobject;  var outfile:text);
      begin
         if not(assigned(p)) then
           begin
              write_type_specifier(simple_type,  outfile);
              exit;
           end;
         case p^.typ of
            t_pointerdef : begin
                              (* procedure variable ? *)
                              if assigned(p^.p1) and (p^.p1^.typ=t_procdef) then
                                begin
                                   (* distinguish between procedure and function *)
                                   if (simple_type^.typ=t_void) and (p^.p1^.p1=nil) then
                                     begin
                                        write(outfile,'procedure');

                                        (* write arguments *)
                                        if assigned(p^.p1^.p2) then
                                          write_args(p^.p1^.p2,  outfile);
                                     flush(outfile);
                                     end
                                   else
                                     begin
                                        write(outfile,'function');
                                        (* write arguments *)
                                        if assigned(p^.p1^.p2) then
                                          write_args(p^.p1^.p2,  outfile);
                                        write(outfile,':');
                                        flush(outfile);
                                        write_p_a_def(p^.p1^.p1,simple_type,  outfile);
                                     end
                                end
                              else
                                begin
                                   (* generate "pointer" ? *)
                                   if (simple_type^.typ=t_void) and (p^.p1=nil) then
                                      begin
                                       write(outfile,'pointer');
                                       flush(outfile);
                                      end
                                   else
                                     begin
                                        write(outfile,'^');
                                        flush(outfile);
                                        write_p_a_def(p^.p1,simple_type,  outfile);
                                     end;
                                end;
                           end;
            t_arraydef : begin
                             write(outfile,'array[0..(');
                             write_expr(p^.p2,  outfile);
                             write(outfile,')-1] of ');
                             flush(outfile);
                             write_p_a_def(p^.p1,simple_type,  outfile);
                          end;
            else internalerror(1);
         end;
      end;

    procedure write_type_specifier(p : presobject;  var outfile:text);
      var
         hp1,hp2,hp3 : presobject;
         l,w : longint; 
      begin
         case p^.typ of
            t_id : write(outfile,p^.p);
            t_enumdef : begin
                           write(outfile,'(');
                           hp1:=p^.p1;   w:=length(aktspace);
                           while assigned(hp1) do
                             begin
                                write(outfile,hp1^.p1^.p);
                                w:=w+length(hp1^.p1^.p);
                                hp1:=hp1^.p2;
                                if assigned(hp1) then
                                  write(outfile,',');
                                if w>40 then begin 
                                  writeln(outfile);
                                  write(outfile,aktspace);
                                  end;
                                flush(outfile);
                             end;
                           write(outfile,')');
                           flush(outfile);
                        end;
            t_structdef : begin
                          writeln(outfile,'record');
                          aktspace:=aktspace+'   ';
                          hp1:=p^.p1;

                          { walk through all members }
                          while assigned(hp1) do
                            begin
                               { hp2 is t_memberdec }
                               hp2:=hp1^.p1;
                               { hp3 is t_declist }
                               hp3:=hp2^.p2;
                               while assigned(hp3) do
                                 begin
                                    write(outfile,aktspace,hp3^.p1^.p2^.p);
                                    write(outfile,' : ');
                                    aktspace:=aktspace+'  ';
                                    write_p_a_def(hp3^.p1^.p1,hp2^.p1, outfile);
                                    dec(byte(aktspace[0]),2);
                                    writeln(outfile,';');
                                    hp3:=hp3^.p2;
                                 end;
                               hp1:=hp1^.p2;
                            end;
                          dec(byte(aktspace[0]),3);
                          write(outfile,aktspace,'end');
                       flush(outfile);
                       end;
            t_uniondef : begin
                            writeln(outfile,'record');
                            writeln(outfile,aktspace,'  case longint of');
                            aktspace:=aktspace+'     ';
                            l:=0;
                            hp1:=p^.p1;

                            { walk through all members }
                            while assigned(hp1) do
                              begin
                                 { hp2 is t_memberdec }
                                 hp2:=hp1^.p1;
                                 { hp3 is t_declist }
                                 hp3:=hp2^.p2;
                                 while assigned(hp3) do
                                   begin
                                      write(outfile,aktspace,l,' : ( ');
                                      write(outfile,hp3^.p1^.p2^.p);
                                      write(outfile,':');
                                      aktspace:=aktspace+'  ';
                                      write_p_a_def(hp3^.p1^.p1,hp2^.p1, outfile);
                                      dec(byte(aktspace[0]),2);
                                      writeln(outfile,' );');
                                      hp3:=hp3^.p2;
                                      inc(l);
                                   end;
                                 hp1:=hp1^.p2;
                              end;
                            dec(byte(aktspace[0]),5);
                            write(outfile,aktspace,'end');
                           flush(outfile);
                      end;
         else internalerror(3);
         end;
      end;

    procedure write_def_params(p : presobject;  var outfile:text);
      var
         hp1,hp2,hp3 : presobject;
         l : longint;
      begin
         case p^.typ of
            t_enumdef : begin
                           hp1:=p^.p1;
                           while assigned(hp1) do
                             begin
                                write(outfile,hp1^.p1^.p);
                                hp1:=hp1^.p2;
                                if assigned(hp1) then
                                  write(outfile,',')
                                else
                                  write(outfile);
                                flush(outfile);
                             end;
                           flush(outfile);
                        end;
         else internalerror(3);
         end;
      end;



{Set up as you need!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!}
const __ExtrnlLibrary='system.dll';

%} 

%token EXTERN
%token ID TYPEDEF INT SEMICOLON STRUCT LGKLAMMER RGKLAMMER 
%token LECKKLAMMER RECKKLAMMER LKLAMMER RKLAMMER _CHAR DEFINE
%token NUMBER SHORT UNSIGNED LONG VOID UNION CSTRING ENUM _CONST
%token _SHR _SHL _OR _PLUS MINUS COMMA
%token STAR _SLASH _AND
%token _NOT 

%%

file : declaration_list
     ;

declaration_list : declaration declaration_list |
     declaration
     ;

dec_specifier : 
     EXTERN { $$:=new(presobject,init_id('extern')); }
     |{ $$:=new(presobject,init_id('intern')); }
     ;

declaration :
     dec_specifier type_specifier declarator_list SEMICOLON
     { IsExtern:=false;

    (*  writeln(outfile,'{*dec_specifier type_specifier declarator_list SEMICOLON}');

     if assigned($3) then writeln(outfile,'{*$3}');
     if assigned($3)and assigned($3.p1)
         then writeln(outfile,'{*$3.p1}');
     if assigned($3)and assigned($3.p1)and assigned($3.p1^.p1)
         then writeln(outfile,'{*$3^.p1^.p1}');
    *)

      if (assigned($3)and assigned($3.p1)and assigned($3.p1^.p1)) 
        and ($3^.p1^.p1^.typ=t_procdef) then
         begin
            IsExtern:=assigned($1)and($1^.p='extern'); 
            if block_type<>bt_func then
              writeln(outfile);

            block_type:=bt_func;
            write(outfile,aktspace);
            aktspace:=aktspace+'  ';
            if IsExtern then
             write(extfile,aktspace);

            (* distinguish between procedure and function *)
            if assigned($2) then
            if ($2^.typ=t_void) and ($3^.p1^.p1^.p1=nil) then
              begin
               write(outfile,'procedure ',$3^.p1^.p2^.p);
                 (* write arguments *)
               if assigned($3^.p1^.p1^.p2) then
                   write_args($3^.p1^.p1^.p2, outfile);
               if IsExtern then
                 begin
                 write(extfile,'procedure ',$3^.p1^.p2^.p);
                 (* write arguments *)
                 if assigned($3^.p1^.p1^.p2) then
                   write_args($3^.p1^.p1^.p2,extfile);
                 end
              end
            else
              begin
                 write(outfile,'function ',$3^.p1^.p2^.p);
                 if IsExtern then
                   write(extfile,'function ',$3^.p1^.p2^.p);

                 (* write arguments *)
                 if assigned($3^.p1^.p1^.p2) then
                   write_args($3^.p1^.p1^.p2,outfile);
                 if IsExtern then
                 if assigned($3^.p1^.p1^.p2) then
                   write_args($3^.p1^.p1^.p2, extfile);
  
                 write(outfile,':');
                 if IsExtern then
                  write(extfile,':');
                 write_p_a_def($3^.p1^.p1^.p1,$2,outfile);
                 if IsExtern then
                  write_p_a_def($3^.p1^.p1^.p1,$2,extfile);
              end;
            if IsExtern then 
             writeln(outfile,';CDECL;')
             else writeln(outfile,';');

            if IsExtern then writeln(extfile,';CDECL; external __ExtrnlLibrary;');
            dec(byte(aktspace[0]),2);
            IsExtern:=false;
         end
       else (* $3^.p1^.p1^.typ=t_procdef *)
       if assigned($3)and assigned($3.p1) then
         begin
            if block_type<>bt_var then
              begin
                 writeln(outfile);
                 writeln(outfile,aktspace,'var');
              end;
            block_type:=bt_var;

            aktspace:=aktspace+'   ';

            (* walk through all declarations *)
            hp:=$3;
            while assigned(hp) do
              begin
                 (* write new type name *)
                 if assigned(hp^.p1^.p2)and assigned(hp^.p1^.p2^.p)then
                   write(outfile,aktspace,hp^.p1^.p2^.p);
                 write(outfile,' : ');
                 aktspace:=aktspace+'  ';
                 write_p_a_def(hp^.p1^.p1,$2,outfile);
                 dec(byte(aktspace[0]),2);
                 writeln(outfile,';');
                 hp:=hp^.p2;
              end;
            dec(byte(aktspace[0]),3);
         end;
       if assigned($1)then  dispose($1,done);
       if assigned($2)then  dispose($2,done);
       if assigned($3)then  dispose($3,done);
     } |
     TYPEDEF type_specifier declarator_list SEMICOLON
     {
       if block_type<>bt_type then
         begin
            writeln(outfile);
            writeln(outfile,aktspace,'type');
         end;
       block_type:=bt_type;

       aktspace:=aktspace+'   ';

       (* walk through all declarations *)
       hp:=$3;
       while assigned(hp) do
         begin
            (* write new type name *)
            write(outfile,aktspace,hp^.p1^.p2^.p);
            write(outfile,' = ');
            aktspace:=aktspace+'  ';
            write_p_a_def(hp^.p1^.p1,$2,outfile);
            dec(byte(aktspace[0]),2);
            writeln(outfile,';');
            flush(outfile);
            hp:=hp^.p2;
         end;
       dec(byte(aktspace[0]),3);
       if assigned($2)then
       dispose($2,done);
       if assigned($3)then
       dispose($3,done);
     }|
     DEFINE dname LKLAMMER enum_list RKLAMMER expr 
     {
       writeln (outfile,'{it was #define name(param) expression}   ');
       writeln (extfile,'{it was #define name(param) expression}   ');
       aktspace:=aktspace+'   ';
       if block_type<>bt_func then
              writeln(outfile);
       block_type:=bt_func;
       write(outfile,aktspace,'function ',$2^.p,'(');
       write(extfile,aktspace,'function ',$2^.p,'(');
       
       ph:=new(presobject,init_one(t_enumdef,$4)); 
       write_def_params(ph,outfile);
       write_def_params(ph,extfile);
       if assigned(ph) then dispose(ph);

       writeln(outfile,':integer):integer;');
       writeln(extfile,':integer):integer;'); 
       write(extfile,aktspace,' begin ',$2^.p,':=(');
       flush(outfile);
       write_expr($6,extfile);
       writeln(extfile,') end;');
       dec(byte(aktspace[0]),3);
       if assigned($2)then dispose($2,done);
       if assigned($4)then dispose($4,done);
       if assigned($6)then dispose($6,done);
     }|
     DEFINE dname LKLAMMER dname RKLAMMER expr 
     {
       writeln (outfile,'{it was #define name(param) expression}   ');
       writeln (extfile,'{it was #define name(param) expression}   ');
       aktspace:=aktspace+'   ';
       if block_type<>bt_func then
              writeln(outfile);
       block_type:=bt_func;
       write(outfile,aktspace,'function ',$2^.p,'(');
       write(extfile,aktspace,'function ',$2^.p,'(');
       
       writeln(outfile,$4^.p,':integer):integer;');
       writeln(extfile,$4^.p,':integer):integer;'); 
       write(extfile,aktspace,' begin ',$2^.p,':=(');
       flush(outfile);
       write_expr($6,extfile);
       writeln(extfile,') end;');
       dec(byte(aktspace[0]),3);
       if assigned($2)then dispose($2,done);
       if assigned($4)then dispose($4,done);
       if assigned($6)then dispose($6,done);
     }|
     DEFINE dname LKLAMMER dname RKLAMMER
     {
       if block_type<>bt_const then
         begin
            writeln(outfile);
            writeln(outfile,aktspace,'const');
         end;
       block_type:=bt_const;
       aktspace:=aktspace+'   ';
       write(outfile,aktspace,$2^.p);
       write(outfile,' = ',$4^.p);
       flush(outfile);
       writeln(outfile,';');
       dec(byte(aktspace[0]),3);
       if assigned($2)then
       dispose($2,done);
       if assigned($3)then
       dispose($4,done);
     }|
     DEFINE dname expr
     {
       if block_type<>bt_const then
         begin
            writeln(outfile);
            writeln(outfile,aktspace,'const');
         end;
       block_type:=bt_const;

       aktspace:=aktspace+'   ';
       write(outfile,aktspace,$2^.p);
       write(outfile,' = ');
       flush(outfile);
       write_expr($3,outfile);
       writeln(outfile,';');
       dec(byte(aktspace[0]),3);
       if assigned($2)then
       dispose($2,done);
       if assigned($3)then
       dispose($3,done);
     }|
     DEFINE dname
     {
       writeln(outfile,'{$define ',$2^.p,'}');
       flush(outfile);
       if assigned($2)then
        dispose($2,done);
     }
     ;

type_specifier : 
     STRUCT dname LGKLAMMER member_list RGKLAMMER
     {
       writeln(line_no,' warning: struct xxx ignored');
       if assigned($2)then dispose($2);
       $$:=new(presobject,init_one(t_structdef,$4));
     } |
     STRUCT LGKLAMMER member_list RGKLAMMER
     {
       $$:=new(presobject,init_one(t_structdef,$3));
     } |
     UNION LGKLAMMER member_list RGKLAMMER
     {
       $$:=new(presobject,init_one(t_uniondef,$3));
     } |
     STRUCT simple_type_name { $$:=$2; } |
     ENUM LGKLAMMER enum_list RGKLAMMER
     {
       $$:=new(presobject,init_one(t_enumdef,$3));
     } |
     simple_type_name { $$:=$1; } 
     ;

member_list : member_list member_declaration
     {
       $$:=$1;
       hp:=$1;
       while assigned(hp^.p2) do
         hp:=hp^.p2;
       hp^.p2:=new(presobject,init_two(t_memberdeclist,$2,nil));
     } |
     member_declaration
     {
       $$:=new(presobject,init_two(t_memberdeclist,$1,nil));
     }
     ;

member_declaration : type_specifier declarator_list SEMICOLON
     {
       $$:=new(presobject,init_two(t_memberdec,$1,$2));
     }
     ;

simple_type_name : INT
     {
       $$:=new(presobject,init_id(INT_STR));
     } |
     UNSIGNED INT
     {
       $$:=new(presobject,init_id(UINT_STR));
     } |
     LONG
     {
       $$:=new(presobject,init_id(INT_STR));
     } |
     UNSIGNED
     {
       $$:=new(presobject,init_id(UINT_STR));
     } |
     UNSIGNED LONG
     {
       $$:=new(presobject,init_id(UINT_STR));
     } |
     UNSIGNED SHORT
     {
       $$:=new(presobject,init_id(USHORT_STR));
     } |
     UNSIGNED _CHAR
     {
       $$:=new(presobject,init_id(UCHAR_STR));
     } |
     VOID
     {
       $$:=new(presobject,init_no(t_void));
     } |
     SHORT
     {
       $$:=new(presobject,init_id(SHORT_STR));
     } |
     _CHAR
     {
       $$:=new(presobject,init_id(CHAR_STR));
     } |
     ID
     {
       $$:=new(presobject,init_id(act_token));
     }
     ;

declarator_list : 
     declarator_list COMMA declarator
     {
       $$:=$1;
       hp:=$1;
       while assigned(hp^.p2) do
         hp:=hp^.p2;
       hp^.p2:=new(presobject,init_two(t_declist,$3,nil));
     }|
     declarator { $$:=new(presobject,init_two(t_declist,$1,nil)); }
     ;

arg_specifier: _CONST|;

argument_declaration : arg_specifier type_specifier declarator
     {
       $$:=new(presobject,init_two(t_arg,$2,$3));
     } |
     arg_specifier type_specifier abstract_declarator
     {
       $$:=new(presobject,init_two(t_arg,$2,$3));
     }
     ;

argument_declaration_list : argument_declaration
     {
       $$:=new(presobject,init_two(t_arglist,$1,nil));
     } |
     argument_declaration_list COMMA argument_declaration
     {
       $$:=$1;
       hp:=$1;
       while assigned(hp^.p2) do
         hp:=hp^.p2;
       hp^.p2:=new(presobject,init_two(t_arglist,$3,nil));
     }
     ;

declarator :
     dname
       { 
         $$:=new(presobject,init_two(t_dec,nil,$1));
        }|
     STAR declarator
     {
       hp:=$2;
       $$:=hp;
       while assigned(hp^.p1) do
         hp:=hp^.p1;
       hp^.p1:=new(presobject,init_one(t_pointerdef,nil));
     } |
     declarator LKLAMMER argument_declaration_list RKLAMMER
     {
       hp:=$1;
       $$:=hp;
       while assigned(hp^.p1) do
         hp:=hp^.p1;
       hp^.p1:=new(presobject,init_two(t_procdef,nil,$3));
     } |
     declarator LKLAMMER RKLAMMER
     {
       (* the double declaration makes parsing easier *)

       hp:=$1;
       $$:=hp;
       while assigned(hp^.p1) do
         hp:=hp^.p1;
       hp^.p1:=new(presobject,init_two(t_procdef,nil,nil));
     } |
     declarator LKLAMMER VOID RKLAMMER
     {
       (* the double declaration makes parsing easier *)

       hp:=$1;
       $$:=hp;
       while assigned(hp^.p1) do
         hp:=hp^.p1;
       hp^.p1:=new(presobject,init_two(t_procdef,nil,nil));
     } |
     declarator LECKKLAMMER expr RECKKLAMMER
     {
       hp:=$1;
       $$:=hp;
       while assigned(hp^.p1) do
         hp:=hp^.p1;
       hp^.p1:=new(presobject,init_two(t_arraydef,nil,$3));
     } |
     LKLAMMER declarator RKLAMMER { $$:=$2; }
     ;

abstract_declarator :
     STAR abstract_declarator
     {
       hp:=$2;
       $$:=hp;
       while assigned(hp^.p1) do
         hp:=hp^.p1;
       hp^.p1:=new(presobject,init_one(t_pointerdef,nil));
     } |
     abstract_declarator LKLAMMER argument_declaration_list RKLAMMER
     {
       hp:=$1;
       $$:=hp;
       while assigned(hp^.p1) do
         hp:=hp^.p1;
       hp^.p1:=new(presobject,init_two(t_procdef,nil,$3));
     } |
     abstract_declarator LKLAMMER RKLAMMER
     {
       (* the double declaration makes parsing easier *)

       hp:=$1;
       $$:=hp;
       while assigned(hp^.p1) do
         hp:=hp^.p1;
       hp^.p1:=new(presobject,init_two(t_procdef,nil,nil));
     } |
     abstract_declarator LKLAMMER VOID RKLAMMER
     {
       (* the double declaration makes parsing easier *)

       hp:=$1;
       $$:=hp;
       while assigned(hp^.p1) do
         hp:=hp^.p1;
       hp^.p1:=new(presobject,init_two(t_procdef,nil,nil));
     } |
     abstract_declarator LECKKLAMMER expr RECKKLAMMER
     {
       hp:=$1;
       $$:=hp;
       while assigned(hp^.p1) do
         hp:=hp^.p1;
       hp^.p1:=new(presobject,init_two(t_arraydef,nil,$3));
     } |
     LKLAMMER abstract_declarator RKLAMMER
     { $$:=$2; } |
     {
       $$:=new(presobject,init_two(t_dec,nil,nil));
     }
     ;

ptr_operator : STAR
     ;

dname : ID { (*dname*)
           $$:=new(presobject,init_id(act_token));
           }
     ;

expr    : 
          shift_expr{ $$:=$1;}
        ;

shift_expr :   
          shift_expr _PLUS unary_expr	 
          { $$:=new(presobject,init_bop(' + ',$1,$3));}
	| shift_expr MINUS unary_expr	 
          { $$:=new(presobject,init_bop(' - ',$1,$3));}
	| shift_expr STAR unary_expr	 
          { $$:=new(presobject,init_bop(' * ',$1,$3));}
	| shift_expr _SLASH unary_expr	 
          { $$:=new(presobject,init_bop(' / ',$1,$3));}
	| shift_expr _OR unary_expr	 
          { $$:=new(presobject,init_bop(' or ',$1,$3));}
	| shift_expr _AND unary_expr	 
          { $$:=new(presobject,init_bop(' and ',$1,$3));}
	| shift_expr _NOT unary_expr	 
          { $$:=new(presobject,init_bop(' not ',$1,$3));}
	| shift_expr _SHL unary_expr	 
          { $$:=new(presobject,init_bop(' shl ',$1,$3));}
	| shift_expr _SHR unary_expr	 
          { $$:=new(presobject,init_bop(' shr ',$1,$3));}
        |unary_expr
        ;

unary_expr:
     ID { $$:=new(presobject,init_id(act_token)); } |
     dname| 
     LKLAMMER expr RKLAMMER {$$:=$2;}|
     CSTRING { $$:=new(presobject,init_id(''''+copy(act_token,2,length(act_token)-2)+'''')); } |
     NUMBER { $$:=new(presobject,init_id(act_token)); } |
     MINUS unary_expr { $$:=new(presobject,init_preop('-',$2)); }|
     _NOT unary_expr { $$:=new(presobject,init_preop(' not ',$2)); }
     ;



enum_list : enum_list COMMA dname
     { (*enum_list COMMA dname*)
       $$:=$1;
       hp:=$1;
       while assigned(hp^.p2) do
         hp:=hp^.p2;
       hp^.p2:=new(presobject,init_two(t_enumlist,$3,nil));
     } |
     dname {begin (*enum_list: dname*)
       $$:=new(presobject,init_two(t_enumlist,$1,nil)); 
        end;}
     ;

klaimed_enum_list: RKLAMMER enum_list LKLAMMER {$$:=$2};

%%

function yylex : Integer; 
 begin
 yylex:=scan.yylex;
 end;

var r:integer; SS:string;
begin
   debug:=true;
   aktspace:='    ';
   block_type:=bt_no;
   IsExtern:=false;
   Assign(extfile,'ext.tmp'); rewrite(extfile);
   r:=yyparse;
   writeln(outfile);
   writeln(outfile,'  implementation');
   writeln(outfile);
   writeln(outfile,'const __ExtrnlLibrary=''system.dll''; {Setup as you need!}');
   writeln(outfile);
   reset(extfile);
   while not eof(extfile) do
    begin
    readln(extfile,SS); 
    writeln(outfile,SS);
    end;

   writeln(outfile);
   writeln(outfile,'end.');
   close(extfile);
   close(outfile);

end.
