%{
unit scan;

  interface

  uses
   {$IFDEF WIN32}
   SysUtils,
   {$else}
   strings,
   {$endif}
   lexlib,yacclib;

    type
       Char=system.char;
       ttyp = (t_id,t_arraydef,t_pointerdef,t_void,t_dec,t_declist,
          t_memberdec,t_structdef,t_memberdeclist,t_procdef,t_uniondef,
          t_enumdef,t_enumlist,t_preop,t_bop,t_arg,t_arglist);

       tdtyp = (dt_id,dt_one,dt_two,dt_no,dt_uop,dt_bop);

       presobject = ^tresobject;

       tresobject = object
          typ : ttyp;
          p : pchar;
          next : presobject;
          p1,p2 : presobject;
          dtyp : tdtyp;
          constructor init_no(t : ttyp);
          constructor init_one(t : ttyp;_p1 : presobject);
          constructor init_two(t : ttyp;_p1,_p2 : presobject);
          constructor init_id(const s : string);
          constructor init_bop(const s : string;_p1,_p2 : presobject);
          constructor init_preop(const s : string;_p1 : presobject);
          destructor done;
       end;

    var
       infile : string;
       line_no : longint;
       outfile : text;
       c : char;
       aktspace : string;


    function yylex : integer;
    function act_token : string;
    procedure internalerror(i : integer);

    function strpnew(const s : string) : pchar;

  implementation
    uses converu;

    procedure internalerror(i : integer);
      begin
         writeln('Internal error ',i,' in line ',line_no);
         halt(1);
      end;

    procedure commenteof;
      begin
         writeln('unexpected EOF inside comment at line ',line_no);
      end;

    var         p : pchar;
    function strpnew(const s : string) : pchar;
      begin
         p:=StrAlloc(length(s)+1);
         strpcopy(p,s);
         strpnew:=p;
      end;

    const
       newline = #10;

    constructor tresobject.init_preop(const s : string;_p1 : presobject);
      begin
         typ:=t_preop;
         dtyp:=dt_uop;
         p:=strpnew(s);
         p1:=_p1;
         p2:=nil;
         next:=nil;
      end;

    constructor tresobject.init_bop(const s : string;_p1,_p2 : presobject);
      begin
         typ:=t_bop;
         dtyp:=dt_bop;
         p:=strpnew(s);
         p1:=_p1;
         p2:=_p2;
         next:=nil;
      end;

    constructor tresobject.init_id(const s : string);
      begin
         typ:=t_id;
         dtyp:=dt_id;
         p:=strpnew(s);
         p1:=nil;
         p2:=nil;
         next:=nil;
      end;

    constructor tresobject.init_two(t : ttyp;_p1,_p2 : presobject);
      begin
         typ:=t;
         p1:=_p1;
         p2:=_p2;
         dtyp:=dt_two;
         p:=nil;
         next:=nil;
      end;

    constructor tresobject.init_one(t : ttyp;_p1 : presobject);
      begin
         typ:=t;
         p1:=_p1;
         dtyp:=dt_one;
         p2:=nil;
         next:=nil;
         p:=nil;
      end;

    constructor tresobject.init_no(t : ttyp);
      begin
         typ:=t;
         dtyp:=dt_no;
         p:=nil;
         p1:=nil;
         p2:=nil;
         next:=nil;
      end;

    destructor tresobject.done;
      begin
         (* writeln('disposing ',byte(typ)); *)
         case dtyp of
            dt_id : if assigned(p)then strdispose(p);
            dt_one : if assigned(p1) then
                       dispose(p1,done);
            dt_two : begin
                        if assigned(p1) then
                          dispose(p1,done);
                        if assigned(p2) then
                          dispose(p2,done);
                     end;
            dt_uop : begin
                        if assigned(p1) then
                          dispose(p1,done);
                        if assigned(p) then
                          strdispose(p);
                     end;
              dt_bop : begin
                        if assigned(p1) then
                          dispose(p1,done);
                        if assigned(p2) then
                          dispose(p2,done);
                        if assigned(p) then
                          strdispose(p);
                     end;
            dt_no : ;
         end;
      end;
%}

D [0-9]
%%

"/*"                   begin
                          write(outfile,aktspace,'{');
			  repeat
			    c:=get_char;
 	  	            case c of
   			       '*' : begin
				         c:=get_char;
				         if c='/' then
                                           begin
                                              writeln(outfile,'}');
                                              flush(outfile);
                                              exit;
                                           end
                                         else
                                           begin
                                              write(outfile,' ');
                                              unget_char(c)
                                           end;
				      end;
			       newline : begin
                                            inc(line_no);
                                            writeln(outfile);
                                            write(outfile,aktspace);
                                         end;
                               #0 : commenteof;
                               else write(outfile,c);
			    end;
			  until false;
                          flush(outfile);
                        end;

"//"                   begin
                          write(outfile,aktspace,'{');
			  repeat
			    c:=get_char;
 	  	            case c of
                              newline : begin
                                        writeln(outfile,'}');
                                        flush(outfile);
                                        exit;
                                        end;
                               #0 : commenteof;
                               else write(outfile,c);
                            flush(outfile);
			    end;
			  until false;
                          flush(outfile);
                        end;
\"[^\"]*\"              return(CSTRING);
{D}*[L]?              begin
                           if yytext[length(yytext)]='L' then
                             dec(byte(yytext[0]));
                           return(NUMBER);
                        end;
"0x"[0-9A-Fa-f]*[L]?    begin
                           (* handle pre- and postfixes *)
                           if copy(yytext,1,2)='0x' then
                             begin
                                delete(yytext,1,2);
                                yytext:='$'+yytext;
                             end;
                           if yytext[length(yytext)]='L' then
                             dec(byte(yytext[0]));
                           return(NUMBER);
                        end;

{D}+(\.{D}+)?([Ee][+-]?{D}+)? 
                       begin
                       return(NUMBER);
                       end; 

"-"                     return(MINUS);
">>"                    return(_SHR);
"<<"                    return(_SHL);
"|"                    return(_OR);
"&"                   return(_AND);
"!"                   return(_NOT);
"+"                   return(_PLUS);
"/"                   return(_SLASH);
","                     return(COMMA);
"["                     return(LECKKLAMMER);
"]"                     return(RECKKLAMMER);
"("                     return(LKLAMMER);
")"                     return(RKLAMMER);
"*"                     return(STAR);
"extern"                return(EXTERN);
"void"                  return(VOID);
"#else"                 begin writeln(outfile,'{$else}');
                           flush(outfile);
                        end;
"#endif"                begin writeln(outfile,'{$endif}');
                           flush(outfile);
                        end;
"#elif"                begin
                           write(outfile,'{$elif');
			   c:=get_char;
                           while c<>newline do
                             begin write(outfile,c);c:=get_char;end;
                           writeln(outfile,'}');
                           flush(outfile);
                           inc(line_no);
                        end;
"#undef"                begin
                           write(outfile,'{$undef');
			   c:=get_char;
                           while c<>newline do
                             begin write(outfile,c);c:=get_char;end;
                           writeln(outfile,'}');
                           flush(outfile);
                           inc(line_no);
                        end;
"#include"              begin
                           write(outfile,'{$include');
			   c:=get_char;
                           while c<>newline do
                             begin write(outfile,c);c:=get_char;end;
                           writeln(outfile,'}');
                           flush(outfile);
                           inc(line_no);
                        end;
"#if"                   begin
                           write(outfile,'{$if');
			   c:=get_char;
                           while c<>newline do
                             begin write(outfile,c);c:=get_char;end;
                           writeln(outfile,'}');
                           flush(outfile);
                           inc(line_no);
                        end;
"#pragma"               begin
                           write(outfile,'#pragma');
			   c:=get_char;
                           while c<>newline do
                             begin write(outfile,c);c:=get_char;end;
                           writeln(outfile,'}');
                           flush(outfile);
                           inc(line_no);
                        end;
"#define"               return(DEFINE);
"char"                  return(_CHAR);
"union"                 return(UNION);
"enum"                  return(ENUM);
"struct"                return(STRUCT);
"{"                     return(LGKLAMMER);
"}"                     return(RGKLAMMER);
"typedef"               return(TYPEDEF);
"int"                   return(INT);
"short"                 return(SHORT);
"long"                  return(LONG);
"unsigned"              return(UNSIGNED);
"const"                 return(_CONST);
[A-Za-z_][A-Za-z0-9_]*  return(ID);
";"                     return(SEMICOLON);
[ \t\f]			;
\n                      begin
                           inc(line_no); 
                        end;
.                       begin
                           writeln('Illegal character in line ',line_no);
                           halt(1);
                        end;
%%

    function act_token : string;
      begin
         act_token:=yytext;
      end;

  begin
     if paramcount<>1 then
       begin
          writeln('usage: h2pas <file> ');
          halt(1);
       end
     else
       infile:=paramstr(1);
     line_no := 1;
     assign(yyinput,infile);
     reset(yyinput);
     assign(outfile,ChangeFileExt(paramstr(1),'.PAS'));
     rewrite(outfile);

     writeln(outfile,'unit ',ChangeFileExt(paramstr(1),''),';');
     writeln(outfile);
     writeln(outfile,'{  Automatically converted by H2PAS.EXE from '+infile);
     writeln(outfile,'   Utility is made by creator of FTN Pascal 25-28.9.96');
     writeln(outfile,'   Improvements is made by Mark A. Malakanov 22-25.5.97 }');

     writeln(outfile);
     writeln(outfile);
     writeln(outfile,'  interface');
     writeln(outfile);
  end.
