
(* lexical analyzer template (TP Lex V3.0), V1.0 3-2-91 AG *)

(* global definitions: *)
unit scan;

  interface

  uses
   {$IFDEF WIN32}
   SysUtils,
   {$else}
   strings,
   {$endif}
   lexlib,yacclib;

    type
       Char=system.char;
       ttyp = (t_id,t_arraydef,t_pointerdef,t_void,t_dec,t_declist,
          t_memberdec,t_structdef,t_memberdeclist,t_procdef,t_uniondef,
          t_enumdef,t_enumlist,t_preop,t_bop,t_arg,t_arglist);

       tdtyp = (dt_id,dt_one,dt_two,dt_no,dt_uop,dt_bop);

       presobject = ^tresobject;

       tresobject = object
          typ : ttyp;
          p : pchar;
          next : presobject;
          p1,p2 : presobject;
          dtyp : tdtyp;
          constructor init_no(t : ttyp);
          constructor init_one(t : ttyp;_p1 : presobject);
          constructor init_two(t : ttyp;_p1,_p2 : presobject);
          constructor init_id(const s : string);
          constructor init_bop(const s : string;_p1,_p2 : presobject);
          constructor init_preop(const s : string;_p1 : presobject);
          destructor done;
       end;

    var
       infile : string;
       line_no : longint;
       outfile : text;
       c : char;
       aktspace : string;


    function yylex : integer;
    function act_token : string;
    procedure internalerror(i : integer);

    function strpnew(const s : string) : pchar;

  implementation
    uses converu;

    procedure internalerror(i : integer);
      begin
         writeln('Internal error ',i,' in line ',line_no);
         halt(1);
      end;

    procedure commenteof;
      begin
         writeln('unexpected EOF inside comment at line ',line_no);
      end;

    var         p : pchar;
    function strpnew(const s : string) : pchar;
      begin
         p:=StrAlloc(length(s)+1);
         strpcopy(p,s);
         strpnew:=p;
      end;

    const
       newline = #10;

    constructor tresobject.init_preop(const s : string;_p1 : presobject);
      begin
         typ:=t_preop;
         dtyp:=dt_uop;
         p:=strpnew(s);
         p1:=_p1;
         p2:=nil;
         next:=nil;
      end;

    constructor tresobject.init_bop(const s : string;_p1,_p2 : presobject);
      begin
         typ:=t_bop;
         dtyp:=dt_bop;
         p:=strpnew(s);
         p1:=_p1;
         p2:=_p2;
         next:=nil;
      end;

    constructor tresobject.init_id(const s : string);
      begin
         typ:=t_id;
         dtyp:=dt_id;
         p:=strpnew(s);
         p1:=nil;
         p2:=nil;
         next:=nil;
      end;

    constructor tresobject.init_two(t : ttyp;_p1,_p2 : presobject);
      begin
         typ:=t;
         p1:=_p1;
         p2:=_p2;
         dtyp:=dt_two;
         p:=nil;
         next:=nil;
      end;

    constructor tresobject.init_one(t : ttyp;_p1 : presobject);
      begin
         typ:=t;
         p1:=_p1;
         dtyp:=dt_one;
         p2:=nil;
         next:=nil;
         p:=nil;
      end;

    constructor tresobject.init_no(t : ttyp);
      begin
         typ:=t;
         dtyp:=dt_no;
         p:=nil;
         p1:=nil;
         p2:=nil;
         next:=nil;
      end;

    destructor tresobject.done;
      begin
         (* writeln('disposing ',byte(typ)); *)
         case dtyp of
            dt_id : if assigned(p)then strdispose(p);
            dt_one : if assigned(p1) then
                       dispose(p1,done);
            dt_two : begin
                        if assigned(p1) then
                          dispose(p1,done);
                        if assigned(p2) then
                          dispose(p2,done);
                     end;
            dt_uop : begin
                        if assigned(p1) then
                          dispose(p1,done);
                        if assigned(p) then
                          strdispose(p);
                     end;
              dt_bop : begin
                        if assigned(p1) then
                          dispose(p1,done);
                        if assigned(p2) then
                          dispose(p2,done);
                        if assigned(p) then
                          strdispose(p);
                     end;
            dt_no : ;
         end;
      end;


function yylex : Integer;

procedure yyaction ( yyruleno : Integer );
  (* local definitions: *)

begin
  (* actions: *)
  case yyruleno of
  1:
                       begin
                          write(outfile,aktspace,'{');
			  repeat
			    c:=get_char;
 	  	            case c of
   			       '*' : begin
				         c:=get_char;
				         if c='/' then
                                           begin
                                              writeln(outfile,'}');
                                              flush(outfile);
                                              exit;
                                           end
                                         else
                                           begin
                                              write(outfile,' ');
                                              unget_char(c)
                                           end;
				      end;
			       newline : begin
                                            inc(line_no);
                                            writeln(outfile);
                                            write(outfile,aktspace);
                                         end;
                               #0 : commenteof;
                               else write(outfile,c);
			    end;
			  until false;
                          flush(outfile);
                        end;

  2:
                       begin
                          write(outfile,aktspace,'{');
			  repeat
			    c:=get_char;
 	  	            case c of
                              newline : begin
                                        writeln(outfile,'}');
                                        flush(outfile);
                                        exit;
                                        end;
                               #0 : commenteof;
                               else write(outfile,c);
                            flush(outfile);
			    end;
			  until false;
                          flush(outfile);
                        end;
  3:
                        return(CSTRING);
  4:
                      begin
                           if yytext[length(yytext)]='L' then
                             dec(byte(yytext[0]));
                           return(NUMBER);
                        end;
  5:
                        begin
                           (* handle pre- and postfixes *)
                           if copy(yytext,1,2)='0x' then
                             begin
                                delete(yytext,1,2);
                                yytext:='$'+yytext;
                             end;
                           if yytext[length(yytext)]='L' then
                             dec(byte(yytext[0]));
                           return(NUMBER);
                        end;

  6:
                              
                       begin
                       return(NUMBER);
                       end; 

  7:
                        return(MINUS);
  8:
                        return(_SHR);
  9:
                        return(_SHL);
  10:
                       return(_OR);
  11:
                      return(_AND);
  12:
                      return(_NOT);
  13:
                      return(_PLUS);
  14:
                      return(_SLASH);
  15:
                        return(COMMA);
  16:
                        return(LECKKLAMMER);
  17:
                        return(RECKKLAMMER);
  18:
                        return(LKLAMMER);
  19:
                        return(RKLAMMER);
  20:
                        return(STAR);
  21:
                        return(EXTERN);
  22:
                        return(VOID);
  23:
                        begin writeln(outfile,'{$else}');
                           flush(outfile);
                        end;
  24:
                        begin writeln(outfile,'{$endif}');
                           flush(outfile);
                        end;
  25:
                       begin
                           write(outfile,'{$elif');
			   c:=get_char;
                           while c<>newline do
                             begin write(outfile,c);c:=get_char;end;
                           writeln(outfile,'}');
                           flush(outfile);
                           inc(line_no);
                        end;
  26:
                        begin
                           write(outfile,'{$undef');
			   c:=get_char;
                           while c<>newline do
                             begin write(outfile,c);c:=get_char;end;
                           writeln(outfile,'}');
                           flush(outfile);
                           inc(line_no);
                        end;
  27:
                        begin
                           write(outfile,'{$include');
			   c:=get_char;
                           while c<>newline do
                             begin write(outfile,c);c:=get_char;end;
                           writeln(outfile,'}');
                           flush(outfile);
                           inc(line_no);
                        end;
  28:
                        begin
                           write(outfile,'{$if');
			   c:=get_char;
                           while c<>newline do
                             begin write(outfile,c);c:=get_char;end;
                           writeln(outfile,'}');
                           flush(outfile);
                           inc(line_no);
                        end;
  29:
                        begin
                           write(outfile,'#pragma');
			   c:=get_char;
                           while c<>newline do
                             begin write(outfile,c);c:=get_char;end;
                           writeln(outfile,'}');
                           flush(outfile);
                           inc(line_no);
                        end;
  30:
                        return(DEFINE);
  31:
                        return(_CHAR);
  32:
                        return(UNION);
  33:
                        return(ENUM);
  34:
                        return(STRUCT);
  35:
                        return(LGKLAMMER);
  36:
                        return(RGKLAMMER);
  37:
                        return(TYPEDEF);
  38:
                        return(INT);
  39:
                        return(SHORT);
  40:
                        return(LONG);
  41:
                        return(UNSIGNED);
  42:
                        return(_CONST);
  43:
                        return(ID);
  44:
                        return(SEMICOLON);
  45:
       			;
  46:
                        begin
                           inc(line_no); 
                        end;
  47:
                        begin
                           writeln('Illegal character in line ',line_no);
                           halt(1);
                        end;
  end;
end(*yyaction*);

(* DFA table: *)

type YYTRec = record
                cc : set of Char;
                s  : Integer;
              end;

const

yynmarks   = 148;
yynmatches = 148;
yyntrans   = 240;
yynstates  = 135;

yyk : array [1..yynmarks] of Integer = (
  { 0: }
  4,
  { 1: }
  4,
  { 2: }
  14,
  47,
  { 3: }
  47,
  { 4: }
  4,
  6,
  47,
  { 5: }
  4,
  43,
  47,
  { 6: }
  4,
  6,
  47,
  { 7: }
  7,
  47,
  { 8: }
  47,
  { 9: }
  47,
  { 10: }
  10,
  47,
  { 11: }
  11,
  47,
  { 12: }
  12,
  47,
  { 13: }
  13,
  47,
  { 14: }
  15,
  47,
  { 15: }
  16,
  47,
  { 16: }
  17,
  47,
  { 17: }
  18,
  47,
  { 18: }
  19,
  47,
  { 19: }
  20,
  47,
  { 20: }
  43,
  47,
  { 21: }
  43,
  47,
  { 22: }
  47,
  { 23: }
  43,
  47,
  { 24: }
  43,
  47,
  { 25: }
  43,
  47,
  { 26: }
  35,
  47,
  { 27: }
  36,
  47,
  { 28: }
  43,
  47,
  { 29: }
  43,
  47,
  { 30: }
  43,
  47,
  { 31: }
  43,
  47,
  { 32: }
  44,
  47,
  { 33: }
  45,
  47,
  { 34: }
  46,
  { 35: }
  47,
  { 36: }
  1,
  { 37: }
  2,
  { 38: }
  { 39: }
  3,
  { 40: }
  4,
  6,
  { 41: }
  4,
  { 42: }
  { 43: }
  { 44: }
  43,
  { 45: }
  5,
  { 46: }
  8,
  { 47: }
  9,
  { 48: }
  43,
  { 49: }
  43,
  { 50: }
  43,
  { 51: }
  { 52: }
  { 53: }
  { 54: }
  { 55: }
  { 56: }
  43,
  { 57: }
  43,
  { 58: }
  43,
  { 59: }
  43,
  { 60: }
  43,
  { 61: }
  43,
  { 62: }
  43,
  { 63: }
  43,
  { 64: }
  6,
  { 65: }
  { 66: }
  6,
  { 67: }
  5,
  { 68: }
  43,
  { 69: }
  43,
  { 70: }
  43,
  { 71: }
  { 72: }
  { 73: }
  { 74: }
  { 75: }
  28,
  { 76: }
  { 77: }
  { 78: }
  43,
  { 79: }
  43,
  { 80: }
  43,
  { 81: }
  43,
  { 82: }
  43,
  { 83: }
  43,
  { 84: }
  43,
  { 85: }
  38,
  43,
  { 86: }
  43,
  { 87: }
  43,
  { 88: }
  33,
  43,
  { 89: }
  22,
  43,
  { 90: }
  { 91: }
  { 92: }
  { 93: }
  { 94: }
  { 95: }
  { 96: }
  { 97: }
  31,
  43,
  { 98: }
  43,
  { 99: }
  43,
  { 100: }
  43,
  { 101: }
  43,
  { 102: }
  43,
  { 103: }
  43,
  { 104: }
  40,
  43,
  { 105: }
  43,
  { 106: }
  23,
  { 107: }
  25,
  { 108: }
  { 109: }
  { 110: }
  { 111: }
  { 112: }
  { 113: }
  42,
  43,
  { 114: }
  32,
  43,
  { 115: }
  43,
  { 116: }
  43,
  { 117: }
  39,
  43,
  { 118: }
  43,
  { 119: }
  21,
  43,
  { 120: }
  24,
  { 121: }
  26,
  { 122: }
  { 123: }
  { 124: }
  { 125: }
  43,
  { 126: }
  34,
  43,
  { 127: }
  43,
  { 128: }
  { 129: }
  29,
  { 130: }
  30,
  { 131: }
  43,
  { 132: }
  37,
  43,
  { 133: }
  27,
  { 134: }
  41,
  43
);

yym : array [1..yynmatches] of Integer = (
{ 0: }
  4,
{ 1: }
  4,
{ 2: }
  14,
  47,
{ 3: }
  47,
{ 4: }
  4,
  6,
  47,
{ 5: }
  4,
  43,
  47,
{ 6: }
  4,
  6,
  47,
{ 7: }
  7,
  47,
{ 8: }
  47,
{ 9: }
  47,
{ 10: }
  10,
  47,
{ 11: }
  11,
  47,
{ 12: }
  12,
  47,
{ 13: }
  13,
  47,
{ 14: }
  15,
  47,
{ 15: }
  16,
  47,
{ 16: }
  17,
  47,
{ 17: }
  18,
  47,
{ 18: }
  19,
  47,
{ 19: }
  20,
  47,
{ 20: }
  43,
  47,
{ 21: }
  43,
  47,
{ 22: }
  47,
{ 23: }
  43,
  47,
{ 24: }
  43,
  47,
{ 25: }
  43,
  47,
{ 26: }
  35,
  47,
{ 27: }
  36,
  47,
{ 28: }
  43,
  47,
{ 29: }
  43,
  47,
{ 30: }
  43,
  47,
{ 31: }
  43,
  47,
{ 32: }
  44,
  47,
{ 33: }
  45,
  47,
{ 34: }
  46,
{ 35: }
  47,
{ 36: }
  1,
{ 37: }
  2,
{ 38: }
{ 39: }
  3,
{ 40: }
  4,
  6,
{ 41: }
  4,
{ 42: }
{ 43: }
{ 44: }
  43,
{ 45: }
  5,
{ 46: }
  8,
{ 47: }
  9,
{ 48: }
  43,
{ 49: }
  43,
{ 50: }
  43,
{ 51: }
{ 52: }
{ 53: }
{ 54: }
{ 55: }
{ 56: }
  43,
{ 57: }
  43,
{ 58: }
  43,
{ 59: }
  43,
{ 60: }
  43,
{ 61: }
  43,
{ 62: }
  43,
{ 63: }
  43,
{ 64: }
  6,
{ 65: }
{ 66: }
  6,
{ 67: }
  5,
{ 68: }
  43,
{ 69: }
  43,
{ 70: }
  43,
{ 71: }
{ 72: }
{ 73: }
{ 74: }
{ 75: }
  28,
{ 76: }
{ 77: }
{ 78: }
  43,
{ 79: }
  43,
{ 80: }
  43,
{ 81: }
  43,
{ 82: }
  43,
{ 83: }
  43,
{ 84: }
  43,
{ 85: }
  38,
  43,
{ 86: }
  43,
{ 87: }
  43,
{ 88: }
  33,
  43,
{ 89: }
  22,
  43,
{ 90: }
{ 91: }
{ 92: }
{ 93: }
{ 94: }
{ 95: }
{ 96: }
{ 97: }
  31,
  43,
{ 98: }
  43,
{ 99: }
  43,
{ 100: }
  43,
{ 101: }
  43,
{ 102: }
  43,
{ 103: }
  43,
{ 104: }
  40,
  43,
{ 105: }
  43,
{ 106: }
  23,
{ 107: }
  25,
{ 108: }
{ 109: }
{ 110: }
{ 111: }
{ 112: }
{ 113: }
  42,
  43,
{ 114: }
  32,
  43,
{ 115: }
  43,
{ 116: }
  43,
{ 117: }
  39,
  43,
{ 118: }
  43,
{ 119: }
  21,
  43,
{ 120: }
  24,
{ 121: }
  26,
{ 122: }
{ 123: }
{ 124: }
{ 125: }
  43,
{ 126: }
  34,
  43,
{ 127: }
  43,
{ 128: }
{ 129: }
  29,
{ 130: }
  30,
{ 131: }
  43,
{ 132: }
  37,
  43,
{ 133: }
  27,
{ 134: }
  41,
  43
);

yyt : array [1..yyntrans] of YYTrec = (
{ 0: }
  ( cc: [ #1..#8,#11,#13..#31,'$','%','''','.',':',
            '=','?','@','\','^','`','~'..#255 ]; s: 35),
  ( cc: [ #9,#12,' ' ]; s: 33),
  ( cc: [ #10 ]; s: 34),
  ( cc: [ '!' ]; s: 12),
  ( cc: [ '"' ]; s: 3),
  ( cc: [ '#' ]; s: 22),
  ( cc: [ '&' ]; s: 11),
  ( cc: [ '(' ]; s: 17),
  ( cc: [ ')' ]; s: 18),
  ( cc: [ '*' ]; s: 19),
  ( cc: [ '+' ]; s: 13),
  ( cc: [ ',' ]; s: 14),
  ( cc: [ '-' ]; s: 7),
  ( cc: [ '/' ]; s: 2),
  ( cc: [ '0' ]; s: 6),
  ( cc: [ '1'..'9' ]; s: 4),
  ( cc: [ ';' ]; s: 32),
  ( cc: [ '<' ]; s: 9),
  ( cc: [ '>' ]; s: 8),
  ( cc: [ 'A'..'K','M'..'Z','_','a','b','d','f'..'h',
            'j','k','m'..'r','w'..'z' ]; s: 31),
  ( cc: [ 'L' ]; s: 5),
  ( cc: [ '[' ]; s: 15),
  ( cc: [ ']' ]; s: 16),
  ( cc: [ 'c' ]; s: 23),
  ( cc: [ 'e' ]; s: 20),
  ( cc: [ 'i' ]; s: 29),
  ( cc: [ 'l' ]; s: 30),
  ( cc: [ 's' ]; s: 25),
  ( cc: [ 't' ]; s: 28),
  ( cc: [ 'u' ]; s: 24),
  ( cc: [ 'v' ]; s: 21),
  ( cc: [ '{' ]; s: 26),
  ( cc: [ '|' ]; s: 10),
  ( cc: [ '}' ]; s: 27),
{ 1: }
  ( cc: [ #1..#8,#11,#13..#31,'$','%','''','.',':',
            '=','?','@','\','^','`','~'..#255 ]; s: 35),
  ( cc: [ #9,#12,' ' ]; s: 33),
  ( cc: [ #10 ]; s: 34),
  ( cc: [ '!' ]; s: 12),
  ( cc: [ '"' ]; s: 3),
  ( cc: [ '#' ]; s: 22),
  ( cc: [ '&' ]; s: 11),
  ( cc: [ '(' ]; s: 17),
  ( cc: [ ')' ]; s: 18),
  ( cc: [ '*' ]; s: 19),
  ( cc: [ '+' ]; s: 13),
  ( cc: [ ',' ]; s: 14),
  ( cc: [ '-' ]; s: 7),
  ( cc: [ '/' ]; s: 2),
  ( cc: [ '0' ]; s: 6),
  ( cc: [ '1'..'9' ]; s: 4),
  ( cc: [ ';' ]; s: 32),
  ( cc: [ '<' ]; s: 9),
  ( cc: [ '>' ]; s: 8),
  ( cc: [ 'A'..'K','M'..'Z','_','a','b','d','f'..'h',
            'j','k','m'..'r','w'..'z' ]; s: 31),
  ( cc: [ 'L' ]; s: 5),
  ( cc: [ '[' ]; s: 15),
  ( cc: [ ']' ]; s: 16),
  ( cc: [ 'c' ]; s: 23),
  ( cc: [ 'e' ]; s: 20),
  ( cc: [ 'i' ]; s: 29),
  ( cc: [ 'l' ]; s: 30),
  ( cc: [ 's' ]; s: 25),
  ( cc: [ 't' ]; s: 28),
  ( cc: [ 'u' ]; s: 24),
  ( cc: [ 'v' ]; s: 21),
  ( cc: [ '{' ]; s: 26),
  ( cc: [ '|' ]; s: 10),
  ( cc: [ '}' ]; s: 27),
{ 2: }
  ( cc: [ '*' ]; s: 36),
  ( cc: [ '/' ]; s: 37),
{ 3: }
  ( cc: [ #1..'!','#'..#255 ]; s: 38),
  ( cc: [ '"' ]; s: 39),
{ 4: }
  ( cc: [ '.' ]; s: 42),
  ( cc: [ '0'..'9' ]; s: 40),
  ( cc: [ 'E','e' ]; s: 43),
  ( cc: [ 'L' ]; s: 41),
{ 5: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'z' ]; s: 44),
{ 6: }
  ( cc: [ '.' ]; s: 42),
  ( cc: [ '0'..'9' ]; s: 40),
  ( cc: [ 'E','e' ]; s: 43),
  ( cc: [ 'L' ]; s: 41),
  ( cc: [ 'x' ]; s: 45),
{ 7: }
{ 8: }
  ( cc: [ '>' ]; s: 46),
{ 9: }
  ( cc: [ '<' ]; s: 47),
{ 10: }
{ 11: }
{ 12: }
{ 13: }
{ 14: }
{ 15: }
{ 16: }
{ 17: }
{ 18: }
{ 19: }
{ 20: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'m','o'..'w','y','z' ]; s: 44),
  ( cc: [ 'n' ]; s: 49),
  ( cc: [ 'x' ]; s: 48),
{ 21: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'n','p'..'z' ]; s: 44),
  ( cc: [ 'o' ]; s: 50),
{ 22: }
  ( cc: [ 'd' ]; s: 55),
  ( cc: [ 'e' ]; s: 51),
  ( cc: [ 'i' ]; s: 53),
  ( cc: [ 'p' ]; s: 54),
  ( cc: [ 'u' ]; s: 52),
{ 23: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'g','i'..'n','p'..'z' ]; s: 44),
  ( cc: [ 'h' ]; s: 56),
  ( cc: [ 'o' ]; s: 57),
{ 24: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'m','o'..'z' ]; s: 44),
  ( cc: [ 'n' ]; s: 58),
{ 25: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'g','i'..'s','u'..'z' ]; s: 44),
  ( cc: [ 'h' ]; s: 60),
  ( cc: [ 't' ]; s: 59),
{ 26: }
{ 27: }
{ 28: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'x','z' ]; s: 44),
  ( cc: [ 'y' ]; s: 61),
{ 29: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'m','o'..'z' ]; s: 44),
  ( cc: [ 'n' ]; s: 62),
{ 30: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'n','p'..'z' ]; s: 44),
  ( cc: [ 'o' ]; s: 63),
{ 31: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'z' ]; s: 44),
{ 32: }
{ 33: }
{ 34: }
{ 35: }
{ 36: }
{ 37: }
{ 38: }
  ( cc: [ #1..'!','#'..#255 ]; s: 38),
  ( cc: [ '"' ]; s: 39),
{ 39: }
{ 40: }
  ( cc: [ '.' ]; s: 42),
  ( cc: [ '0'..'9' ]; s: 40),
  ( cc: [ 'E','e' ]; s: 43),
  ( cc: [ 'L' ]; s: 41),
{ 41: }
{ 42: }
  ( cc: [ '0'..'9' ]; s: 64),
{ 43: }
  ( cc: [ '+','-' ]; s: 65),
  ( cc: [ '0'..'9' ]; s: 66),
{ 44: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'z' ]; s: 44),
{ 45: }
  ( cc: [ '0'..'9','A'..'F','a'..'f' ]; s: 45),
  ( cc: [ 'L' ]; s: 67),
{ 46: }
{ 47: }
{ 48: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'s','u'..'z' ]; s: 44),
  ( cc: [ 't' ]; s: 68),
{ 49: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'t','v'..'z' ]; s: 44),
  ( cc: [ 'u' ]; s: 69),
{ 50: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'h','j'..'z' ]; s: 44),
  ( cc: [ 'i' ]; s: 70),
{ 51: }
  ( cc: [ 'l' ]; s: 71),
  ( cc: [ 'n' ]; s: 72),
{ 52: }
  ( cc: [ 'n' ]; s: 73),
{ 53: }
  ( cc: [ 'f' ]; s: 75),
  ( cc: [ 'n' ]; s: 74),
{ 54: }
  ( cc: [ 'r' ]; s: 76),
{ 55: }
  ( cc: [ 'e' ]; s: 77),
{ 56: }
  ( cc: [ '0'..'9','A'..'Z','_','b'..'z' ]; s: 44),
  ( cc: [ 'a' ]; s: 78),
{ 57: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'m','o'..'z' ]; s: 44),
  ( cc: [ 'n' ]; s: 79),
{ 58: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'h','j'..'r','t'..'z' ]; s: 44),
  ( cc: [ 'i' ]; s: 80),
  ( cc: [ 's' ]; s: 81),
{ 59: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'q','s'..'z' ]; s: 44),
  ( cc: [ 'r' ]; s: 82),
{ 60: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'n','p'..'z' ]; s: 44),
  ( cc: [ 'o' ]; s: 83),
{ 61: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'o','q'..'z' ]; s: 44),
  ( cc: [ 'p' ]; s: 84),
{ 62: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'s','u'..'z' ]; s: 44),
  ( cc: [ 't' ]; s: 85),
{ 63: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'m','o'..'z' ]; s: 44),
  ( cc: [ 'n' ]; s: 86),
{ 64: }
  ( cc: [ '0'..'9' ]; s: 64),
  ( cc: [ 'E','e' ]; s: 43),
{ 65: }
  ( cc: [ '0'..'9' ]; s: 66),
{ 66: }
  ( cc: [ '0'..'9' ]; s: 66),
{ 67: }
{ 68: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'d','f'..'z' ]; s: 44),
  ( cc: [ 'e' ]; s: 87),
{ 69: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'l','n'..'z' ]; s: 44),
  ( cc: [ 'm' ]; s: 88),
{ 70: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'c','e'..'z' ]; s: 44),
  ( cc: [ 'd' ]; s: 89),
{ 71: }
  ( cc: [ 'i' ]; s: 91),
  ( cc: [ 's' ]; s: 90),
{ 72: }
  ( cc: [ 'd' ]; s: 92),
{ 73: }
  ( cc: [ 'd' ]; s: 93),
{ 74: }
  ( cc: [ 'c' ]; s: 94),
{ 75: }
{ 76: }
  ( cc: [ 'a' ]; s: 95),
{ 77: }
  ( cc: [ 'f' ]; s: 96),
{ 78: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'q','s'..'z' ]; s: 44),
  ( cc: [ 'r' ]; s: 97),
{ 79: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'r','t'..'z' ]; s: 44),
  ( cc: [ 's' ]; s: 98),
{ 80: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'n','p'..'z' ]; s: 44),
  ( cc: [ 'o' ]; s: 99),
{ 81: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'h','j'..'z' ]; s: 44),
  ( cc: [ 'i' ]; s: 100),
{ 82: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'t','v'..'z' ]; s: 44),
  ( cc: [ 'u' ]; s: 101),
{ 83: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'q','s'..'z' ]; s: 44),
  ( cc: [ 'r' ]; s: 102),
{ 84: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'d','f'..'z' ]; s: 44),
  ( cc: [ 'e' ]; s: 103),
{ 85: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'z' ]; s: 44),
{ 86: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'f','h'..'z' ]; s: 44),
  ( cc: [ 'g' ]; s: 104),
{ 87: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'q','s'..'z' ]; s: 44),
  ( cc: [ 'r' ]; s: 105),
{ 88: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'z' ]; s: 44),
{ 89: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'z' ]; s: 44),
{ 90: }
  ( cc: [ 'e' ]; s: 106),
{ 91: }
  ( cc: [ 'f' ]; s: 107),
{ 92: }
  ( cc: [ 'i' ]; s: 108),
{ 93: }
  ( cc: [ 'e' ]; s: 109),
{ 94: }
  ( cc: [ 'l' ]; s: 110),
{ 95: }
  ( cc: [ 'g' ]; s: 111),
{ 96: }
  ( cc: [ 'i' ]; s: 112),
{ 97: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'z' ]; s: 44),
{ 98: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'s','u'..'z' ]; s: 44),
  ( cc: [ 't' ]; s: 113),
{ 99: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'m','o'..'z' ]; s: 44),
  ( cc: [ 'n' ]; s: 114),
{ 100: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'f','h'..'z' ]; s: 44),
  ( cc: [ 'g' ]; s: 115),
{ 101: }
  ( cc: [ '0'..'9','A'..'Z','_','a','b','d'..'z' ]; s: 44),
  ( cc: [ 'c' ]; s: 116),
{ 102: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'s','u'..'z' ]; s: 44),
  ( cc: [ 't' ]; s: 117),
{ 103: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'c','e'..'z' ]; s: 44),
  ( cc: [ 'd' ]; s: 118),
{ 104: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'z' ]; s: 44),
{ 105: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'m','o'..'z' ]; s: 44),
  ( cc: [ 'n' ]; s: 119),
{ 106: }
{ 107: }
{ 108: }
  ( cc: [ 'f' ]; s: 120),
{ 109: }
  ( cc: [ 'f' ]; s: 121),
{ 110: }
  ( cc: [ 'u' ]; s: 122),
{ 111: }
  ( cc: [ 'm' ]; s: 123),
{ 112: }
  ( cc: [ 'n' ]; s: 124),
{ 113: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'z' ]; s: 44),
{ 114: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'z' ]; s: 44),
{ 115: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'m','o'..'z' ]; s: 44),
  ( cc: [ 'n' ]; s: 125),
{ 116: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'s','u'..'z' ]; s: 44),
  ( cc: [ 't' ]; s: 126),
{ 117: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'z' ]; s: 44),
{ 118: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'d','f'..'z' ]; s: 44),
  ( cc: [ 'e' ]; s: 127),
{ 119: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'z' ]; s: 44),
{ 120: }
{ 121: }
{ 122: }
  ( cc: [ 'd' ]; s: 128),
{ 123: }
  ( cc: [ 'a' ]; s: 129),
{ 124: }
  ( cc: [ 'e' ]; s: 130),
{ 125: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'d','f'..'z' ]; s: 44),
  ( cc: [ 'e' ]; s: 131),
{ 126: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'z' ]; s: 44),
{ 127: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'e','g'..'z' ]; s: 44),
  ( cc: [ 'f' ]; s: 132),
{ 128: }
  ( cc: [ 'e' ]; s: 133),
{ 129: }
{ 130: }
{ 131: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'c','e'..'z' ]; s: 44),
  ( cc: [ 'd' ]; s: 134),
{ 132: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'z' ]; s: 44),
{ 133: }
{ 134: }
  ( cc: [ '0'..'9','A'..'Z','_','a'..'z' ]; s: 44)
);

yykl : array [0..yynstates-1] of Integer = (
{ 0: } 1,
{ 1: } 2,
{ 2: } 3,
{ 3: } 5,
{ 4: } 6,
{ 5: } 9,
{ 6: } 12,
{ 7: } 15,
{ 8: } 17,
{ 9: } 18,
{ 10: } 19,
{ 11: } 21,
{ 12: } 23,
{ 13: } 25,
{ 14: } 27,
{ 15: } 29,
{ 16: } 31,
{ 17: } 33,
{ 18: } 35,
{ 19: } 37,
{ 20: } 39,
{ 21: } 41,
{ 22: } 43,
{ 23: } 44,
{ 24: } 46,
{ 25: } 48,
{ 26: } 50,
{ 27: } 52,
{ 28: } 54,
{ 29: } 56,
{ 30: } 58,
{ 31: } 60,
{ 32: } 62,
{ 33: } 64,
{ 34: } 66,
{ 35: } 67,
{ 36: } 68,
{ 37: } 69,
{ 38: } 70,
{ 39: } 70,
{ 40: } 71,
{ 41: } 73,
{ 42: } 74,
{ 43: } 74,
{ 44: } 74,
{ 45: } 75,
{ 46: } 76,
{ 47: } 77,
{ 48: } 78,
{ 49: } 79,
{ 50: } 80,
{ 51: } 81,
{ 52: } 81,
{ 53: } 81,
{ 54: } 81,
{ 55: } 81,
{ 56: } 81,
{ 57: } 82,
{ 58: } 83,
{ 59: } 84,
{ 60: } 85,
{ 61: } 86,
{ 62: } 87,
{ 63: } 88,
{ 64: } 89,
{ 65: } 90,
{ 66: } 90,
{ 67: } 91,
{ 68: } 92,
{ 69: } 93,
{ 70: } 94,
{ 71: } 95,
{ 72: } 95,
{ 73: } 95,
{ 74: } 95,
{ 75: } 95,
{ 76: } 96,
{ 77: } 96,
{ 78: } 96,
{ 79: } 97,
{ 80: } 98,
{ 81: } 99,
{ 82: } 100,
{ 83: } 101,
{ 84: } 102,
{ 85: } 103,
{ 86: } 105,
{ 87: } 106,
{ 88: } 107,
{ 89: } 109,
{ 90: } 111,
{ 91: } 111,
{ 92: } 111,
{ 93: } 111,
{ 94: } 111,
{ 95: } 111,
{ 96: } 111,
{ 97: } 111,
{ 98: } 113,
{ 99: } 114,
{ 100: } 115,
{ 101: } 116,
{ 102: } 117,
{ 103: } 118,
{ 104: } 119,
{ 105: } 121,
{ 106: } 122,
{ 107: } 123,
{ 108: } 124,
{ 109: } 124,
{ 110: } 124,
{ 111: } 124,
{ 112: } 124,
{ 113: } 124,
{ 114: } 126,
{ 115: } 128,
{ 116: } 129,
{ 117: } 130,
{ 118: } 132,
{ 119: } 133,
{ 120: } 135,
{ 121: } 136,
{ 122: } 137,
{ 123: } 137,
{ 124: } 137,
{ 125: } 137,
{ 126: } 138,
{ 127: } 140,
{ 128: } 141,
{ 129: } 141,
{ 130: } 142,
{ 131: } 143,
{ 132: } 144,
{ 133: } 146,
{ 134: } 147
);

yykh : array [0..yynstates-1] of Integer = (
{ 0: } 1,
{ 1: } 2,
{ 2: } 4,
{ 3: } 5,
{ 4: } 8,
{ 5: } 11,
{ 6: } 14,
{ 7: } 16,
{ 8: } 17,
{ 9: } 18,
{ 10: } 20,
{ 11: } 22,
{ 12: } 24,
{ 13: } 26,
{ 14: } 28,
{ 15: } 30,
{ 16: } 32,
{ 17: } 34,
{ 18: } 36,
{ 19: } 38,
{ 20: } 40,
{ 21: } 42,
{ 22: } 43,
{ 23: } 45,
{ 24: } 47,
{ 25: } 49,
{ 26: } 51,
{ 27: } 53,
{ 28: } 55,
{ 29: } 57,
{ 30: } 59,
{ 31: } 61,
{ 32: } 63,
{ 33: } 65,
{ 34: } 66,
{ 35: } 67,
{ 36: } 68,
{ 37: } 69,
{ 38: } 69,
{ 39: } 70,
{ 40: } 72,
{ 41: } 73,
{ 42: } 73,
{ 43: } 73,
{ 44: } 74,
{ 45: } 75,
{ 46: } 76,
{ 47: } 77,
{ 48: } 78,
{ 49: } 79,
{ 50: } 80,
{ 51: } 80,
{ 52: } 80,
{ 53: } 80,
{ 54: } 80,
{ 55: } 80,
{ 56: } 81,
{ 57: } 82,
{ 58: } 83,
{ 59: } 84,
{ 60: } 85,
{ 61: } 86,
{ 62: } 87,
{ 63: } 88,
{ 64: } 89,
{ 65: } 89,
{ 66: } 90,
{ 67: } 91,
{ 68: } 92,
{ 69: } 93,
{ 70: } 94,
{ 71: } 94,
{ 72: } 94,
{ 73: } 94,
{ 74: } 94,
{ 75: } 95,
{ 76: } 95,
{ 77: } 95,
{ 78: } 96,
{ 79: } 97,
{ 80: } 98,
{ 81: } 99,
{ 82: } 100,
{ 83: } 101,
{ 84: } 102,
{ 85: } 104,
{ 86: } 105,
{ 87: } 106,
{ 88: } 108,
{ 89: } 110,
{ 90: } 110,
{ 91: } 110,
{ 92: } 110,
{ 93: } 110,
{ 94: } 110,
{ 95: } 110,
{ 96: } 110,
{ 97: } 112,
{ 98: } 113,
{ 99: } 114,
{ 100: } 115,
{ 101: } 116,
{ 102: } 117,
{ 103: } 118,
{ 104: } 120,
{ 105: } 121,
{ 106: } 122,
{ 107: } 123,
{ 108: } 123,
{ 109: } 123,
{ 110: } 123,
{ 111: } 123,
{ 112: } 123,
{ 113: } 125,
{ 114: } 127,
{ 115: } 128,
{ 116: } 129,
{ 117: } 131,
{ 118: } 132,
{ 119: } 134,
{ 120: } 135,
{ 121: } 136,
{ 122: } 136,
{ 123: } 136,
{ 124: } 136,
{ 125: } 137,
{ 126: } 139,
{ 127: } 140,
{ 128: } 140,
{ 129: } 141,
{ 130: } 142,
{ 131: } 143,
{ 132: } 145,
{ 133: } 146,
{ 134: } 148
);

yyml : array [0..yynstates-1] of Integer = (
{ 0: } 1,
{ 1: } 2,
{ 2: } 3,
{ 3: } 5,
{ 4: } 6,
{ 5: } 9,
{ 6: } 12,
{ 7: } 15,
{ 8: } 17,
{ 9: } 18,
{ 10: } 19,
{ 11: } 21,
{ 12: } 23,
{ 13: } 25,
{ 14: } 27,
{ 15: } 29,
{ 16: } 31,
{ 17: } 33,
{ 18: } 35,
{ 19: } 37,
{ 20: } 39,
{ 21: } 41,
{ 22: } 43,
{ 23: } 44,
{ 24: } 46,
{ 25: } 48,
{ 26: } 50,
{ 27: } 52,
{ 28: } 54,
{ 29: } 56,
{ 30: } 58,
{ 31: } 60,
{ 32: } 62,
{ 33: } 64,
{ 34: } 66,
{ 35: } 67,
{ 36: } 68,
{ 37: } 69,
{ 38: } 70,
{ 39: } 70,
{ 40: } 71,
{ 41: } 73,
{ 42: } 74,
{ 43: } 74,
{ 44: } 74,
{ 45: } 75,
{ 46: } 76,
{ 47: } 77,
{ 48: } 78,
{ 49: } 79,
{ 50: } 80,
{ 51: } 81,
{ 52: } 81,
{ 53: } 81,
{ 54: } 81,
{ 55: } 81,
{ 56: } 81,
{ 57: } 82,
{ 58: } 83,
{ 59: } 84,
{ 60: } 85,
{ 61: } 86,
{ 62: } 87,
{ 63: } 88,
{ 64: } 89,
{ 65: } 90,
{ 66: } 90,
{ 67: } 91,
{ 68: } 92,
{ 69: } 93,
{ 70: } 94,
{ 71: } 95,
{ 72: } 95,
{ 73: } 95,
{ 74: } 95,
{ 75: } 95,
{ 76: } 96,
{ 77: } 96,
{ 78: } 96,
{ 79: } 97,
{ 80: } 98,
{ 81: } 99,
{ 82: } 100,
{ 83: } 101,
{ 84: } 102,
{ 85: } 103,
{ 86: } 105,
{ 87: } 106,
{ 88: } 107,
{ 89: } 109,
{ 90: } 111,
{ 91: } 111,
{ 92: } 111,
{ 93: } 111,
{ 94: } 111,
{ 95: } 111,
{ 96: } 111,
{ 97: } 111,
{ 98: } 113,
{ 99: } 114,
{ 100: } 115,
{ 101: } 116,
{ 102: } 117,
{ 103: } 118,
{ 104: } 119,
{ 105: } 121,
{ 106: } 122,
{ 107: } 123,
{ 108: } 124,
{ 109: } 124,
{ 110: } 124,
{ 111: } 124,
{ 112: } 124,
{ 113: } 124,
{ 114: } 126,
{ 115: } 128,
{ 116: } 129,
{ 117: } 130,
{ 118: } 132,
{ 119: } 133,
{ 120: } 135,
{ 121: } 136,
{ 122: } 137,
{ 123: } 137,
{ 124: } 137,
{ 125: } 137,
{ 126: } 138,
{ 127: } 140,
{ 128: } 141,
{ 129: } 141,
{ 130: } 142,
{ 131: } 143,
{ 132: } 144,
{ 133: } 146,
{ 134: } 147
);

yymh : array [0..yynstates-1] of Integer = (
{ 0: } 1,
{ 1: } 2,
{ 2: } 4,
{ 3: } 5,
{ 4: } 8,
{ 5: } 11,
{ 6: } 14,
{ 7: } 16,
{ 8: } 17,
{ 9: } 18,
{ 10: } 20,
{ 11: } 22,
{ 12: } 24,
{ 13: } 26,
{ 14: } 28,
{ 15: } 30,
{ 16: } 32,
{ 17: } 34,
{ 18: } 36,
{ 19: } 38,
{ 20: } 40,
{ 21: } 42,
{ 22: } 43,
{ 23: } 45,
{ 24: } 47,
{ 25: } 49,
{ 26: } 51,
{ 27: } 53,
{ 28: } 55,
{ 29: } 57,
{ 30: } 59,
{ 31: } 61,
{ 32: } 63,
{ 33: } 65,
{ 34: } 66,
{ 35: } 67,
{ 36: } 68,
{ 37: } 69,
{ 38: } 69,
{ 39: } 70,
{ 40: } 72,
{ 41: } 73,
{ 42: } 73,
{ 43: } 73,
{ 44: } 74,
{ 45: } 75,
{ 46: } 76,
{ 47: } 77,
{ 48: } 78,
{ 49: } 79,
{ 50: } 80,
{ 51: } 80,
{ 52: } 80,
{ 53: } 80,
{ 54: } 80,
{ 55: } 80,
{ 56: } 81,
{ 57: } 82,
{ 58: } 83,
{ 59: } 84,
{ 60: } 85,
{ 61: } 86,
{ 62: } 87,
{ 63: } 88,
{ 64: } 89,
{ 65: } 89,
{ 66: } 90,
{ 67: } 91,
{ 68: } 92,
{ 69: } 93,
{ 70: } 94,
{ 71: } 94,
{ 72: } 94,
{ 73: } 94,
{ 74: } 94,
{ 75: } 95,
{ 76: } 95,
{ 77: } 95,
{ 78: } 96,
{ 79: } 97,
{ 80: } 98,
{ 81: } 99,
{ 82: } 100,
{ 83: } 101,
{ 84: } 102,
{ 85: } 104,
{ 86: } 105,
{ 87: } 106,
{ 88: } 108,
{ 89: } 110,
{ 90: } 110,
{ 91: } 110,
{ 92: } 110,
{ 93: } 110,
{ 94: } 110,
{ 95: } 110,
{ 96: } 110,
{ 97: } 112,
{ 98: } 113,
{ 99: } 114,
{ 100: } 115,
{ 101: } 116,
{ 102: } 117,
{ 103: } 118,
{ 104: } 120,
{ 105: } 121,
{ 106: } 122,
{ 107: } 123,
{ 108: } 123,
{ 109: } 123,
{ 110: } 123,
{ 111: } 123,
{ 112: } 123,
{ 113: } 125,
{ 114: } 127,
{ 115: } 128,
{ 116: } 129,
{ 117: } 131,
{ 118: } 132,
{ 119: } 134,
{ 120: } 135,
{ 121: } 136,
{ 122: } 136,
{ 123: } 136,
{ 124: } 136,
{ 125: } 137,
{ 126: } 139,
{ 127: } 140,
{ 128: } 140,
{ 129: } 141,
{ 130: } 142,
{ 131: } 143,
{ 132: } 145,
{ 133: } 146,
{ 134: } 148
);

yytl : array [0..yynstates-1] of Integer = (
{ 0: } 1,
{ 1: } 35,
{ 2: } 69,
{ 3: } 71,
{ 4: } 73,
{ 5: } 77,
{ 6: } 78,
{ 7: } 83,
{ 8: } 83,
{ 9: } 84,
{ 10: } 85,
{ 11: } 85,
{ 12: } 85,
{ 13: } 85,
{ 14: } 85,
{ 15: } 85,
{ 16: } 85,
{ 17: } 85,
{ 18: } 85,
{ 19: } 85,
{ 20: } 85,
{ 21: } 88,
{ 22: } 90,
{ 23: } 95,
{ 24: } 98,
{ 25: } 100,
{ 26: } 103,
{ 27: } 103,
{ 28: } 103,
{ 29: } 105,
{ 30: } 107,
{ 31: } 109,
{ 32: } 110,
{ 33: } 110,
{ 34: } 110,
{ 35: } 110,
{ 36: } 110,
{ 37: } 110,
{ 38: } 110,
{ 39: } 112,
{ 40: } 112,
{ 41: } 116,
{ 42: } 116,
{ 43: } 117,
{ 44: } 119,
{ 45: } 120,
{ 46: } 122,
{ 47: } 122,
{ 48: } 122,
{ 49: } 124,
{ 50: } 126,
{ 51: } 128,
{ 52: } 130,
{ 53: } 131,
{ 54: } 133,
{ 55: } 134,
{ 56: } 135,
{ 57: } 137,
{ 58: } 139,
{ 59: } 142,
{ 60: } 144,
{ 61: } 146,
{ 62: } 148,
{ 63: } 150,
{ 64: } 152,
{ 65: } 154,
{ 66: } 155,
{ 67: } 156,
{ 68: } 156,
{ 69: } 158,
{ 70: } 160,
{ 71: } 162,
{ 72: } 164,
{ 73: } 165,
{ 74: } 166,
{ 75: } 167,
{ 76: } 167,
{ 77: } 168,
{ 78: } 169,
{ 79: } 171,
{ 80: } 173,
{ 81: } 175,
{ 82: } 177,
{ 83: } 179,
{ 84: } 181,
{ 85: } 183,
{ 86: } 184,
{ 87: } 186,
{ 88: } 188,
{ 89: } 189,
{ 90: } 190,
{ 91: } 191,
{ 92: } 192,
{ 93: } 193,
{ 94: } 194,
{ 95: } 195,
{ 96: } 196,
{ 97: } 197,
{ 98: } 198,
{ 99: } 200,
{ 100: } 202,
{ 101: } 204,
{ 102: } 206,
{ 103: } 208,
{ 104: } 210,
{ 105: } 211,
{ 106: } 213,
{ 107: } 213,
{ 108: } 213,
{ 109: } 214,
{ 110: } 215,
{ 111: } 216,
{ 112: } 217,
{ 113: } 218,
{ 114: } 219,
{ 115: } 220,
{ 116: } 222,
{ 117: } 224,
{ 118: } 225,
{ 119: } 227,
{ 120: } 228,
{ 121: } 228,
{ 122: } 228,
{ 123: } 229,
{ 124: } 230,
{ 125: } 231,
{ 126: } 233,
{ 127: } 234,
{ 128: } 236,
{ 129: } 237,
{ 130: } 237,
{ 131: } 237,
{ 132: } 239,
{ 133: } 240,
{ 134: } 240
);

yyth : array [0..yynstates-1] of Integer = (
{ 0: } 34,
{ 1: } 68,
{ 2: } 70,
{ 3: } 72,
{ 4: } 76,
{ 5: } 77,
{ 6: } 82,
{ 7: } 82,
{ 8: } 83,
{ 9: } 84,
{ 10: } 84,
{ 11: } 84,
{ 12: } 84,
{ 13: } 84,
{ 14: } 84,
{ 15: } 84,
{ 16: } 84,
{ 17: } 84,
{ 18: } 84,
{ 19: } 84,
{ 20: } 87,
{ 21: } 89,
{ 22: } 94,
{ 23: } 97,
{ 24: } 99,
{ 25: } 102,
{ 26: } 102,
{ 27: } 102,
{ 28: } 104,
{ 29: } 106,
{ 30: } 108,
{ 31: } 109,
{ 32: } 109,
{ 33: } 109,
{ 34: } 109,
{ 35: } 109,
{ 36: } 109,
{ 37: } 109,
{ 38: } 111,
{ 39: } 111,
{ 40: } 115,
{ 41: } 115,
{ 42: } 116,
{ 43: } 118,
{ 44: } 119,
{ 45: } 121,
{ 46: } 121,
{ 47: } 121,
{ 48: } 123,
{ 49: } 125,
{ 50: } 127,
{ 51: } 129,
{ 52: } 130,
{ 53: } 132,
{ 54: } 133,
{ 55: } 134,
{ 56: } 136,
{ 57: } 138,
{ 58: } 141,
{ 59: } 143,
{ 60: } 145,
{ 61: } 147,
{ 62: } 149,
{ 63: } 151,
{ 64: } 153,
{ 65: } 154,
{ 66: } 155,
{ 67: } 155,
{ 68: } 157,
{ 69: } 159,
{ 70: } 161,
{ 71: } 163,
{ 72: } 164,
{ 73: } 165,
{ 74: } 166,
{ 75: } 166,
{ 76: } 167,
{ 77: } 168,
{ 78: } 170,
{ 79: } 172,
{ 80: } 174,
{ 81: } 176,
{ 82: } 178,
{ 83: } 180,
{ 84: } 182,
{ 85: } 183,
{ 86: } 185,
{ 87: } 187,
{ 88: } 188,
{ 89: } 189,
{ 90: } 190,
{ 91: } 191,
{ 92: } 192,
{ 93: } 193,
{ 94: } 194,
{ 95: } 195,
{ 96: } 196,
{ 97: } 197,
{ 98: } 199,
{ 99: } 201,
{ 100: } 203,
{ 101: } 205,
{ 102: } 207,
{ 103: } 209,
{ 104: } 210,
{ 105: } 212,
{ 106: } 212,
{ 107: } 212,
{ 108: } 213,
{ 109: } 214,
{ 110: } 215,
{ 111: } 216,
{ 112: } 217,
{ 113: } 218,
{ 114: } 219,
{ 115: } 221,
{ 116: } 223,
{ 117: } 224,
{ 118: } 226,
{ 119: } 227,
{ 120: } 227,
{ 121: } 227,
{ 122: } 228,
{ 123: } 229,
{ 124: } 230,
{ 125: } 232,
{ 126: } 233,
{ 127: } 235,
{ 128: } 236,
{ 129: } 236,
{ 130: } 236,
{ 131: } 238,
{ 132: } 239,
{ 133: } 239,
{ 134: } 240
);


var yyn : Integer;

label start, scan, action;

begin

start:

  (* initialize: *)

  yynew;

scan:

  (* mark positions and matches: *)

  for yyn := yykl[yystate] to     yykh[yystate] do yymark(yyk[yyn]);
  for yyn := yymh[yystate] downto yyml[yystate] do yymatch(yym[yyn]);

  if yytl[yystate]>yyth[yystate] then goto action; (* dead state *)

  (* get next character: *)

  yyscan;

  (* determine action: *)

  yyn := yytl[yystate];
  while (yyn<=yyth[yystate]) and not (yyactchar in yyt[yyn].cc) do inc(yyn);
  if yyn>yyth[yystate] then goto action;
    (* no transition on yyactchar in this state *)

  (* switch to new state: *)

  yystate := yyt[yyn].s;

  goto scan;

action:

  (* execute action: *)

  if yyfind(yyrule) then
    begin
      yyaction(yyrule);
      if yyreject then goto action;
    end
  else if not yydefault and yywrap then
    begin
      yyclear;
      return(0);
    end;

  if not yydone then goto start;

  yylex := yyretval;

end(*yylex*);



    function act_token : string;
      begin
         act_token:=yytext;
      end;

Function ForceExtension(Const HStr,ext:String):String;
{
  Return a filename which certainly has the extension ext
  (no dot in ext !!)
}
var
  j : longint;
begin
  j:=length(Hstr);
  while (j>0) and (Hstr[j]<>'.') do
   dec(j);
  if j=0 then
   j:=255;
  ForceExtension:=Copy(Hstr,1,j-1)+'.'+Ext;
end;


  begin
     if paramcount<>1 then
       begin
          writeln('usage: h2pas <file> ');
          halt(1);
       end
     else
       infile:=paramstr(1);
     line_no := 1;
     assign(yyinput,infile);
     reset(yyinput);
     assign(outfile,ForceExtension(paramstr(1),'pp'));
     rewrite(outfile);

     writeln(outfile,'unit ',ForceExtension(paramstr(1),''),';');
     writeln(outfile);
     writeln(outfile,'{  Automatically converted by H2PAS.EXE from '+infile);
     writeln(outfile,'   Utility is made by creator of FTN Pascal 25-28.9.96');
     writeln(outfile,'   Improvements is made by Mark A. Malakanov 22-25.5.97 }');

     writeln(outfile);
     writeln(outfile);
     writeln(outfile,'  interface');
     writeln(outfile);
  end.
