{
    $Id: mkdep.pp,v 1.2 1998/03/27 18:39:20 peter Exp $
    This program is part of the Free Pascal run time library.
    Copyright (c) 1997 by Peter Vreman
    member of the Free Pascal development team

    See the file COPYING.FPC, included in this distribution,
    for details about the copyright.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

 **********************************************************************}

{ Program to create a depend makefile for a program with multiple units }

program mkdep;
uses Dos;

{.$define debug}

const
{$ifdef linux}
  exeext='';
{$else}
  exeext='.EXE';
{$endif}

type
  PUses=^TUses;
  TUses=record
    Name : string[32];
    Next : PUses;
  end;
  PUnit=^TUnit;
  TUnit=record
    UsesList : PUses;
    PasFn,
    Name     : string[32];
    IsUnit   : boolean;
    Next     : PUnit;
  end;
  PDefine=^TDefine;
  TDefine = Record
    Name : String[32];
    Next : PDefine;
    end;


var
  UnitList         : PUnit;
  Define           : PDefine;
  ParaFile         : string;
  Verbose,addcall  : boolean;
  CallLine,
  OutFile          : String;


{****************************************************************************
                           Handy Routines
****************************************************************************}

function UCase(Const Hstr:string):string;
var
  i : longint;
begin
  for i:=1to Length(Hstr) do
   UCase[i]:=Upcase(Hstr[i]);
  UCase[0]:=chr(Length(Hstr));
end;


function FixFn(const s:string):string;
var
  i      : longint;
  NoPath : boolean;
begin
  NoPath:=true;
  for i:=length(s) downto 1 do
   begin
     case s[i] of
 {$ifdef Linux}
  '/','\' : begin
              FixFn[i]:='/';
              NoPath:=false; {Skip lowercasing path: 'X11'<>'x11' }
            end;
 'A'..'Z' : if NoPath then
             FixFn[i]:=char(byte(s[i])+32)
            else
             FixFn[i]:=s[i];
 {$else}
      '/' : FixFn[i]:='\';
 'a'..'z' : FixFn[i]:=char(byte(s[i])-32);
 {$endif}
     else
      FixFn[i]:=s[i];
     end;
   end;
  FixFn[0]:=Chr(Length(s));
end;


{****************************************************************************
                             Main Program
****************************************************************************}

Function SearchPas(const fn:string):string;
var
  Dir : SearchRec;
begin
  FindFirst(FixFn(fn+'.PP'),$20,Dir);
  if Doserror=0 then
   SearchPas:=FixFn(fn+'.PP')
  else
   SearchPas:=FixFn(fn+'.PAS')
end;


Function UnitDone(const fn:string):boolean;
var
  hp : PUnit;
begin
  hp:=UnitList;
  while not (hp=nil) do
   begin
     if hp^.Name=fn then
      begin
        UnitDone:=true;
        exit;
      end;
     hp:=hp^.Next;
   end;
  UnitDone:=false;
end;


Function CheckDefine(const s:string):boolean;

var
  ss : string[32];
  P : PDefine;

begin
  ss:=ucase(s);
  P:=Define;
  while (p<>Nil) do
   begin
     if ss=p^.name then
      begin
        CheckDefine:=true;
        exit;
      end;
     P:=P^.Next;
   end;
  CheckDefine:=false;
end;

Procedure AddDefine(Const S : String);

Var P : PDefine;

begin
  New(P);
  P^.Name:=Ucase(S);
  P^.Next:=Define;
  Define:=P;
end;

procedure RemoveSep(var fn:string);
var
  i : longint;
begin
  i:=0;
  while (i<length(fn)) and (fn[i+1] in [',',' ',#9]) do
   inc(i);
  Delete(fn,1,i);
end;


function GetName(var fn:string):string;
var
  i : longint;
begin
  i:=0;
  while (i<length(fn)) and (fn[i+1] in ['A'..'Z','0'..'9','_','-']) do
   inc(i);
  GetName:=Copy(fn,1,i);
  Delete(fn,1,i);
end;


procedure ListDepend(const fn:string);
const
  MaxLevel=16;
var
  f  : text;
  hs : ^string;
  curruses,lastuses : PUses;
  currunit,lastunit : PUnit;
  i,j : longint;
  UsesDone,
  Done,Comment,
  InImplementation : boolean;
  Skip : array[0..MaxLevel] of boolean;
  Level : byte;
begin
  if UnitDone(fn) then
   exit;
  new(hs);
  new(currunit);
  currunit^.next:=nil;
  currunit^.Name:=fn;
  currunit^.IsUnit:=true;
  currunit^.PasFn:=SearchPas(fn);
  currunit^.useslist:=nil;
  assign(f,currunit^.PasFn);
  {$I-}
   reset(f);
  {$I+}
  if ioresult=0 then
   begin
     if verbose then
      Writeln('Processing ',currunit^.PasFn);
   {Add to Linked List}
     if unitlist=nil then
      unitlist:=currunit
     else
      begin
        lastunit:=UnitList;
        while not (lastunit^.Next=nil) do
         lastunit:=lastunit^.next;
        lastunit^.next:=currunit;
      end;
   {Parse file}
     InImplementation:=false;
     done:=false;
     usesdone:=true;
     Comment:=false;
     FillChar(skip,sizeof(Skip),0);
     hs^:='';
     Level:=0;
     while (not done) and (not Eof(f)) do
      begin
        repeat
          if hs^='' then
           begin
             ReadLn(f,hs^);
             hs^:=UCase(hs^);
           end;
          RemoveSep(hs^);
        until (hs^<>'') or Eof(f);
        if Comment then
         begin
           i:=pos('}',hs^);
           if (i>0) then
            begin
              j:=pos('{',hs^);
              if (j>0) and (j<i) then
               begin
                 Comment:=true;
                 Delete(hs^,1,j-1);
               end
              else
               begin
                 Comment:=false;
                 Delete(hs^,1,i-1);
               end;
            end
           else
            hs^:='';
         end;
        if (hs^<>'') then
         begin
           case hs^[1] of
            '}' : begin
                    Comment:=false;
                    hs^:='';
                  end;
            '{' : begin
                    if (Copy(hs^,2,6)='$IFDEF') then
                     begin
                       Delete(hs^,1,7);
                       RemoveSep(hs^);
                       inc(Level);
                       if Level>=MaxLevel then
                        begin
                          Writeln('Too many IF(N)DEFs');
                          Halt(1);
                        end;
                       skip[level]:=skip[level-1] or (not CheckDefine(GetName(hs^)));
                     end
                    else
                     if (Copy(hs^,2,7)='$IFNDEF') then
                      begin
                        Delete(hs^,1,7);
                        RemoveSep(hs^);
                        inc(Level);
                        if Level>=MaxLevel then
                         begin
                           Writeln('Too many IF(N)DEFs');
                           Halt(1);
                         end;
                        skip[level]:=skip[level-1] or (CheckDefine(GetName(hs^)));
                      end
                    else
                     if (Copy(hs^,2,6)='$ELSE') then
                      begin
                        skip[level]:=skip[level-1] or (not skip[level]);
                      end
                    else
                     if (Copy(hs^,2,6)='$ENDIF') then
                      begin
                        skip[level]:=false;
                        if Level=0 then
                         begin
                           Writeln('Too many ENDIFs');
                           Halt(1);
                         end;
                        dec(level);
                      end
                    else
                     if (Copy(hs^,2,6)='$IFOPT') then
                      begin
                        inc(Level);
                        if Level>=MaxLevel then
                         begin
                           Writeln('Too many IF(N)DEFs');
                           Halt(1);
                         end;
                        skip[level]:=true;
                      end
                    else
                     Comment:=(pos('}',hs^)=0);
                    hs^:='';
                  end;
            ';' : begin
                    UsesDone:=true;
                    Done:=(UsesDone and InImplementation);
                    hs^:='';
                  end;
           else
            begin
              if (not skip[level]) and (not UsesDone) then
               begin
                 new(curruses);
                 curruses^.Name:=GetName(hs^);
                 curruses^.next:=nil;
                 if currunit^.useslist=nil then
                  currunit^.useslist:=curruses
                 else
                  begin
                    lastuses:=currunit^.useslist;
                    while not (lastuses^.Next=nil) do
                     lastuses:=lastuses^.next;
                    lastuses^.next:=curruses;
                  end;
{$ifndef debug}
                 ListDepend(curruses^.Name);
{$endif}
                 RemoveSep(hs^);
               end
              else
               if (Copy(hs^,1,4)='USES') and ((length(hs^)=4) or (hs^[5] in [' ',#9])) then
                begin
                  Delete(hs^,1,4);
                  UsesDone:=false;
                end
              else
               begin
                 if (hs^='IMPLEMENTATION') then
                  InImplementation:=true
                 else
                  if (Copy(hs^,1,7)='PROGRAM') then
                   currunit^.IsUnit:=false;
                 hs^:='';
               end;
            end;
           end;
         end;
      end;
     Close(f);
   end
  else
   dispose(currunit);
  dispose(hs);
end;


procedure ShowDepend;
var
  currunit : PUnit;
  curruses : PUses;
  t        : text;
  P        : PDefine;
begin
  if CallLine='' then
   begin
     CallLine:='ppc386 ';
     P:=Define;
     While P<>Nil do
      begin
        CallLine:=CallLine+' -d'+P^.Name;
        P:=P^.Next;
      end;
   end;      
  assign(t,OutFile);
  rewrite(t);
  currunit:=UnitList;
  while not (currunit=nil) do
   begin
     if currunit^.IsUnit then
      Write(t,FixFn(currunit^.Name+'.PPU')+': '+currunit^.PasFn)
     else
      Write(t,FixFn(currunit^.Name+exeext)+': '+currunit^.PasFn);
     curruses:=currunit^.useslist;
     while not (curruses=nil) do
      begin
{$ifndef debug}
        if UnitDone(curruses^.name) then
{$endif}
         begin
           writeln(t,' \');
           write(t,#9+FixFn(curruses^.name+'.PPU'));
         end;
        curruses:=curruses^.next;
      end;
     writeln(t,'');
     If AddCall then
      writeln(t,#9,CallLine,' ',currunit^.PasFn);
     writeln(t,'');
     currunit:=currunit^.next;
   end;
  close(t);
end;


procedure getpara;
var
  ch   : char;
  para : string[128];
  i    : word;

  procedure helpscreen;
  begin
    writeln('MKDep [Options] <File>');
    Writeln;
    Writeln('Options can be: -D<define>   Define a symbol');
    Writeln('                -oFile       Write output to file');
    WRiteln('                             (default stdout)');
    Writeln('                -V           Be more verbose');
    Writeln('          -? or -H           This HelpScreen');
    Writeln('                -A[call]     Add compiler calls to makefile');
    halt(1);
  end;

begin
  Define:=Nil;
  Outfile:='';
  AddCall:=False;
  Verbose:=False;
  for i:=1 to paramcount do
   begin
     para:=Paramstr(i);
     if (para[1]='-') then
      begin
        ch:=Upcase(para[2]);
        delete(para,1,2);
        case ch of
         'A' : begin
	         AddCall:=True;
		 CallLine:=Para;
               end;	 
         'D' : AddDefine(para);
         'O' : OutFile:=Para;
         'V' : verbose:=true;
     '?','H' : helpscreen;
        end;
     end
    else
     begin
       ParaFile:=Para;
       if Pos('.',ParaFile)>0 then
        Delete(Parafile,Pos('.',ParaFile),255);
     end;
    end;
  if (ParaFile='') then
   HelpScreen;
end;


begin
  GetPara;
  ListDepend(ParaFile);
  ShowDepend;
end.

{
  $Log: mkdep.pp,v $
  Revision 1.2  1998/03/27 18:39:20  peter
    * fixed the preprocessor
    + -A<call> to add the <call> to all the entries

  Revision 1.1.1.1  1998/03/25 11:18:48  root
  * Restored version

  Revision 1.5  1998/03/15 15:39:23  peter
    * fixed $ELSE

  Revision 1.4  1998/03/10 12:53:24  peter
    * much better preprocessor
    + IFOPT skipping

  Revision 1.3  1998/03/02 13:45:34  peter
    * works better and exefile name is written when a program is parsed

  Revision 1.2  1998/02/24 13:55:07  michael
  + Added option to write to file. Default is to stdout
  + Added option to write/not write a compiler call
  + Nr of defines is now unlimited.
  * Fixed bug with uppercase filenames under linux.

  Revision 1.1  1998/02/23 23:16:48  peter
    + Initial implementation
}
