{
    $Id: msg2inc.pp,v 1.2 1998/03/30 12:06:17 peter Exp $
    This program is part of the Free Pascal run time library.
    Copyright (c) 1998 by Peter Vreman

    Convert a .msg file to an .inc file with a const array of char
    And for the lazy docwriters it can also generate some TeX output
    
    See the file COPYING.FPC, included in this distribution,
    for details about the copyright.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

 **********************************************************************}
program msg2inc;
uses strings;

var
  InFile,
  OutFile,
  OutName    : string;
  TexOption,
  TexError   : boolean;

function ReadFileInBuf(const fn:string;var buf:pchar;var bufcnt:longint):boolean;
var
  bufread,
  bufsize : word;
  f       : file;
begin
  readfileinbuf:=false;
  bufsize:=0;
  assign(f,fn);
  {$I-}
   reset(f,1);
  {$I+}
  if ioresult<>0 then
   begin
     WriteLn('Error: ',fn,' not found');
     exit;
   end;
  bufsize:=filesize(f);         {The language file is <64K}
  getmem(buf,bufsize+1);
  blockread(f,buf^,bufsize,bufread);
  buf[bufsize]:=#10;            {Allways mark an End}
  close(f);
  bufcnt:=bufread;
  readfileinbuf:=true;
end;


procedure ParseToInc(const fn,fn2,constname:string);

  function l0(l:longint):string;
  var
    s : string[16];
  begin
    str(l,s);
    while (length(s)<5) do
     s:='0'+s;
    l0:=s;
  end;

  function createconst(b:byte):string;
  begin
    if (b in [32..127]) and (b<>39) then
     createconst:=''''+chr(b)+''''
    else
     createconst:='#'+chr(b div 100+48)+chr((b mod 100) div 10+48)+chr(b mod 10+48)
  end;

var
  t          : text;
  f          : file;
  cidx,
  bufread    : longint;
  i,j,k,oldj : longint;
  buf,
  p          : pchar;
  s          : string;
begin
  writeln('converting '+fn+' -> '+fn2);
{Read the message file}
  if not ReadFileInBuf(fn,buf,bufread) then
   exit;
{Open textfile}
  assign(t,fn2);
  rewrite(t);
  write(t,'const '+constname+' : array[1..00000] of char=(');
{Parse buffer in msgbuf and create indexs}
  p:=buf;
  i:=0;
  cidx:=0;
  while (i<bufread) do
   begin
     j:=0;
     while (not (p[j] in [#10,#13])) and (j<255) and (i<bufread) do
      begin
        inc(i);
        inc(j);
      end;
     if not (p[0] in [';','#','%']) then
      begin
        oldj:=j;
        if (p[0]='<') and (p[1]='l') and (p[2]='f') and (p[3]='>') then
         j:=0;
        p[j]:=#0;
        for k:=0to j do
         begin
           if cidx mod 15=0 then
            begin
              if cidx>0 then
               writeln(t,',')
              else
               writeln(t,'');
              write(t,'  ');
            end
           else
            write(t,',');
           write(t,createconst(ord(p[k])));
           inc(cidx);
         end;
        j:=oldj;
      end;
     repeat
       inc(i);
       inc(j);
     until not (p[j] in [#10,#13]);
     p:=pchar(@p[j]);
   end;
  writeln(t,');');
  close(t);
{update arraysize}
  s:=l0(cidx);
  assign(f,fn2);
  reset(f,1);
  seek(f,18+length(constname));
  blockwrite(f,s[1],5);
  close(f);
end;


procedure ParseToTex(const fn,fn2:string);
var
  t          : text;
  bufread    : longint;
  i,j,k,oldj : longint;
  buf,
  p          : pchar;
  s1,s2      : string;
begin
  writeln('TeXing '+fn+' -> '+fn2);
{Read the message file}
  if not ReadFileInBuf(fn,buf,bufread) then
   exit;
{Open textfile}
  assign(t,fn2);
  rewrite(t);
{Parse buffer in msgbuf and create indexs}
  p:=buf;
  i:=0;
  while (i<bufread) do
   begin
     {Scan line length}
     j:=0;
     while (not (p[j] in [#10,#13])) and (j<255) and (i<bufread) do
      begin
        inc(i);
        inc(j);
      end;
     if not (p[0] in [';','#']) then
      begin
        oldj:=j;
        if (p[0]='<') and (p[1]='l') and (p[2]='f') and (p[3]='>') then
         j:=0;
        p[j]:=#0;
      {Write to Tex Lines}
        if p[0]='%' then
         begin
           k:=1;
           if p[1]=' ' then
            inc(k);
           writeln(t,pchar(@p[k]))
         end
        else
         begin
           s1:='';
           k:=0;
           while (k<5) and (p[k]<>'_') do
            begin
              case p[k] of
               'W' : s1:='Warning: ';
               'E' : s1:='Error: ';
               'F' : s1:='Fatal Error: ';
               'N' : s1:='Note: ';
               'I' : s1:='Info: ';
               'H' : s1:='Hint: ';
              end;
              inc(k);
            end;
           if p[k]='_' then
            inc(k);
           s2:=StrPas(pchar(@p[k]));
           writeln(t,'\item ['+s1+s2+']');
         end;
      {Goto Next line}
        j:=oldj;
      end;
     {Skip enters}
     repeat
       inc(i);
       inc(j);
     until not (p[j] in [#10,#13]);
     p:=pchar(@p[j]);
   end;
  close(t);
end;

procedure getpara;
var
  ch      : char;
  para    : string;
  files,i : word;

  procedure helpscreen;
  begin
    writeln('usage : msg2inc [Options] <msgfile> <incfile> <constname>');
    writeln('<Options> can be : -TE    Create .doc TeX file (error style)');
    writeln('                   -TO    Create .doc TeX file (options style)');
    writeln('             -? or -H     This HelpScreen');
    halt(1);
  end;

begin
  FIles:=0;
  for i:=1to paramcount do
   begin
     para:=paramstr(i);
     if (para[1]='-') then
      begin
        ch:=upcase(para[2]);
        delete(para,1,2);
        case ch of
         'T' : case upcase(para[1]) of
                'O' : TexOption:=true;
                'E' : TexError:=true;
               end;
     '?','H' : helpscreen;
        end;
     end
    else
     begin
       inc(Files);
       if Files>3 then
        HelpScreen;
       case Files of
        1 : InFile:=Para;
        2 : OutFile:=Para;
        3 : OutName:=Para;
       end;
     end;
   end;
  if (Files<2) or
     ((not (TexOption or TexError)) and (Files<3)) then
   HelpScreen;
end;

begin
  getpara;
  if TexOption or TexError then
   ParseToTex(InFile,OutFile)
  else
   ParseToInc(Infile,OutFile,OutName);
end.
{
  $Log: msg2inc.pp,v $
  Revision 1.2  1998/03/30 12:06:17  peter
    + support for tex output for the lazy docwriter ;)

  Revision 1.1.1.1  1998/03/25 11:18:48  root
  * Restored version

  Revision 1.3  1998/03/11 12:12:14  peter
    + header and log
    * fixed <lf>

}
