{
    $Id: ppumove.pp,v 1.1.1.1 1998/03/25 11:18:47 root Exp $
    This program is part of the Free Pascal run time library.
    Copyright (c) 1997 by Michael Van Canneyt
    member of the Free Pascal development team

    See the file COPYING.FPC, included in this distribution,
    for details about the copyright.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

 **********************************************************************}

{ Program to produce shared libraries from Free Pascal units. }

Program ppumove;

uses

{$ifdef linux}
     linux,
{$else linux}
     dos,
{$endif linux}
     getopts;

Type TUnitHeader = Array [0..19] of char;

     PLinkOEnt = ^TLinkOEnt;
     TLinkOEnt = record
        Name : string;
        Next : PLinkOEnt;
        end;

Const ShortOpts = 'o:e:d:qhsv';
      PPUExt = 'ppu';
      ObjExt = 'o';
      ProgramVersion = '$Revision: 1.1.1.1 $';
      BufSize = 4096;
      uf_in_library = 8;
      uf_shared_library = 16;
      ibend = 255;
      iblibraries = 31;
      iblinkofile = 8;


Var OutputFile,
    DestPath,
    PPLExt,LibExt     : string;
    Quiet,MakeStatic  : boolean;
    I                 : Integer;
    Buffer            : Pointer;
    ObjFiles          : PLinkOEnt;

{$ifdef dos}
Procedure Exec(prog,comline : string);

  begin
     Writeln('Executing ',prog,' ',comline);
     dos.exec(prog,comline);
  end;
{$endif}

Function FileExists (Const F : String) : Boolean;
{
 Returns True if the file exists, False if not.
}
Var
{$ifdef linux}
info : Stat;
{$else}
info : searchrec;
{$endif}

begin
{$ifdef linux}
  FileExists:=FStat (F,Info);
{$else}
  FindFirst (F,anyfile,Info);
  FileExists:=DosError=0;
{$endif}
end;

Function ForceExtension(Const HStr,ext:String):String;
{
  Return a filename which certainly has the extension ext
}
var
  j : longint;
begin
  j:=length(Hstr);
  while (j>0) and (Hstr[j]<>'.') do
   dec(j);
  if j=0 then
   j:=255;
  ForceExtension:=Copy(Hstr,1,j-1)+'.'+Ext;
end;

Function CheckHeader (Var Header : TUnitHeader) : Boolean;
{
  Check if Header is a valid unit header.
}
begin
  If Not ((Header[0]='P') and (Header[1]='P') and (Header[2]='U')) then
    begin
    Writeln (stderr,'Error: File is not a valid unit file.');
    Flush (stderr);
    Exit(False);
    end;
  If (Byte(Header[9]) and uf_in_library)=uf_in_library then
    begin
    Writeln (stderr,'Error: Unit already is in a library.');
    Flush (stderr);
    Exit(False);
    end;
  CheckHeader:=True;
end;

Procedure AddToLinkFiles (S : String);
{
  Adds a filename to a list of object files to link to.
  No duplicates allowed.
}
Var P : PLinKOEnt;


begin
  P:=ObjFiles;
  { Don't add files twice }
  While (P<>nil) and (p^.name<>s) do
    p:=p^.next;
  if p=nil then
    begin
    new(p);
    p^.next:=ObjFiles;
    p^.name:=s;
    ObjFiles:=P;
    end;
end;

function readbyte (var Fin : file): byte;
{
 Read 1 byte from Fin and return it.
}
Var T : Byte;

begin
  BlockRead (Fin,T,1);
  readbyte:=T;
end;

function readlong (var Fin : file) : longint;
{
 Read 1 longint from Fin and return it.
}
var T : longint;

begin
  BlockRead (Fin,T,4);
  readlong:=T;
end;

Function ReadString (var Fin : File) : String;
{
 Read 1 string from Fin and return it.
}
Var Temp : String;

begin
  BlockRead (Fin,Temp,1);
  BlockRead (Fin,Temp[1],byte(Temp[0]));
  ReadString:=Temp;
end;

procedure writebyte (var Fout : file; B : byte);
{
  Write 1 byte to Fout.
}
begin
  BlockWrite (Fout,B,1)
end;

procedure writelong (var Fout : file; L : longint);
{
  Write 1 longint to Fout.
}
begin
  BlockWrite (Fout,L,4)
end;

procedure writeString (var Fout : File; S : String);
{
  Write 1 string to Fout.
}
begin
  BlockWrite (Fout,S,length(S)+1);
end;

Procedure TreatLinkOFiles (Var Fin,Fout : File);
{
  Read the ppufile until the list of files to link to has been read.

  Then read the names of the files that need to be linked in,
  and add them to the list of object files. Don't write them back to the
  ppl file.
}
Var
  b        : byte;
  found    : boolean;
  ourlib,S : string;
begin
  { The header has been read, so now comes the list of source files... }
  repeat
    b:=readbyte(fin);
    writebyte (fout,b);
    if b<>ibend then
      writestring(Fout,Readstring(Fin));
  until b=ibend;
  { Now the list of used units }
  repeat
    b:=readbyte(fin);
    writebyte(fout,b);
    if b<>ibend then
      begin
      writestring(Fout,Readstring(Fin)); { name }
      writelong(fout,readlong(fin));     { crc }
      end;
  until b=ibend;

  {create our libraryname}
  ourlib:=Copy(OutputFile,1,pos('.'+libext,OutputFile)-1);
  if lowercase(Copy(ourlib,1,3))='lib' then
   Delete(ourlib,1,3);


  { Now the list of library files should come ... }
  found:=false;
  b:=readbyte(fin);
  While b=iblibraries do
    begin
      s:=readstring(fin);
      writestring(fout,s);
      if s=ourlib then
       found:=true;
      b:=readbyte(fin);
      writebyte(fout,b);
    end;
  if not found then
   begin
     writebyte(fout,iblibraries);
     writestring(fout,ourlib);
   end;


  { Now the list of link files should come,
    they are removed and also added to the lib }
  While b=iblinkofile do
    begin
    s:=readstring(fin);
    if not Quiet then write (' ',s);
    addtolinkfiles (s);
    b:=readbyte(fin);
    end;
  writebyte(Fout,b);
end;



Procedure usage;
{
 Print usage and exit.
}
begin
  Writeln (paramstr(0),': [-qhvs] [-e ext] [-o name] [-d path] file [file ...]');
  Halt (0);
end;



Procedure processopts;
{
  Process command line opions, and checks if command line options OK.
}
var
  C : char;
begin
  ObjFiles:=Nil;
  if paramcount=0 then usage;
  Quiet:=False;
  OutputFile:='';
  PPLExt:='ppl';
{$ifdef Linux}
  LibExt:='so';
{$else }
  LibExt:='a';
{$endif Linux}
  c:=Getopt (ShortOpts);
  While c<>EndOfOptions do
    begin
    Case C of
     's' : begin
              MakeStatic:=True;
              LibExt:='a';
           end;

     'o' : OutputFile:=ForceExtension(OptArg,LibExt);
     'd' : DestPath:=OptArg;
     'e' : PPLext:=OptArg;
     'q' : Quiet:=True;
     'h' : Usage;
     'v' : writeln ('ppumove version: ',ProgramVersion);
    end;
    c:=getopt(ShortOpts);
    end;
  if (OptInd>ParamCount) then
    begin
    Writeln (stderr,'Error: no input files');
    Halt(1);
    end;
  if (OptInd<ParamCount) and (length(OutputFile)=0) then
    begin
    Writeln (stderr,'Error: when moving multiple units, specify an output name.');
    Halt(1);
    end;
  GetMem (Buffer,Bufsize);
  If Buffer=Nil then
    begin
    Writeln (StdErr,'Error: could not allocate memory for buffer.');
    Halt(1);
    end;
end;

Function DoFile (FileName : String) : Boolean;
{
 Convert one file (in Filename) to library format.
 Return true if successful, false otherwise.

}
Var Header : TUnitHeader;
    Fin, Fout : File;
    Count : LongInt;
begin
 If Not Quiet then

   Write ('Processing : ',ForceExtension(FileName,PPUExt),'...');
 Assign (Fin,ForceExtension(FileName,PPUExt));
 Assign (Fout,ForceExtension(FileName,PPLExt));
 {$i-}
 Reset (Fin,1);
 if IOResult<>0 then

   begin
   Writeln (stderr,'Error: Could not open : ',ForceExtension(FileName,PPUExt));
   Exit(False);
   End;
 Rewrite (Fout,1);
 if IOResult<>0 then

   begin
   Close (Fin);
   Writeln (stderr,'Error: Could not open : ',ForceExtension(FileName,PPLExt));
   Exit(False);
   End;
 BlockRead (Fin,Header,SizeOf(Header),Count);
 If Count<SizeOf(Header) then
   begin
   Close (Fin);
   Close (Fout);
   Writeln (stderr,'Error: Invalid unit file : ',ForceExtension(FileName,PPUExt));
   Exit(False);
   end;
 If Not CheckHeader (Header) then
   begin
     Close (Fin);
     Close (Fout);
     exit(False);
   end;
 { set library flag }
 byte(header[9]):=Byte(Header[9]) or uf_in_library;
 if not MakeStatic then
   byte(header[9]):=Byte(Header[9]) or uf_shared_library;

 BlockWrite (Fout,Header,SizeOf(Header),Count);
 If Count<SizeOf(Header) then

   begin
   Close (Fin);
   Close (Fout);
   Writeln (stderr,'Error: Could not write unit header.');
   Exit(False);
   end;

 { Now, check the object files that are needed...}
 TreatLinkOFiles (Fin,Fout);


 { Just Append the rest of the unit file. }
 Repeat
   BlockRead  (Fin,Buffer^,BufSize,Count);
   BlockWrite (Fout,Buffer^,Count);
 until Count<BufSize;
 Close (Fin);
 Close (Fout);
 If Not Quiet then
   Writeln ('Done.');
 DoFile:=True;
 AddToLinkFiles (ForceExtension(FileName,ObjExt));
end;

Procedure DoLink;
{
  Link the object files together to form a (shared) library.
}
Var I : longint;
    Names : String;
    Error : Longint;
    F     : File;
    P     : PLinkOEnt;


begin
  if not Quiet then
    Write ('Linking : ');
  i:=OptInd;
  P:=ObjFiles;
  names:='';
  While p<>nil do
    begin
      Names:=Names+' '+P^.name;
      p:=p^.next;
    end;
 If not Quiet then

   write (names,'... ');
{$ifdef linux}
  if Copy(OutputFile,1,3)<>'lib' then
   OutputFile:='lib'+OutputFile;
  If MakeStatic then
    begin
    shell ('ar r '+outputfile+' '+names);
    if linuxerror=0 then

      shell ('ranlib '+outputfile);
    end
  else
    shell ('ld -shared -o '+OutputFile+' '+names);
  Error:=LinuxError;
{$else}
{  exec ('ld.exe','-shared -o '+OutputFile+' '+names);}
{ lets try to make a static library }
  exec(fsearch('ar.exe',getenv('PATH')),' r '+Outputfile+'  '+names);
  Error:=DosError;
  if error<>0 then
    writeln (Stderr,'Error: ar not run doserror = ',error)
  else
    begin
       error:=DosExitCode;
       if error<>0 then
         writeln (Stderr,'Error: ar failed exitcode = ',error)
       else
         begin
            exec(fsearch('ranlib.exe',getenv('PATH')),Outputfile);
            Error:=DosError;
            if error<>0 then
              writeln (Stderr,'Error: ranlib not run, doserror = ',error)
            else
              begin
                 error:=DosExitCode;
                 if error<>0 then
                   writeln (Stderr,'Error: ranlib failed exitcode = ',error);
              end;
         end;
    end;
{$endif}
  If Error<>0 then

    begin
    Writeln (Stderr,'Error: Library building stage failed.');
    { we should delete the PPL files, no ?}
    P:=ObjFiles;
    names:='';
    While p<>nil do
      begin
        Names:=Names+' '+ForceExtension(P^.name,'ppl');
        p:=p^.next;
      end;
{$ifndef Linux}
    exec(fsearch('rm.exe',getenv('PATH')),names);
{$else }
    shell('rm '+names);
{$endif }
    exit;
    end;
  if DestPath<>'' then
   begin
     Assign(F, OutputFile);
     Rename(F,DestPath+'/'+OutputFile);
   end;

  if Not Quiet then

    Writeln('Done.');
end;

begin
  ProcessOpts;
  i:=OptInd;
  if OutputFile='' then
   OutPutFile:=ForceExtension(Paramstr(OptInd),LibExt);
  While (i<=ParamCount) and Dofile(Paramstr(i)) do Inc(i);
  DoLink;
end.

{
  $Log: ppumove.pp,v $
  Revision 1.1.1.1  1998/03/25 11:18:47  root
  * Restored version

  Revision 1.9  1998/02/24 14:22:42  peter
    * support for new iblibraries and more ppu flags

  Revision 1.8  1998/01/26 12:02:55  michael
  + Added log at the end




  Working file: rtl/utils/ppumove.pp
  description:
  ----------------------------
  revision 1.7
  date: 1998/01/23 17:59:33;  author: michael;  state: Exp;  lines: +57 -12
  + Implemented Pierres changes for dos
  ----------------------------
  revision 1.6
  date: 1998/01/20 13:10:53;  author: michael;  state: Exp;  lines: +179 -22
  + Implemented static libs. Usable under DOS too.
  ----------------------------
  revision 1.5
  date: 1998/01/19 15:22:16;  author: peter;  state: Exp;  lines: +13 -5
  option -d to add a destination path where the lib must be moved into
  ----------------------------
  revision 1.4
  date: 1998/01/17 01:52:41;  author: michael;  state: Exp;  lines: +14 -3
  + Added check to see if unit already is in a library.
  ----------------------------
  revision 1.3
  date: 1998/01/15 00:21:19;  author: michael;  state: Exp;  lines: +63 -61
  * Applied fixes for bug (From Peter Vreman)
  + Implementation of ForceFileExtension (From Peter Vreman)
  + VersionString prints RCS revision number.
  + Added comments for every function.
  ----------------------------
  revision 1.2
  date: 1998/01/14 14:14:29;  author: michael;  state: Exp;  lines: +28 -10
  * fixed FindFirst call and made buffer of variable length. Added commandline option for that.
  ----------------------------
  revision 1.1
  date: 1998/01/13 23:54:06;  author: michael;  state: Exp;
  Initial versions under CVS
  =============================================================================
}
